@@ -1,5 +1,5 @@
 //
-// Copyright (C) 2005-2020 Claes Nästén <pekdon@gmail.com>
+// Copyright (C) 2005-2023 Claes Nästén <pekdon@gmail.com>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -8,7 +8,7 @@
 #include "CfgParser.hh"
 #include "Debug.hh"
 #include "Compat.hh"
-#include "Util.hh"
+#include "String.hh"
 
 #include <algorithm>
 #include <cstring>
@@ -53,8 +53,8 @@ TimeFiles::requireReload(const std::string &file)
 
 //! @brief CfgParser::Entry constructor.
 CfgParser::Entry::Entry(const std::string &source_name, int line,
-                        const std::string &name, const std::string &value,
-                        CfgParser::Entry *section)
+			const std::string &name, const std::string &value,
+			CfgParser::Entry *section)
 	: _section(section),
 	  _name(name),
 	  _value(value),
@@ -122,8 +122,9 @@ CfgParser::Entry::addEntry(CfgParser::Entry *entry, bool overwrite)
 	CfgParser::Entry *entry_search = 0;
 	if (overwrite) {
 		if (entry->getSection()) {
-			entry_search = findEntry(entry->getName(), true,
-						 entry->getSection()->getValue().c_str());
+			const char* name =
+				entry->getSection()->getValue().c_str();
+			entry_search = findEntry(entry->getName(), true, name);
 		} else {
 			entry_search = findEntry(entry->getName(), false);
 		}
@@ -134,8 +135,8 @@ CfgParser::Entry::addEntry(CfgParser::Entry *entry, bool overwrite)
 	// when the value is the same.
 	if (entry_search
 	    && (! entry_search->getSection()
-		|| strcasecmp(entry->getValue().c_str(),
-			      entry_search->getValue().c_str()) == 0)) {
+		|| pekwm::ascii_ncase_equal(entry->getValue(),
+					    entry_search->getValue()))) {
 		entry_search->_value = entry->getValue();
 		entry_search->setSection(entry->getSection(), overwrite);
 
@@ -153,8 +154,8 @@ CfgParser::Entry::addEntry(CfgParser::Entry *entry, bool overwrite)
 //! @brief Adds Entry to the end of Entry list at current depth.
 CfgParser::Entry*
 CfgParser::Entry::addEntry(const std::string &source_name, int line,
-                           const std::string &name, const std::string &value,
-                           CfgParser::Entry *section, bool overwrite)
+			   const std::string &name, const std::string &value,
+			   CfgParser::Entry *section, bool overwrite)
 {
 	return addEntry(new Entry(source_name, line, name, value, section),
 			overwrite);
@@ -185,16 +186,16 @@ CfgParser::Entry::setSection(CfgParser::Entry *section, bool overwrite)
 //! @param name Name of Entry to look for.
 CfgParser::Entry*
 CfgParser::Entry::findEntry(const std::string &name, bool include_sections,
-                            const char *value) const
+			    const char *value) const
 {
-	CfgParser::Entry *value_check;
-	entry_cit it = begin();
-	for (; it != end(); ++it) {
-		value_check = include_sections ? (*it)->getSection() : *it;
+	for (entry_cit it = begin(); it != end(); ++it) {
+		CfgParser::Entry *value_check =
+			include_sections ? (*it)->getSection() : *it;
 
 		if (*(*it) == name.c_str()
 		    && (! (*it)->getSection() || include_sections)
-		    && (! value || (value_check && value_check->getValue() == value))) {
+		    && (! value || (value_check
+				    && value_check->getValue() == value))) {
 			return *it;
 		}
 	}
@@ -222,13 +223,12 @@ CfgParser::Entry::findSection(const std::string &name, const char *value) const
 //! @brief Sets and validates data specified by key list.
 void
 CfgParser::Entry::parseKeyValues(std::vector<CfgParserKey*>::const_iterator it,
-                                 std::vector<CfgParserKey*>::const_iterator end)
+				 std::vector<CfgParserKey*>::const_iterator
+				     end)
 	const
 {
-	CfgParser::Entry *value;
-
 	for (; it != end; ++it) {
-		value = findEntry((*it)->getName());
+		CfgParser::Entry *value = findEntry((*it)->getName());
 		if (value) {
 			try {
 				(*it)->parseValue(value->getValue());
@@ -288,8 +288,9 @@ CfgParser::Entry::copyTreeInto(CfgParser::Entry *from, bool overwrite)
 		if ((*it)->getSection()) {
 			entry_section = new Entry(*((*it)->getSection()));
 		}
-		addEntry((*it)->getSourceName(), (*it)->getLine(), (*it)->getName(),
-			 (*it)->getValue(), entry_section, true);
+		addEntry((*it)->getSourceName(), (*it)->getLine(),
+			 (*it)->getName(), (*it)->getValue(),
+			 entry_section, true);
 	}
 }
 
@@ -302,11 +303,35 @@ operator<<(std::ostream &stream, const CfgParser::Entry &entry)
 	return stream;
 }
 
-//! @brief CfgParser constructor.
-CfgParser::CfgParser(void)
-	: _source(0), _root_entry(0), _is_dynamic_content(false),
-	  _section(_root_entry), _overwrite(false)
+CfgParser::CfgParser(const CfgParserOpt &opt)
+	: _source(nullptr),
+	  _opt(opt),
+	  _root_entry(nullptr),
+	  _is_dynamic_content(false),
+	  _section(_root_entry),
+	  _overwrite(false)
 {
+	CfgParserVarExpanderType types[] = {
+		CFG_PARSER_VAR_EXPANDER_OS_ENV,
+		CFG_PARSER_VAR_EXPANDER_X11_ATOM,
+		CFG_PARSER_VAR_EXPANDER_X11_RES
+	};
+	for (int i = 0; i < sizeof(types)/sizeof(types[0]); i++) {
+		CfgParserVarExpander* exp =
+			mkCfgParserVarExpander(types[i],
+					       _opt.registerXResource());
+		if (exp != nullptr) {
+			_var_expanders.push_back(exp);
+		}
+	}
+
+	// memory expander is put last as it does not require a
+	// specific prefix to lookup and would lookup all vars if
+	// placed first.
+	_var_expander_mem = mkCfgParserVarExpander(CFG_PARSER_VAR_EXPANDER_MEM,
+						   _opt.registerXResource());
+	_var_expanders.push_back(_var_expander_mem);
+
 	_root_entry = new CfgParser::Entry(_root_source_name, 0, "ROOT", "");
 	_section = _root_entry;
 }
@@ -315,6 +340,11 @@ CfgParser::CfgParser(void)
 CfgParser::~CfgParser(void)
 {
 	clear(false);
+
+	std::vector<CfgParserVarExpander*>::iterator it(_var_expanders.begin());
+	for (; it != _var_expanders.end(); ++it) {
+		delete *it;
+	}
 }
 
 /**
@@ -331,7 +361,8 @@ CfgParser::clear(bool realloc)
 	delete _root_entry;
 
 	if (realloc) {
-		_root_entry = new CfgParser::Entry(_root_source_name, 0, "ROOT", "");
+		_root_entry = new CfgParser::Entry(_root_source_name, 0,
+						   "ROOT", "");
 	} else {
 		_root_entry = 0;
 	}
@@ -344,7 +375,7 @@ CfgParser::clear(bool realloc)
 	_source_names.clear();
 	_source_name_set.clear();
 	_sections.clear();
-	_var_map.clear();
+	_var_expander_mem->clear();
 
 	// Remove sections
 	section_map_it it = _section_map.begin();
@@ -363,13 +394,13 @@ CfgParser::clear(bool realloc)
  */
 bool
 CfgParser::parse(const std::string &src, CfgParserSource::Type type,
-                 bool overwrite)
+		 bool overwrite)
 {
 	// Set overwrite
 	_overwrite = overwrite;
 
 	// Init parse buffer and reserve memory.
-	std::string buf, value;
+	std::string buf;
 	buf.reserve(PARSE_BUF_SIZE);
 
 	// Open initial source.
@@ -407,24 +438,27 @@ CfgParser::parse()
 	std::string buf, value;
 	buf.reserve(PARSE_BUF_SIZE);
 
-	int c, next;
 	while (_sources.size()) {
 		_source = _sources.back();
 		if (_source->isDynamic()) {
 			_is_dynamic_content = true;
 		}
 
+		int c;
 		while ((c = _source->get_char()) != EOF) {
 			switch (c) {
-			case '\n':
-				// To be able to handle entry ends AND { after \n a check
-				// to see what comes after the newline is done. If { appears
-				// we continue as nothing happened else we finish the entry.
-				next = parseSkipBlank(_source);
+			case '\n': {
+				// To be able to handle entry ends AND { after
+				// \n a check to see what comes after the
+				// newline is done. If { appears we continue as
+				// nothing happened else we finish the entry.
+				int next = parseSkipBlank(_source);
 				if (next != '{') {
-					parseEntryFinish(buf, value, have_value);
+					parseEntryFinish(buf, value,
+							 have_value);
 				}
 				break;
+			}
 			case ';':
 				parseEntryFinish(buf, value, have_value);
 				break;
@@ -432,7 +466,8 @@ CfgParser::parse()
 				if (parseName(buf)) {
 					parseSectionFinish(buf, value);
 				} else {
-					USER_WARN("Ignoring section as name is empty.");
+					USER_WARN("Ignoring section as name "
+						  "is empty.");
 				}
 				buf = "";
 				value = "";
@@ -441,7 +476,8 @@ CfgParser::parse()
 			case '}':
 				if (_sections.size() > 0) {
 					if (buf.size() && parseName(buf)) {
-						parseEntryFinish(buf, value, have_value);
+						parseEntryFinish(buf, value,
+								 have_value);
 						buf = "";
 						value = "";
 						have_value = false;
@@ -449,7 +485,8 @@ CfgParser::parse()
 					_section = _sections.back();
 					_sections.pop_back();
 				} else {
-					USER_WARN("Extra } character found, ignoring.");
+					USER_WARN("Extra } character found, "
+						  "ignoring.");
 				}
 				break;
 			case '=':
@@ -459,8 +496,8 @@ CfgParser::parse()
 			case '#':
 				parseCommentLine(_source);
 				break;
-			case '/':
-				next = _source->get_char();
+			case '/': {
+				int next = _source->get_char();
 				if (next == '/') {
 					parseCommentLine(_source);
 				} else if (next == '*') {
@@ -470,6 +507,7 @@ CfgParser::parse()
 					_source->unget_char(next);
 				}
 				break;
+			}
 			default:
 				buf += c;
 				break;
@@ -481,17 +519,20 @@ CfgParser::parse()
 		} catch (std::string &ex) {
 			P_LOG("Exception: " << ex);
 		}
-		delete _source;
+
+		// source might change in parseEntryFinish
+		CfgParserSource* source = _source;
 		_sources.pop_back();
 		_source_names.pop_back();
 
 		// done inside of the loop to ensure COMMAND and INCLUDE
 		// statements without a new line at the end of the file will
-		// be used.
+		// be used before deleting the source.
 		if (buf.size()) {
 			parseEntryFinish(buf, value, have_value);
 		}
 
+		delete source;
 	}
 
 	return true;
@@ -500,22 +541,24 @@ CfgParser::parse()
 //! @brief Creates and opens new CfgParserSource.
 void
 CfgParser::parseSourceNew(const std::string &name_orig,
-                          CfgParserSource::Type type)
+			  CfgParserSource::Type type)
 {
 	int done = 0;
 	std::string name(name_orig);
 
 	do {
 		CfgParserSource *source = sourceNew(name, type);
-		P_ERR_IF(!source, "source == 0");
+		if (! source) {
+			P_ERR("source == 0: " << name);
+			return;
+		}
 
 		// Open and set as active, delete if fails.
 		try {
 			source->open();
-			time_t time;
 			// Add source to file list if file
 			if (type == CfgParserSource::SOURCE_FILE) {
-				time = Util::getMtime(name);
+				time_t time = Util::getMtime(name);
 				_cfg_files.files.push_back(name);
 				if (_cfg_files.mtime < time) {
 					_cfg_files.mtime = time;
@@ -536,11 +579,13 @@ CfgParser::parseSourceNew(const std::string &name_orig,
 				USER_WARN(ex);
 			}
 
-			// If the open fails and we are trying to open a file, try
-			// to open the file from the current files directory.
+			// If the open fails and we are trying to open a file,
+			// try to open the file from the current files
+			// directory.
 			if (! done && (type == CfgParserSource::SOURCE_FILE)) {
 				if (_source_names.size() && name[0] != '/') {
-					name = Util::getDir(_source_names.back());
+					name = Util::getDir(
+							_source_names.back());
 					name += "/" + name_orig;
 				}
 			}
@@ -566,7 +611,8 @@ CfgParser::parseName(std::string &buf)
 
 	// Check if there is any garbage after the value.
 	if (end != std::string::npos) {
-		if (buf.find_first_not_of(CP_PARSE_BLANKS, end) != std::string::npos) {
+		if (buf.find_first_not_of(CP_PARSE_BLANKS, end)
+		    != std::string::npos) {
 			// Pass, do notihng
 		}
 	}
@@ -616,7 +662,7 @@ CfgParser::parseValue(std::string &value)
  */
 void
 CfgParser::parseEntryFinish(std::string &buf, std::string &value,
-                            bool &have_value)
+			    bool &have_value)
 {
 	if (have_value) {
 		parseEntryFinishStandard(buf, value, have_value);
@@ -633,20 +679,24 @@ CfgParser::parseEntryFinish(std::string &buf, std::string &value,
  */
 void
 CfgParser::parseEntryFinishStandard(std::string &buf, std::string &value,
-                                    bool &have_value)
+				    bool &have_value)
 {
 	if (parseName(buf)) {
 		if (buf[0] == '$') {
-			variableDefine(buf, value);
+			variableDefine(buf.substr(1), value);
 		} else  {
 			variableExpand(value);
 
 			if (buf == "INCLUDE")  {
-				parseSourceNew(value, CfgParserSource::SOURCE_FILE);
+				parseSourceNew(
+					value, CfgParserSource::SOURCE_FILE);
 			} else if (buf == "COMMAND") {
-				parseSourceNew(value, CfgParserSource::SOURCE_COMMAND);
+				parseSourceNew(
+					value,
+					CfgParserSource::SOURCE_COMMAND);
 			} else {
-				_section->addEntry(_source->getName(), _source->getLine(),
+				_section->addEntry(_source->getName(),
+						   _source->getLine(),
 						   buf, value, 0, _overwrite);
 			}
 		}
@@ -676,11 +726,11 @@ CfgParser::parseEntryFinishTemplate(std::string &name)
 
 //! @brief Creates new Section on {
 void
-CfgParser::parseSectionFinish(std::string &buf, std::string &value)
+CfgParser::parseSectionFinish(const std::string &buf, std::string &value)
 {
 	// Create Entry representing Section
 	Entry *section = 0;
-	if (buf.size() == 6 && strcasecmp(buf.c_str(), "DEFINE") == 0) {
+	if (buf.size() == 6 && pekwm::ascii_ncase_equal(buf, "DEFINE")) {
 		// Look for define section, started with Define = "Name" {
 		section_map_it it = _section_map.find(value);
 		if (it != _section_map.end()) {
@@ -688,17 +738,23 @@ CfgParser::parseSectionFinish(std::string &buf, std::string &value)
 			_section_map.erase(it);
 		}
 
-		section = new Entry(_source->getName(), _source->getLine(), buf, value);
+		section = new Entry(_source->getName(), _source->getLine(),
+				    buf, value);
 		_section_map[value] = section;
 	} else {
+		// Expand value in Section = "$VAR" { }
+		variableExpand(value);
+
 		// Create Entry for sub-section.
-		section = new Entry(_source->getName(), _source->getLine(), buf, value);
+		section = new Entry(_source->getName(), _source->getLine(),
+				    buf, value);
 
 		// Add parent section, get section from parent section as it
 		// can be different from the newly created if it is not
 		// overwritten.
 		CfgParser::Entry *parent =
-			_section->addEntry(_source->getName(), _source->getLine(),
+			_section->addEntry(_source->getName(),
+					   _source->getLine(),
 					   buf, value, section, _overwrite);
 		section = parent->getSection();
 	}
@@ -764,10 +820,13 @@ CfgParser::sourceNew(const std::string &name, CfgParserSource::Type type)
 	_source_name_set.insert(name);
 	switch (type) {
 	case CfgParserSource::SOURCE_FILE:
-		source = new CfgParserSourceFile(*_source_name_set.find(name));
+		source = new CfgParserSourceFile(
+				*_source_name_set.find(name));
 		break;
 	case CfgParserSource::SOURCE_COMMAND:
-		source = new CfgParserSourceCommand(*_source_name_set.find(name));
+		source = new CfgParserSourceCommand(
+				*_source_name_set.find(name),
+				_opt.commandPath());
 		break;
 	default:
 		break;
@@ -780,72 +839,94 @@ CfgParser::sourceNew(const std::string &name, CfgParserSource::Type type)
 void
 CfgParser::variableDefine(const std::string &name, const std::string &value)
 {
-	_var_map[name] = value;
-
-	// If the variable begins with $_ it should update the environment aswell.
-	if ((name.size() > 2) && (name[1] == '_')) {
-		setenv(name.c_str() + 2, value.c_str(), 1);
-	}
+	_var_expander_mem->define(name, value);
 }
 
 //! @brief Expands all $ variables in a string.
 void
-CfgParser::variableExpand(std::string &var)
+CfgParser::variableExpand(std::string& line)
 {
 	bool did_expand;
-
 	do {
 		did_expand = false;
 
+		std::string var;
 		std::string::size_type begin = 0, end = 0;
-		while ((begin = var.find_first_of('$', end)) != std::string::npos) {
-			end = begin + 1;
-
-			// Skip escaped \$
-			if ((begin > 0) && (var[begin - 1] == '\\')) {
-				continue;
-			}
-
-			// Find end of variable
-			for (; end != var.size(); ++end) {
-				if ((isalnum(var[end]) == 0) && (var[end] != '_'))  {
-					break;
-				}
-			}
-
-			did_expand = variableExpandName(var, begin, end) || did_expand;
+		for (; parseVarName(line, begin, end, var); var = "") {
+			did_expand = variableExpandName(line, begin, end, var)
+				|| did_expand;
+			begin = end;
 		}
 	} while (did_expand);
 }
 
 bool
-CfgParser::variableExpandName(std::string &var,
-                              std::string::size_type begin,
-                              std::string::size_type &end)
+CfgParser::parseVarName(const std::string& line,
+			std::string::size_type& begin,
+			std::string::size_type& end,
+			std::string& name)
+{
+	bool in_escape = false, in_var = false, in_curly = false;
+	for (std::string::size_type pos = begin; pos < line.size(); pos++) {
+		char c = line[pos];
+		if (in_escape) {
+			if (in_var) {
+				name += c;
+			}
+			in_escape = false;
+		} else if (c == '\\') {
+			in_escape = true;
+		} else if (in_var) {
+			if (pos == (begin + 1) && c == '{') {
+				in_curly = true;
+			} else if (in_curly) {
+				if (c == '}') {
+					end = pos + 1;
+					return true;
+				}
+				name += c;
+			} else if (isalnum(c) || c == '_'
+				   || (pos == (begin + 1)
+				       && (c == '@' || c == '&'))) {
+				name += c;
+			} else {
+				end = pos;
+				return true;
+			}
+		} else if (line[pos] == '$') {
+			in_var = true;
+			begin = pos;
+		}
+	}
+
+	// variable to end of line, not supported in {} vars at the end }
+	// is required and end of line while in escape also treated as
+	// error.
+	if (in_var && (! in_curly && ! in_escape)) {
+		end = line.size();
+		return true;
+	}
+
+	// remove buffered data
+	name = "";
+
+	return false;
+}
+
+bool
+CfgParser::variableExpandName(std::string& line,
+			      const std::string::size_type begin,
+			      std::string::size_type &end,
+			      const std::string& var)
 {
 	bool did_expand = false;
-	std::string var_name(var.substr(begin, end - begin));
 
-	// If the variable starts with _ it is considered an environment
-	// variable, use getenv to see if it is available
-	if (var_name.size() > 2 && var_name[1] == '_') {
-		char *value = getenv(var_name.c_str() + 2);
-		if (value) {
-			var.replace(begin, end - begin, value);
-			end = begin + strlen(value);
-			did_expand = true;
-		} else {
-			USER_WARN("Trying to use undefined environment variable: "
-				  << var_name);
-		}
-	} else {
-		var_map_it it = _var_map.find(var_name);
-		if (it != _var_map.end()) {
-			var.replace(begin, end - begin, it->second);
-			end = begin + it->second.size();
-			did_expand = true;
-		} else  {
-			USER_WARN("Trying to use undefined variable: " << var_name);
+	std::vector<CfgParserVarExpander*>::iterator it(_var_expanders.begin());
+	for (; ! did_expand && it != _var_expanders.end(); ++it) {
+		std::string value;
+		if ((did_expand = (*it)->lookup(var, value))) {
+			line.replace(begin, end - begin, value);
+			end = begin + value.size();
 		}
 	}
 
