@@ -1,6 +1,6 @@
 //
 // Client.cc for pekwm
-// Copyright (C) 2002-2020 Claes Nästén <pekdon@gmail.com>
+// Copyright (C) 2002-2023 Claes Nästén <pekdon@gmail.com>
 //
 // client.cc for aewm++
 // Copyright (C) 2000 Frank Hale <frankhale@yahoo.com>
@@ -24,7 +24,7 @@ extern "C" {
 }
 
 #include "Charset.hh"
-#include "Compat.hh" // setenv, unsetenv
+#include "Compat.hh"
 #include "Debug.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh" // PDecor::TitleItem
@@ -105,8 +105,8 @@ Client::Client(Window new_client, ClientInitConfig &initConfig, bool is_new)
 	// avoid auto-grouping to be to greedy.
 	getTransientForHint();
 
-	AutoProperty *ap =
-		readAutoprops(pekwm::isStarting() ? APPLY_ON_START : APPLY_ON_NEW);
+	AutoProperty *ap = readAutoprops(pekwm::isStarting()
+			? APPLY_ON_START : APPLY_ON_NEW);
 	readHints();
 
 	// We need to set the state before acquiring a frame,
@@ -163,7 +163,8 @@ Client::~Client(void)
 
 	// Focus the parent if we had focus before
 	if (focus && _transient_for) {
-		Frame *trans_frame = static_cast<Frame*>(_transient_for->getParent());
+		Frame *trans_frame =
+			static_cast<Frame*>(_transient_for->getParent());
 		if (trans_frame->getActiveChild() == _transient_for) {
 			trans_frame->giveInputFocus();
 		}
@@ -195,7 +196,8 @@ Client::~Client(void)
 
 	X11::ungrabServer(true);
 
-	P_TRACE(this << " client for window " << FMT_HEX(_window) << " destructed");
+	P_TRACE(this << " client for window " << FMT_HEX(_window)
+		<< " destructed");
 }
 
 /**
@@ -243,10 +245,10 @@ Client::findOrCreateFrame(AutoProperty *autoproperty)
 	bool parent_is_new = false;
 
 	if (! _parent) {
-		findTaggedFrame();
-	}
-	if (! _parent) {
-		findPreviousFrame();
+		_parent = findTaggedFrame();
+		if (! _parent) {
+			_parent = findPreviousFrame();
+		}
 	}
 
 	// Apply Autoproperties again to override EWMH state. It's done twice as
@@ -270,17 +272,16 @@ Client::findOrCreateFrame(AutoProperty *autoproperty)
 /**
  * Find from client from for the currently tagged client.
  */
-bool
+PWinObj*
 Client::findTaggedFrame(void)
 {
 	if (pekwm::isStarting()) {
-		return false;
+		return nullptr;
 	}
 
 	// Check for tagged frame
 	Frame *frame = Frame::getTagFrame();
 	if (frame && frame->isMapped()) {
-		_parent = frame;
 		frame->addChild(this);
 
 		if (! Frame::getTagBehind()) {
@@ -288,32 +289,32 @@ Client::findTaggedFrame(void)
 		}
 	}
 
-	return _parent != 0;
+	return frame;
 }
 
 /**
  * Find frame for client on PEKWM_FRAME_ID hint.
  */
-bool
+PWinObj*
 Client::findPreviousFrame(void)
 {
 	if (! pekwm::isStarting()) {
-		return false;
+		return nullptr;
 	}
 
 	Cardinal id;
 	if (X11::getCardinal(_window, PEKWM_FRAME_ID, id)) {
-		_parent = Frame::findFrameFromID(id);
-		if (_parent) {
-			Frame *frame = static_cast<Frame*>(_parent);
+		Frame *frame = Frame::findFrameFromID(id);
+		if (frame) {
 			frame->addChildOrdered(this);
 			if (getPekwmFrameActive()) {
 				frame->activateChild(this);
 			}
 		}
+		return frame;
 	}
 
-	return _parent != 0;
+	return nullptr;
 }
 
 /**
@@ -367,7 +368,7 @@ Client::setInitialState(void)
  */
 void
 Client::setClientInitConfig(ClientInitConfig &initConfig, bool is_new,
-                            AutoProperty *prop)
+			    AutoProperty *prop)
 {
 	initConfig.map = (! _iconified && _parent->isMapped());
 	initConfig.focus = false;
@@ -508,7 +509,8 @@ Client::resize(uint width, uint height)
 void
 Client::moveResize(int x, int y, uint width, uint height)
 {
-	bool request = ((_gm.x != x) || (_gm.y != y) || (_gm.width != width) || (_gm.height != height));
+	bool request = ((_gm.x != x) || (_gm.y != y)
+			|| (_gm.width != width) || (_gm.height != height));
 
 	_gm.x = x;
 	_gm.y = y;
@@ -529,12 +531,8 @@ Client::setWorkspace(uint workspace)
 			workspace = Workspaces::size() - 1;
 		}
 		_workspace = workspace;
-
-		if (_sticky) {
-			X11::setCardinal(_window, NET_WM_DESKTOP, NET_WM_STICKY_WINDOW);
-		} else {
-			X11::setCardinal(_window, NET_WM_DESKTOP, _workspace);
-		}
+		Cardinal ws_card = _sticky ? NET_WM_STICKY_WINDOW : _workspace;
+		X11::setCardinal(_window, NET_WM_DESKTOP, ws_card);
 	}
 }
 
@@ -563,8 +561,8 @@ Client::reparent(PWinObj *parent, int x, int y)
 	PWinObj::reparent(parent, x, y);
 	_gm.x = parent->getX() + x;
 	_gm.y = parent->getY() + y;
-	X11::selectInput(_window,
-			 PropertyChangeMask|StructureNotifyMask|FocusChangeMask);
+	X11::selectInput(_window, PropertyChangeMask|StructureNotifyMask|
+				  FocusChangeMask);
 }
 
 ActionEvent*
@@ -595,8 +593,9 @@ Client::handleUnmapEvent(XUnmapEvent *ev)
 	X11::unsetProperty(_window, STATE);
 
 	// Extended Window Manager Hints 1.3 specifies that a window manager
-	// should remove the _NET_WM_DESKTOP property when a window is withdrawn.
-	// (to allow legacy applications to reuse a withdrawn window)
+	// should remove the _NET_WM_DESKTOP property when a window is
+	// withdrawn. (to allow legacy applications to reuse a withdrawn
+	// window)
 	X11::unsetProperty(_window, NET_WM_DESKTOP);
 
 	// FIXME: Listen mask should change as this doesn't work?
@@ -682,22 +681,6 @@ Client::findClientFromWindow(Window win)
 	return nullptr;
 }
 
-//! @brief Finds Client with equal ClassHint.
-//! @param class_hint ClassHint to search for.
-//! @return Client if found, else 0.
-Client*
-Client::findClientFromHint(const ClassHint *class_hint)
-{
-	client_it it = _clients.begin();
-	for (; it != _clients.end(); ++it) {
-		if (*class_hint == *(*it)->getClassHint()) {
-			return *it;
-		}
-	}
-
-	return nullptr;
-}
-
 //! @brief Finds Client with id.
 //! @param id ID to search for.
 //! @return Client if found, else 0.
@@ -740,7 +723,8 @@ Client::mapOrUnmapTransients(Window win, bool hide)
 
 	client_it it = client_list.begin();
 	for (; it != client_list.end(); ++it) {
-		if (static_cast<Frame*>((*it)->getParent())->getActiveChild() == *it) {
+		PWinObj* parent = (*it)->getParent();
+		if (static_cast<Frame*>(parent)->getActiveChild() == *it) {
 			if (hide) {
 				(*it)->getParent()->iconify();
 			} else {
@@ -759,8 +743,8 @@ Client::validate(void)
 	X11::sync(False);
 
 	XEvent ev;
-	if (XCheckTypedWindowEvent(X11::getDpy(), _window, DestroyNotify, &ev)
-	    || XCheckTypedWindowEvent(X11::getDpy(), _window, UnmapNotify, &ev)) {
+	if (X11::checkTypedWindowEvent(_window, DestroyNotify, &ev)
+	    || X11::checkTypedWindowEvent(_window, UnmapNotify, &ev)) {
 		XPutBackEvent(X11::getDpy(), &ev);
 		return false;
 	}
@@ -778,30 +762,37 @@ Client::isViewable(void)
 	return (attr.map_state == IsViewable);
 }
 
-//! @brief Grabs all the mouse button actions on the client.
+/**
+ * Grab all mouse button actions on the client excluding the events
+ * without a modifier.
+ */
 void
 Client::grabButtons(void)
 {
-	// Make sure we don't have any buttons grabbed.
+	// make sure all previous grabs are removed
 	X11::ungrabButton(AnyButton, AnyModifier, _window);
 
+	Config *cfg = pekwm::config();
 	std::vector<ActionEvent> *actions =
-		pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
+		cfg->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
 	std::vector<ActionEvent>::iterator it = actions->begin();
+
+	const uint mask = ButtonPressMask;
 	for (; it != actions->end(); ++it) {
-		if ((it->type == MOUSE_EVENT_PRESS)
-		    || (it->type == MOUSE_EVENT_RELEASE)) {
-			// No need to grab mod less events, replied with the frame
-			if ((it->mod == 0) || (it->mod == MOD_ANY)) {
+		if (it->isButtonEvent()) {
+			// not grabbing actions without modifier, will be
+			// grabbed by the Frame code and replied from there
+			if (it->isAnyModifier()) {
 				continue;
 			}
 
-			uint mask = ButtonPressMask|ButtonReleaseMask;
-			X11Util::grabButton(it->sym, it->mod, mask, _window);
+			X11Util::grabButton(it->sym, it->mod, mask, _window,
+					    GrabModeAsync);
 		} else if (it->type == MOUSE_EVENT_MOTION) {
-			// FIXME: Add support for MOD_ANY
-			uint mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
-			X11Util::grabButton(it->sym, it->mod, mask, _window);
+			const uint motion_mask = mask|ButtonMotionMask;
+			X11Util::grabButton(it->sym, it->mod,
+					    motion_mask, _window,
+					    GrabModeAsync);
 		}
 	}
 }
@@ -955,9 +946,10 @@ Client::readMwmHints(void)
 			if (! (mwm_hints.decorations & MWM_DECOR_BORDER)) {
 				setBorder(false);
 			}
-			// Do not handle HANDLE, MENU, ICONFIY or MAXIMIZE. Maybe
-			// one should set the allowed actions for the client based
-			// on this but that might be annoying so ignoring these.
+			// Do not handle HANDLE, MENU, ICONFIY or MAXIMIZE.
+			// Maybe one should set the allowed actions for the
+			// client based on this but that might be annoying so
+			// ignoring these.
 		}
 	}
 }
@@ -1020,13 +1012,15 @@ Client::readAutoprops(ApplyOn type)
 {
 	AutoProperties *aps = pekwm::autoProperties();
 	AutoProperty *property =
-		aps->findAutoProperty(_class_hint, Workspaces::getActive(), type);
+		aps->findAutoProperty(_class_hint, Workspaces::getActive(),
+				      type);
 	if (property) {
 		P_DBG("auto property found for client " << _class_hint);
 
 		// Make sure transient state matches
 		if (isTransient()
-		    ? property->isApplyOn(APPLY_ON_TRANSIENT|APPLY_ON_TRANSIENT_ONLY)
+		    ? property->isApplyOn(
+			APPLY_ON_TRANSIENT|APPLY_ON_TRANSIENT_ONLY)
 		    : ! property->isApplyOn(APPLY_ON_TRANSIENT_ONLY)) {
 			applyAutoprops(property);
 		} else {
@@ -1143,7 +1137,8 @@ void
 Client::readClientRemote(void)
 {
 	std::string client_machine;
-	if (X11::getTextProperty(_window, XA_WM_CLIENT_MACHINE, client_machine)) {
+	if (X11::getTextProperty(_window, XA_WM_CLIENT_MACHINE,
+				 client_machine)) {
 		_is_remote = Util::getHostname() != client_machine;
 	}
 }
@@ -1160,7 +1155,8 @@ Client::findClientID(void)
 		id = _clientids.back();
 		_clientids.pop_back();
 	} else {
-		// No free, get next number (Client is not in list when this is called.)
+		// No free, get next number (Client is not in list when this is
+		// called.)
 		id = _clients.size() + 1;
 	}
 
@@ -1226,7 +1222,7 @@ Client::titleApplyRule(std::string &title)
 //! @param title Title of client to find ID for.
 //! @return Number of clients with that id.
 uint
-Client::titleFindID(std::string &title)
+Client::titleFindID(const std::string &title)
 {
 	// Do not search for unique IDs if it is not enabled.
 	if (! pekwm::config()->getClientUniqueName()) {
@@ -1240,7 +1236,8 @@ Client::titleFindID(std::string &title)
 	for (; it != _clients.end(); ++it) {
 		if (*it != this) {
 			if ((*it)->getTitle()->getReal() == title) {
-				ids_used.push_back((*it)->getTitle()->getCount());
+				int count = (*it)->getTitle()->getCount();
+				ids_used.push_back(count);
 			}
 		}
 	}
@@ -1278,7 +1275,8 @@ Client::setWmState(ulong state)
 	X11::changeProperty(_window,
 			    X11::getAtom(WM_STATE),
 			    X11::getAtom(WM_STATE),
-			    32, PropModeReplace, (uchar*) data, 2);
+			    32, PropModeReplace,
+			    reinterpret_cast<uchar*>(data), 2);
 }
 
 // If we can't find a wm_state we're going to have to assume
@@ -1291,17 +1289,19 @@ Client::getWmState(void)
 {
 	Atom real_type;
 	int real_format;
-	long *data, state = WithdrawnState;
+	long state = WithdrawnState;
 	ulong items_read, items_left;
 	uchar *udata;
 
 	int status =
-		XGetWindowProperty(X11::getDpy(), _window, X11::getAtom(WM_STATE),
-				   0L, 2L, False, X11::getAtom(WM_STATE),
-				   &real_type, &real_format, &items_read, &items_left,
+		XGetWindowProperty(X11::getDpy(), _window,
+				   X11::getAtom(WM_STATE), 0L, 2L, False,
+				   X11::getAtom(WM_STATE),
+				   &real_type, &real_format,
+				   &items_read, &items_left,
 				   &udata);
 	if ((status  == Success) && items_read) {
-		data = reinterpret_cast<long*>(udata);
+		long *data = reinterpret_cast<long*>(udata);
 		state = *data;
 		X11::free(udata);
 	}
@@ -1330,26 +1330,27 @@ Client::configureRequestSend(void)
 	e.above = None;
 	e.override_redirect = False;
 
-	XSendEvent(X11::getDpy(), _window, false, StructureNotifyMask, (XEvent *) &e);
+	X11::sendEvent(_window, False, StructureNotifyMask,
+		       reinterpret_cast<XEvent*>(&e));
 }
 
 //! @brief Send a TAKE_FOCUS client message to the client (if requested by it).
 void Client::sendTakeFocusMessage(void)
 {
-	if (_send_focus_message) {
-		{
-			XEvent ev;
-			X11::changeProperty(X11::getRoot(),
-					    XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
-			XWindowEvent(X11::getDpy(), X11::getRoot(),
-				     PropertyChangeMask, &ev);
-			X11::setLastEventTime(ev.xproperty.time);
-		}
-		X11::sendEvent(_window, _window,
-			       X11::getAtom(WM_PROTOCOLS), NoEventMask,
-			       X11::getAtom(WM_TAKE_FOCUS),
-			       X11::getLastEventTime());
+	if (! _send_focus_message) {
+		return;
 	}
+
+	XEvent ev;
+	X11::changeProperty(X11::getRoot(),
+			    XA_PRIMARY, XA_STRING, 8, PropModeAppend, 0, 0);
+	XWindowEvent(X11::getDpy(), X11::getRoot(), PropertyChangeMask, &ev);
+	X11::setLastEventTime(ev.xproperty.time);
+
+	X11::sendEvent(_window, _window,
+		       X11::getAtom(WM_PROTOCOLS), NoEventMask,
+		       X11::getAtom(WM_TAKE_FOCUS),
+		       X11::getLastEventTime());
 }
 
 /**
@@ -1470,7 +1471,8 @@ Client::getAspectSize(uint *r_w, uint *r_h, uint w, uint h)
 
 		double tmp;
 
-		// Ensure: min_aspect.x/min_aspect.y <= w/h <= max_aspect.x/max_aspect.y
+		// Ensure: min_aspect.x/min_aspect.y <= w/h <=
+		// max_aspect.x/max_aspect.y
 		if (w * amin_y < h * amin_x) {
 			tmp = ((double)(w * amin_x + h * amin_y)) /
 				((double)(amin_x * amin_x + amin_y * amin_y));
@@ -1505,16 +1507,14 @@ Client::getAspectSize(uint *r_w, uint *r_h, uint w, uint h)
  */
 bool
 Client::getIncSize(const XSizeHints& size,
-                   uint *r_w, uint *r_h, uint w, uint h, bool incr)
+		   uint *r_w, uint *r_h, uint w, uint h, bool incr)
 {
-	uint basex, basey;
-
 	if (size.flags&PResizeInc) {
-		basex = (size.flags&PBaseSize)
+		uint basex = (size.flags&PBaseSize)
 			? size.base_width
 			: (size.flags&PMinSize) ? size.min_width : 0;
 
-		basey = (size.flags&PBaseSize)
+		uint basey = (size.flags&PBaseSize)
 			? size.base_height
 			: (size.flags&PMinSize) ? size.min_height : 0;
 
@@ -1587,7 +1587,7 @@ Client::updateEwmhStates(void)
 
 	Atom *atoms = new Atom[(states.size() > 0) ? states.size() : 1];
 	if (states.size() > 0) {
-		copy(states.begin(), states.end(), atoms);
+		std::copy(states.begin(), states.end(), atoms);
 	}
 	X11::setAtoms(_window, STATE, atoms, states.size());
 	delete [] atoms;
@@ -1596,45 +1596,17 @@ Client::updateEwmhStates(void)
 void
 Client::updateWinType(bool set)
 {
-	// try to figure out what kind of window we are and alter it accordingly
+	// try to figure out what kind of window we are and alter it
+	// accordingly
 	int items;
-	Atom *atoms = 0;
-
-	_window_type = WINDOW_TYPE;
-	atoms = (Atom*) X11::getEwmhPropData(_window, WINDOW_TYPE, XA_ATOM, items);
-	if (atoms) {
-		for (int i = 0; _window_type == WINDOW_TYPE && i < items; ++i) {
-			if (atoms[i] == X11::getAtom(WINDOW_TYPE_DESKTOP)) {
-				_window_type = WINDOW_TYPE_DESKTOP;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DOCK)) {
-				_window_type = WINDOW_TYPE_DOCK;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_TOOLBAR)) {
-				_window_type = WINDOW_TYPE_TOOLBAR;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_MENU)) {
-				_window_type = WINDOW_TYPE_MENU;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_UTILITY)) {
-				_window_type = WINDOW_TYPE_UTILITY;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DIALOG)) {
-				_window_type = WINDOW_TYPE_DIALOG;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_SPLASH)) {
-				_window_type = WINDOW_TYPE_SPLASH;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DROPDOWN_MENU)) {
-				_window_type = WINDOW_TYPE_DROPDOWN_MENU;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_POPUP_MENU)) {
-				_window_type = WINDOW_TYPE_POPUP_MENU;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_TOOLTIP)) {
-				_window_type = WINDOW_TYPE_TOOLTIP;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_NOTIFICATION)) {
-				_window_type = WINDOW_TYPE_NOTIFICATION;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_COMBO)) {
-				_window_type = WINDOW_TYPE_COMBO;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DND)) {
-				_window_type = WINDOW_TYPE_DND;
-			} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_NORMAL)) {
-				_window_type = WINDOW_TYPE_NORMAL;
-			}
-		}
+	void *data =
+		X11::getEwmhPropData(_window, WINDOW_TYPE, XA_ATOM, items);
+	if (data) {
+		Atom *atoms = static_cast<Atom*>(data);
+		_window_type = findWinType(atoms, items);
 		X11::free(atoms);
+	} else {
+		_window_type = WINDOW_TYPE;
 	}
 
 	// Set window type to WINDOW_TYPE_NORMAL if it did not match
@@ -1646,6 +1618,44 @@ Client::updateWinType(bool set)
 	}
 }
 
+AtomName
+Client::findWinType(const Atom* atoms, int items)
+{
+	for (int i = 0; i < items; ++i) {
+		if (atoms[i] == X11::getAtom(WINDOW_TYPE_DESKTOP)) {
+			return WINDOW_TYPE_DESKTOP;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DOCK)) {
+			return WINDOW_TYPE_DOCK;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_TOOLBAR)) {
+			return WINDOW_TYPE_TOOLBAR;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_MENU)) {
+			return WINDOW_TYPE_MENU;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_UTILITY)) {
+			return WINDOW_TYPE_UTILITY;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DIALOG)) {
+			return WINDOW_TYPE_DIALOG;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_SPLASH)) {
+			return WINDOW_TYPE_SPLASH;
+		} else if (atoms[i]
+				== X11::getAtom(WINDOW_TYPE_DROPDOWN_MENU)) {
+			return WINDOW_TYPE_DROPDOWN_MENU;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_POPUP_MENU)) {
+			return WINDOW_TYPE_POPUP_MENU;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_TOOLTIP)) {
+			return WINDOW_TYPE_TOOLTIP;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_NOTIFICATION)) {
+			return WINDOW_TYPE_NOTIFICATION;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_COMBO)) {
+			return WINDOW_TYPE_COMBO;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_DND)) {
+			return WINDOW_TYPE_DND;
+		} else if (atoms[i] == X11::getAtom(WINDOW_TYPE_NORMAL)) {
+			return WINDOW_TYPE_NORMAL;
+		}
+	}
+	return WINDOW_TYPE;
+}
+
 /**
  * Read WM Hints, return the initial state of the window.
  */
@@ -1653,20 +1663,19 @@ ulong
 Client::getWMHints(void)
 {
 	ulong initial_state = NormalState;
-	XWMHints* hints = XGetWMHints(X11::getDpy(), _window);
-	if (hints) {
+	XWMHints hints;
+	if (X11::getWMHints(_window, hints)) {
 		// get the input focus mode
-		if (hints->flags&InputHint) { // FIXME: More logic needed
-			_wm_hints_input = hints->input;
+		if (hints.flags&InputHint) { // FIXME: More logic needed
+			_wm_hints_input = hints.input;
 		}
 
 		// Get initial state of the window
-		if (hints->flags&StateHint) {
-			initial_state = hints->initial_state;
+		if (hints.flags&StateHint) {
+			initial_state = hints.initial_state;
 		}
 
-		setDemandsAttention(hints->flags&XUrgencyHint);
-		X11::free(hints);
+		setDemandsAttention(hints.flags&XUrgencyHint);
 	}
 	return initial_state;
 }
@@ -1718,18 +1727,18 @@ Client::getWMProtocols(void)
 {
 	int count;
 	Atom *protocols;
-
-	if (XGetWMProtocols(X11::getDpy(), _window, &protocols, &count) != 0) {
-		for (int i = 0; i < count; ++i) {
-			if (protocols[i] == X11::getAtom(WM_TAKE_FOCUS)) {
-				_send_focus_message = true;
-			} else if (protocols[i] == X11::getAtom(WM_DELETE_WINDOW)) {
-				_send_close_message = true;
-			}
-		}
-
-		X11::free(protocols);
+	if (! XGetWMProtocols(X11::getDpy(), _window, &protocols, &count)) {
+		return;
 	}
+
+	for (int i = 0; i < count; ++i) {
+		if (protocols[i] == X11::getAtom(WM_TAKE_FOCUS)) {
+			_send_focus_message = true;
+		} else if (protocols[i] == X11::getAtom(WM_DELETE_WINDOW)) {
+			_send_close_message = true;
+		}
+	}
+	X11::free(protocols);
 }
 
 /**
@@ -1747,9 +1756,11 @@ Client::getTransientForHint(void)
 			P_ERR(this << " client set transient hint for itself");
 			_transient_for_window = None;
 		} else {
-			transient_for = findClientFromWindow(_transient_for_window);
-			P_TRACE(this << " transient for " << _transient_for_window
-				<< " client " << transient_for);
+			transient_for =
+				findClientFromWindow(_transient_for_window);
+			P_TRACE(this << " transient for "
+				<< _transient_for_window << " client "
+				<< transient_for);
 		}
 	}
 
@@ -1814,15 +1825,14 @@ Client::getStrutHint(void)
 	removeStrutHint();
 
 	int num = 0;
-	long *strut = static_cast<long*>(X11::getEwmhPropData(_window, NET_WM_STRUT,
-							      XA_CARDINAL, num));
-	if (strut) {
-		_strut = new Strut();
-		*_strut = strut;
-		X11::free(strut);
+	void* data = X11::getEwmhPropData(_window, NET_WM_STRUT,
+					  XA_CARDINAL, num);
+	if (data) {
+		_strut = new Strut(static_cast<long*>(data));
 		if (! isCfgDeny(CFG_DENY_STRUT)) {
 			pekwm::rootWo()->addStrut(_strut);
 		}
+		X11::free(data);
 	}
 }
 
@@ -1890,12 +1900,12 @@ void
 Client::setClientEnvironment(Client *client)
 {
 	if (client) {
-		setenv("CLIENT_PID",
-		       std::to_string(client->isRemote()
-				      ? -1 : client->getPid()).c_str(), 1);
-		setenv("CLIENT_WINDOW", std::to_string(client->getWindow()).c_str(), 1);
+		int client_pid = client->isRemote() ? -1 : client->getPid();
+		Util::setEnv("CLIENT_PID", std::to_string(client_pid));
+		Util::setEnv("CLIENT_WINDOW",
+			     std::to_string(client->getWindow()));
 	} else {
-		unsetenv("CLIENT_PID");
-		unsetenv("CLIENT_WINDOW");
+		Util::setEnv("CLIENT_PID", "");
+		Util::setEnv("CLIENT_WINDOW", "");
 	}
 }
