@@ -1,6 +1,6 @@
 //
 // Frame.cc for pekwm
-// Copyright (C) 2002-2020 Claes Nästén <pekdon@gmail.com>
+// Copyright (C) 2002-2023 Claes Nästén <pekdon@gmail.com>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -41,6 +41,7 @@ std::vector<Frame*> Frame::_frames;
 std::vector<uint> Frame::_frameid_list;
 
 ActionEvent Frame::_ae_move = ActionEvent(Action(ACTION_MOVE));
+ActionEvent Frame::_ae_resize = ActionEvent(Action(ACTION_RESIZE));
 ActionEvent Frame::_ae_move_resize = ActionEvent(Action(ACTION_MOVE_RESIZE));
 
 Frame* Frame::_tag_frame = nullptr;
@@ -74,22 +75,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
 	_decor_cfg_bpr_al_title = MOUSE_ACTION_LIST_TITLE_FRAME;
 	_decor_cfg_bpr_al_child = MOUSE_ACTION_LIST_CHILD_FRAME;
 
-	// grab buttons so that we can reply them
-	const std::vector<BoundButton> &cmab =
-		pekwm::config()->getClientMouseActionButtons();
-	std::vector<BoundButton>::const_iterator it = cmab.begin();
-	for (; it != cmab.end(); ++it) {
-		if (it->button == BUTTON_ANY) {
-			continue;
-		}
-
-		std::vector<uint>::const_iterator mit = it->mods.begin();
-		for (; mit != it->mods.end(); ++mit) {
-			X11Util::grabButton(it->button, *mit,
-					    ButtonPressMask|ButtonReleaseMask,
-					    _window, GrabModeSync);
-		}
-	}
+	grabButtons();
 
 	// get unique id of the frame, if the client didn't have an id
 	if (pekwm::isStarting()) {
@@ -97,7 +83,6 @@ Frame::Frame(Client *client, AutoProperty *ap)
 		if (X11::getCardinal(client->getWindow(), PEKWM_FRAME_ID, id)) {
 			_id = id;
 		}
-
 	} else {
 		_id = findFrameID();
 	}
@@ -164,7 +149,7 @@ Frame::Frame(Client *client, AutoProperty *ap)
 
 	// needs to be done before the workspace insert and after the client
 	// has been inserted, in order for layer settings to be propagated
-	setLayer(client->getLayer());
+	Frame::setLayer(client->getLayer());
 
 	// I add these to the list before I insert the client into the frame to
 	// be able to skip an extra updateClientList
@@ -220,6 +205,32 @@ Frame::~Frame(void)
 	workspacesRemove();
 }
 
+void
+Frame::grabButtons(void)
+{
+	// grab buttons so that we can reply them
+	const std::vector<BoundButton> &cmab =
+		pekwm::config()->getClientMouseActionButtons();
+	std::vector<BoundButton>::const_iterator it = cmab.begin();
+	for (; it != cmab.end(); ++it) {
+		if (it->button == BUTTON_ANY) {
+			continue;
+		}
+
+		std::vector<uint>::const_iterator mit = it->mods.begin();
+		for (; mit != it->mods.end(); ++mit) {
+			// only grabbing non-modifier buttons, the rest will
+			// be grabbed by the client code.
+			if (*mit) {
+				continue;
+			}
+			X11Util::grabButton(it->button, *mit,
+					    ButtonPressMask|ButtonReleaseMask,
+					    _window, GrabModeSync);
+		}
+	}
+}
+
 // START - PWinObj interface.
 
 //! @brief Iconifies the Frame.
@@ -260,8 +271,8 @@ Frame::setWorkspace(unsigned int workspace)
 			return;
 		}
 
-		// First we set the workspace, then load autoproperties for possible
-		// overrun of workspace and then set the workspace.
+		// First we set the workspace, then load autoproperties for
+		// possible overrun of workspace and then set the workspace.
 		_workspace = workspace;
 		readAutoprops(APPLY_ON_WORKSPACE);
 		workspace = _workspace;
@@ -280,7 +291,8 @@ Frame::setLayer(Layer layer)
 		_client->setLayer(layer);
 
 		LayerObservation observation(layer);
-		pekwm::observerMapping()->notifyObservers(_client, &observation);
+		pekwm::observerMapping()->notifyObservers(_client,
+							  &observation);
 	}
 }
 
@@ -289,21 +301,21 @@ Frame::setLayer(Layer layer)
 ActionEvent*
 Frame::handleMotionEvent(XMotionEvent *ev)
 {
-	// This is true when we have a title button pressed and then we don't want
-	// to be able to drag windows around, therefore we ignore the event
+	// This is true when we have a title button pressed and then we don't
+	// want to be able to drag windows around, therefore we ignore the
+	// event
 	if (_button) {
 		return 0;
 	}
 
-	ActionEvent *ae = nullptr;
+	Config* cfg = pekwm::config();
+	std::vector<ActionEvent>* al = nullptr;
 	uint button = X11::getButtonFromState(ev->state);
 
 	if (ev->window == getTitleWindow()) {
-		ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
-						    pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_TITLE_FRAME));
+		al = cfg->getMouseActionList(MOUSE_ACTION_LIST_TITLE_FRAME);
 	} else if (ev->window == _client->getWindow()) {
-		ae = ActionHandler::findMouseAction(button, ev->state, MOUSE_EVENT_MOTION,
-						    pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME));
+		al = cfg->getMouseActionList(MOUSE_ACTION_LIST_CHILD_FRAME);
 	} else {
 		uint pos = getBorderPosition(ev->subwindow);
 
@@ -313,13 +325,14 @@ Frame::handleMotionEvent(XMotionEvent *ev)
 			pos = getBorderPosition(ev->window);
 		}
 		if (pos != BORDER_NO_POS) {
-			std::vector<ActionEvent> *bl =
-				pekwm::config()->getBorderListFromPosition(pos);
-			ae = ActionHandler::findMouseAction(button, ev->state,
-							    MOUSE_EVENT_MOTION, bl);
+			al = cfg->getBorderListFromPosition(pos);
 		}
 	}
 
+	ActionEvent* ae = ActionHandler::findMouseAction(button, ev->state,
+							 MOUSE_EVENT_MOTION,
+							 al);
+
 	// check motion threshold
 	if (ae && (ae->threshold > 0)) {
 		if (! ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
@@ -339,7 +352,6 @@ Frame::handleEnterEvent(XCrossingEvent *ev)
 	// returned.
 	PDecor::handleEnterEvent(ev);
 
-	ActionEvent *ae = 0;
 	std::vector<ActionEvent> *al = 0;
 	Config *cfg = pekwm::config();
 
@@ -354,11 +366,8 @@ Frame::handleEnterEvent(XCrossingEvent *ev)
 		}
 	}
 
-	if (al) {
-		ae = ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_ENTER, al);
-	}
-
-	return ae;
+	return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
+					      MOUSE_EVENT_ENTER, al);
 }
 
 ActionEvent*
@@ -368,17 +377,14 @@ Frame::handleLeaveEvent(XCrossingEvent *ev)
 	// returned.
 	PDecor::handleLeaveEvent(ev);
 
-	ActionEvent *ae;
-
 	MouseActionListName ln = MOUSE_ACTION_LIST_TITLE_FRAME;
 	if (ev->window == _client->getWindow()) {
 		ln = MOUSE_ACTION_LIST_CHILD_FRAME;
 	}
 
-	ae = ActionHandler::findMouseAction(BUTTON_ANY, ev->state, MOUSE_EVENT_LEAVE,
-					    pekwm::config()->getMouseActionList(ln));
-
-	return ae;
+	std::vector<ActionEvent>* al = pekwm::config()->getMouseActionList(ln);
+	return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
+					      MOUSE_EVENT_LEAVE, al);
 }
 
 ActionEvent*
@@ -466,11 +472,11 @@ Frame::addChild(PWinObj *child, std::vector<PWinObj*>::iterator *it)
 void
 Frame::addChildOrdered(Client *child)
 {
-	Client *client;
 	std::vector<PWinObj*>::iterator it(_children.begin());
 	for (; it != _children.end(); ++it) {
-		client = static_cast<Client*>(*it);
-		if (child->getInitialFrameOrder() < client->getInitialFrameOrder()) {
+		Client *client = static_cast<Client*>(*it);
+		if (child->getInitialFrameOrder()
+		    < client->getInitialFrameOrder()) {
 			break;
 		}
 	}
@@ -541,10 +547,9 @@ Frame::updatedChildOrder(void)
 {
 	titleClear();
 
-	Client *client;
 	std::vector<PWinObj*>::iterator it(_children.begin());
 	for (long num = 0; it != _children.end(); ++num, ++it) {
-		client = static_cast<Client*>(*it);
+		Client *client = static_cast<Client*>(*it);
 		client->setPekwmFrameOrder(num);
 		titleAdd(client->getTitle());
 	}
@@ -560,10 +565,9 @@ Frame::updatedActiveChild(void)
 {
 	titleSetActive(0);
 
-	Client *client;
 	uint size = _children.size();
 	for (uint i = 0; i < size; ++i) {
-		client = static_cast<Client*>(_children[i]);
+		Client *client = static_cast<Client*>(_children[i]);
 		client->setPekwmFrameActive(_child == client);
 		if (_child == client) {
 			titleSetActive(i);
@@ -578,7 +582,7 @@ Frame::getDecorInfo(char *buf, uint size, const Geometry& gm)
 {
 	uint width, height;
 	calcSizeInCells(width, height, gm);
-	snprintf(buf, size, "%d+%d+%d+%d", width, height, gm.x, gm.y);
+	snprintf(buf, size, "%u+%u+%d+%d", width, height, gm.x, gm.y);
 }
 
 void
@@ -591,21 +595,20 @@ Frame::giveInputFocus(void)
 	PDecor::giveInputFocus();
 }
 
-void
+bool
 Frame::setShaded(StateAction sa)
 {
-	bool shaded = isShaded();
-
 	// Check for DisallowedActions="Shade"
 	if (! _client->allowShade()) {
 		sa = STATE_UNSET;
 	}
 
-	PDecor::setShaded(sa);
-	if (shaded != isShaded()) {
+	bool shaded = isShaded();
+	if (shaded != PDecor::setShaded(sa)) {
 		_client->setShade(isShaded());
 		_client->updateEwmhStates();
 	}
+	return isShaded();
 }
 
 /**
@@ -620,76 +623,6 @@ Frame::decorUpdated(void)
 	}
 }
 
-int
-Frame::resizeHorzStep(int diff) const
-{
-	int diff_ret = 0;
-	uint min = _gm.width - getChildWidth();
-	if (min == 0) { // borderless windows, we don't want X errors
-		min = 1;
-	}
-	XSizeHints hints = _client->getActiveSizeHints();
-
-	// if we have ResizeInc hint set we use it instead of pixel diff
-	if (hints.flags&PResizeInc) {
-		if (diff > 0) {
-			diff_ret = hints.width_inc;
-		} else if ((_gm.width - hints.width_inc) >= min) {
-			diff_ret = -hints.width_inc;
-		}
-	} else if ((_gm.width + diff) >= min) {
-		diff_ret = diff;
-	}
-
-	// check max/min size hints
-	if (diff > 0) {
-		if ((hints.flags&PMaxSize) &&
-		    ((getChildWidth() + diff) > unsigned(hints.max_width))) {
-			diff_ret = _gm.width - hints.max_width + min;
-		}
-	} else if ((hints.flags&PMinSize) &&
-		   ((getChildWidth() + diff) < unsigned(hints.min_width))) {
-		diff_ret = _gm.width - hints.min_width + min;
-	}
-
-	return diff_ret;
-}
-
-int
-Frame::resizeVertStep(int diff) const
-{
-	int diff_ret = 0;
-	uint min = _gm.height - getChildHeight();
-	if (min == 0) { // borderless windows, we don't want X errors
-		min = 1;
-	}
-	XSizeHints hints = _client->getActiveSizeHints();
-
-	// if we have ResizeInc hint set we use it instead of pixel diff
-	if (hints.flags&PResizeInc) {
-		if (diff > 0) {
-			diff_ret = hints.height_inc;
-		} else if ((_gm.height - hints.height_inc) >= min) {
-			diff_ret = -hints.height_inc;
-		}
-	} else {
-		diff_ret = diff;
-	}
-
-	// check max/min size hints
-	if (diff > 0) {
-		if ((hints.flags&PMaxSize) &&
-		    ((getChildHeight() + diff) > unsigned(hints.max_height))) {
-			diff_ret = _gm.height - hints.max_height + min;
-		}
-	} else if ((hints.flags&PMinSize) &&
-		   ((getChildHeight() + diff) < unsigned(hints.min_height))) {
-		diff_ret = _gm.height - hints.min_width + min;
-	}
-
-	return diff_ret;
-}
-
 /**
  * Return decor name for the current client or attention decor if
  * Frame has client which demands attention.
@@ -771,7 +704,7 @@ Frame::applyState(Client *cl)
 	if (! cl) {
 		return;
 	}
-    
+
 	cl->setSticky(_sticky);
 	cl->setMaximizedHorz(_maximized_horz);
 	cl->setMaximizedVert(_maximized_vert);
@@ -813,26 +746,6 @@ Frame::setSkip(uint skip)
 	_client->setSkip(skip);
 }
 
-//! @brief Find Frame with Window
-//! @param win Window to search for.
-//! @return Frame if found, else 0.
-Frame*
-Frame::findFrameFromWindow(Window win)
-{
-	// Validate input window.
-	if ((win == None) || (win == X11::getRoot())) {
-		return 0;
-	}
-
-	frame_cit it = _frames.begin();
-	for (; it != _frames.end(); ++it) {
-		if (win == (*it)->getWindow()) { // operator == does more than that
-			return *it;
-		}
-	}
-	return nullptr;
-}
-
 //! @brief Find Frame with id.
 //! @param id ID to search for.
 //! @return Frame if found, else 0.
@@ -895,13 +808,14 @@ Frame::applyGeometry(Geometry &gm, const Geometry &ap_gm, int mask)
  */
 void
 Frame::applyGeometry(Geometry &gm, const Geometry &ap_gm, int mask,
-                     const Geometry &screen_gm)
+		     const Geometry &screen_gm)
 {
 	// Read size before position so negative position works, if size is
 	// < 1 consider it to be full screen size.
 	if (mask & WIDTH_VALUE) {
 		if (mask & WIDTH_PERCENT) {
-			gm.width = int(screen_gm.width * (float(ap_gm.width) / 100));
+			gm.width = int(screen_gm.width
+					* (float(ap_gm.width) / 100));
 		} else if (ap_gm.width < 1) {
 			gm.width = screen_gm.width;
 		} else {
@@ -911,7 +825,8 @@ Frame::applyGeometry(Geometry &gm, const Geometry &ap_gm, int mask,
 
 	if (mask & HEIGHT_VALUE) {
 		if (mask & HEIGHT_PERCENT) {
-			gm.height = int(screen_gm.height * (float(ap_gm.height) / 100));
+			gm.height = int(screen_gm.height
+					* (float(ap_gm.height) / 100));
 		} else if (ap_gm.height < 1) {
 			gm.height = screen_gm.height;
 		} else {
@@ -952,7 +867,8 @@ Frame::findFrameID(void)
 		id = _frameid_list.back();
 		_frameid_list.pop_back();
 	} else {
-		// No free, get next number (Frame is not in list when this is called.)
+		// No free, get next number (Frame is not in list when this is
+		// called.)
 		id = _frames.size() + 1;
 	}
 
@@ -980,28 +896,35 @@ Frame::resetFrameIDs(void)
 	}
 }
 
-
-//! @brief Removes the client from the Frame and creates a new Frame for it
-void
-Frame::detachClient(Client *client)
+/**
+ * Remove given client from frame moving it to a new frame at the
+ * specified coordinates.
+ *
+ * @return new Frame on success, nullptr if client is not part of frame or is
+ *         the only client in the frame.
+ */
+Frame*
+Frame::detachClient(Client *client, int x, int y)
 {
 	if (client->getParent() != this) {
-		return;
+		return nullptr;
+	} else if (_children.size() < 2) {
+		move(x, y);
+		return nullptr;
 	}
 
-	if (_children.size() > 1) {
-		removeChild(client);
+	removeChild(client);
 
-		client->move(_gm.x, _gm.y + bdTop(this));
+	client->move(x, y + bdTop(this));
 
-		Frame *frame = new Frame(client, 0);
-		frame->workspacesInsert();
+	Frame *frame = new Frame(client, nullptr);
+	frame->workspacesInsert();
+	client->setParent(frame);
+	client->setWorkspace(Workspaces::getActive());
 
-		client->setParent(frame);
-		client->setWorkspace(Workspaces::getActive());
+	setFocused(false);
 
-		setFocused(false);
-	}
+	return frame;
 }
 
 //! @brief Makes sure the frame doesn't cover any struts / the harbour.
@@ -1041,299 +964,8 @@ Frame::fixGeometry(void)
 	return (_gm != before);
 }
 
-/**
- * Initiates grouping move, based on a XMotionEvent.
- *
- * @fixme rewrite
- */
 void
-Frame::doGroupingDrag(XMotionEvent *ev, Client *client, bool behind)
-{
-	if (! client) {
-		return;
-	}
-
-	int o_x, o_y;
-	o_x = ev ? ev->x_root : 0;
-	o_y = ev ? ev->y_root : 0;
-
-	std::string name("Grouping ");
-	if (client->getTitle()->getVisible().size() > 0) {
-		name += client->getTitle()->getVisible();
-	} else {
-		name += "No Name";
-	}
-
-	bool status = X11::grabPointer(X11::getRoot(),
-				       ButtonReleaseMask|PointerMotionMask,
-				       CURSOR_NONE);
-	if (status != true) {
-		return;
-	}
-
-	StatusWindow *sw = pekwm::statusWindow();
-
-	sw->draw(name); // resize window and render bg
-	sw->move(o_x, o_y);
-	sw->mapWindowRaised();
-	sw->draw(name); // redraw after map
-
-	XEvent e;
-	while (true) { // this breaks when we get an button release
-		XMaskEvent(X11::getDpy(), PointerMotionMask|ButtonReleaseMask, &e);
-
-		switch (e.type)  {
-		case MotionNotify:
-			// update the position
-			o_x = e.xmotion.x_root;
-			o_y = e.xmotion.y_root;
-
-			sw->move(o_x, o_y);
-			sw->draw(name);
-			break;
-
-		case ButtonRelease:
-			sw->unmapWindow();
-			X11::ungrabPointer();
-
-			Client *search = 0;
-
-			// only group if we have grouping turned on
-			if (ClientMgr::isAllowGrouping()) {
-				int x, y;
-				Window win;
-
-				// find the frame we dropped the client on
-				XTranslateCoordinates(X11::getDpy(),
-						      X11::getRoot(), X11::getRoot(),
-						      e.xmotion.x_root, e.xmotion.y_root,
-						      &x, &y, &win);
-
-				search = Client::findClient(win);
-			}
-
-			// if we found a client, and it's not in the current frame and
-			// it has a "normal" ( make configurable? ) layer we group
-			if (search && search->getParent() &&
-			    (search->getParent() != this) &&
-			    (search->getLayer() > LAYER_BELOW) &&
-			    (search->getLayer() < LAYER_ONTOP)) {
-
-				// if we currently have focus and the frame exists after we remove
-				// this client we need to redraw it as unfocused
-				bool focus = behind ? false : (_children.size() > 1);
-
-				removeChild(client);
-
-				Frame *frame = static_cast<Frame*>(search->getParent());
-				frame->addChild(client);
-				if (! behind) {
-					frame->activateChild(client);
-					frame->giveInputFocus();
-				}
-
-				if (focus) {
-					setFocused(false);
-				}
-
-			}  else if (_children.size() > 1) {
-				// if we have more than one client in the frame detach this one
-				removeChild(client);
-
-				client->move(e.xmotion.x_root, e.xmotion.y_root);
-
-				Frame *frame = new Frame(client, 0);
-				client->setParent(frame);
-				// make sure the client ends up on the current workspace
-				client->setWorkspace(Workspaces::getActive());
-				frame->workspacesInsert();
-
-				// make sure it get's focus
-				setFocused(false);
-				frame->giveInputFocus();
-			}
-
-			return;
-		}
-	}
-}
-
-//! @brief Initiates resizing of a window based on motion event
-void
-Frame::doResize(XMotionEvent *ev)
-{
-	if (! ev) {
-		return;
-	}
-    
-	// figure out which part of the window we are in
-	bool left = false, top = false;
-	if (ev->x < signed(_gm.width / 2)) {
-		left = true;
-	}
-	if (ev->y < signed(_gm.height / 2)) {
-		top = true;
-	}
-    
-	doResize(left, true, top, true);
-}
-
-//! @brief Initiates resizing of a window based border position
-void
-Frame::doResize(BorderPosition pos)
-{
-	bool x = false, y = false;
-	bool left = false, top = false, resize = true;
-
-	switch (pos) {
-	case BORDER_TOP_LEFT:
-		x = y = left = top = true;
-		break;
-	case BORDER_TOP:
-		y = top = true;
-		break;
-	case BORDER_TOP_RIGHT:
-		x = y = top = true;
-		break;
-	case BORDER_LEFT:
-		x = left = true;
-		break;
-	case BORDER_RIGHT:
-		x = true;
-		break;
-	case BORDER_BOTTOM_LEFT:
-		x = y = left = true;
-		break;
-	case BORDER_BOTTOM:
-		y = true;
-		break;
-	case BORDER_BOTTOM_RIGHT:
-		x = y = true;
-		break;
-	default:
-		resize = false;
-		break;
-	}
-
-	if (resize) {
-		doResize(left, x, top, y);
-	}
-}
-
-//! @brief Resizes the frame by handling MotionNotify events.
-void
-Frame::doResize(bool left, bool x, bool top, bool y)
-{
-	if (! _client->allowResize() || isShaded()) {
-		return;
-	}
-
-	if (! X11::grabPointer(X11::getRoot(), ButtonMotionMask|ButtonReleaseMask,
-			       CURSOR_RESIZE)) {
-		return;
-	}
-
-	setShaded(STATE_UNSET); // make sure the frame isn't shaded
-	setStateFullscreen(STATE_UNSET);
-
-	// Initialize variables
-	int start_x, new_x;
-	int start_y, new_y;
-	uint old_width;
-	uint old_height;
-
-	start_x = new_x = left ? _gm.x : (_gm.x + _gm.width);
-	start_y = new_y = top ? _gm.y : (_gm.y + _gm.height);
-	old_width = _gm.width;
-	old_height = _gm.height;
-
-	// the basepoint of our window
-	_click_x = left ? (_gm.x + _gm.width) : _gm.x;
-	_click_y = top ? (_gm.y + _gm.height) : _gm.y;
-
-	int pointer_x = _gm.x, pointer_y = _gm.y;
-	X11::getMousePosition(pointer_x, pointer_y);
-
-	char buf[128];
-	getDecorInfo(buf, 128, _gm);
-
-	bool center_on_root = pekwm::config()->isShowStatusWindowOnRoot();
-	StatusWindow *sw = pekwm::statusWindow();
-	if (pekwm::config()->isShowStatusWindow()) {
-		sw->draw(buf, true, center_on_root ? 0 : &_gm);
-		sw->mapWindowRaised();
-		sw->draw(buf, true, center_on_root ? 0 : &_gm);
-	}
-
-	bool outline = ! pekwm::config()->getOpaqueResize();
-
-	// grab server, we don't want invert traces
-	if (outline) {
-		X11::grabServer();
-	}
-
-	const long resize_mask = ButtonPressMask|ButtonReleaseMask|ButtonMotionMask;
-	XEvent ev;
-	bool exit = false;
-	while (exit != true) {
-		if (outline) {
-			drawOutline(_gm);
-		}
-		XMaskEvent(X11::getDpy(), resize_mask, &ev);
-		if (outline) {
-			drawOutline(_gm); // clear
-		}
-
-		switch (ev.type) {
-		case MotionNotify:
-			// Flush all pointer motion, no need to redraw and redraw.
-			X11::removeMotionEvents();
-
-			if (x) {
-				new_x = start_x - pointer_x + ev.xmotion.x;
-			}
-			if (y) {
-				new_y = start_y - pointer_y + ev.xmotion.y;
-			}
-
-			recalcResizeDrag(new_x, new_y, left, top);
-
-			getDecorInfo(buf, 128, _gm);
-			if (pekwm::config()->isShowStatusWindow()) {
-				sw->draw(buf, true, center_on_root ? 0 : &_gm);
-			}
-
-			// only updated when needed when in opaque mode
-			if (! outline) {
-				if ((old_width != _gm.width) || (old_height != _gm.height)) {
-					moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
-				}
-				old_width = _gm.width;
-				old_height = _gm.height;
-			}
-			break;
-		case ButtonRelease:
-			exit = true;
-			break;
-		}
-	}
-
-	if (pekwm::config()->isShowStatusWindow()) {
-		sw->unmapWindow();
-	}
-
-	X11::ungrabPointer();
-
-	// Make sure the state isn't set to maximized after we've resized.
-	clearMaximizedStatesAfterResize();
-
-	if (outline) {
-		moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
-		X11::ungrabServer(true);
-	}
-}
-
-void Frame::clearMaximizedStatesAfterResize()
+Frame::clearMaximizedStatesAfterResize(void)
 {
 	if (_maximized_horz || _maximized_vert) {
 		_maximized_horz = false;
@@ -1344,57 +976,43 @@ void Frame::clearMaximizedStatesAfterResize()
 	}
 }
 
-//! @brief Updates the width, height of the frame when resizing it.
 void
-Frame::recalcResizeDrag(int nx, int ny, bool left, bool top)
+Frame::moveToHead(const std::string& arg)
 {
-	if (left) {
-		if (nx >= signed(_click_x - decorWidth(this)))
-			nx = _click_x - decorWidth(this) - 1;
-	} else {
-		if (nx <= signed(_click_x + decorWidth(this)))
-			nx = _click_x + decorWidth(this) + 1;
+	int head_nr;
+	try {
+		head_nr = std::stoi(arg);
+	} catch (std::invalid_argument&) {
+		head_nr = -1;
 	}
 
-	if (top) {
-		if (ny >= signed(_click_y - decorHeight(this))) {
-			ny = _click_y - decorHeight(this) - 1;
-		}
+	if (head_nr != -1) {
+		// valid value from stoi above
+	} else if (pekwm::ascii_ncase_equal(arg, "LEFT")) {
+		head_nr = X11Util::getNearestHead(*this,
+						  DIRECTION_LEFT,
+						  DIRECTION_NO);
+	} else if (pekwm::ascii_ncase_equal(arg, "RIGHT")) {
+		head_nr = X11Util::getNearestHead(*this,
+						  DIRECTION_RIGHT,
+						  DIRECTION_NO);
+	} else if (pekwm::ascii_ncase_equal(arg, "UP")) {
+		head_nr = X11Util::getNearestHead(*this,
+						  DIRECTION_NO,
+						  DIRECTION_UP);
+	} else if (pekwm::ascii_ncase_equal(arg, "DOWN")) {
+		head_nr = X11Util::getNearestHead(*this,
+						  DIRECTION_NO,
+						  DIRECTION_DOWN);
 	} else {
-		if (ny <= signed(_click_y + decorHeight(this))) {
-			ny = _click_y + decorHeight(this) + 1;
+		head_nr = X11::findHeadByName(arg);
+		if (head_nr == -1) {
+			P_ERR("unrecognized MoveToHead argument: " << arg);
 		}
 	}
-
-	uint width = left ? (_click_x - nx) : (nx - _click_x);
-	uint height = top ? (_click_y - ny) : (ny - _click_y);
-
-	width -= decorWidth(this);
-	height -= decorHeight(this);
-
-	_client->getAspectSize(&width, &height, width, height);
-
-	XSizeHints hints = _client->getActiveSizeHints();
-	// check so we aren't overriding min or max size
-	if (hints.flags & PMinSize) {
-		if (signed(width) < hints.min_width)
-			width = hints.min_width;
-		if (signed(height) < hints.min_height)
-			height = hints.min_height;
+	if (head_nr != -1) {
+		moveToHead(head_nr);
 	}
-
-	if (hints.flags & PMaxSize) {
-		if (signed(width) > hints.max_width)
-			width = hints.max_width;
-		if (signed(height) > hints.max_height)
-			height = hints.max_height;
-	}
-
-	_gm.width = width + decorWidth(this);
-	_gm.height = height + decorHeight(this);
-
-	_gm.x = left ? (_click_x - _gm.width) : _click_x;
-	_gm.y = top ? (_click_y - _gm.height) : _click_y;
 }
 
 void
@@ -1406,12 +1024,27 @@ Frame::moveToHead(int head_nr)
 	}
 
 	Geometry old_gm, new_gm;
-	pekwm::rootWo()->getHeadInfoWithEdge(curr_head_nr, old_gm);
-	pekwm::rootWo()->getHeadInfoWithEdge(head_nr, new_gm);
+	if (isFullscreen()) {
+		X11::getHeadInfo(curr_head_nr, old_gm);
+		X11::getHeadInfo(head_nr, new_gm);
+	} else {
+		pekwm::rootWo()->getHeadInfoWithEdge(curr_head_nr, old_gm);
+		pekwm::rootWo()->getHeadInfoWithEdge(head_nr, new_gm);
+	}
 
-	// Ensure the window fits in the new head.
 	_gm.x = new_gm.x + (_gm.x - old_gm.x);
 	_gm.y = new_gm.y + (_gm.y - old_gm.y);
+
+	// Resize if the window is fullscreen or maximized
+	Client *client = getActiveClient();
+	if (isFullscreen() || (client && client->isMaximizedHorz())) {
+		_gm.width = new_gm.width;
+	}
+	if (isFullscreen() || (client && client->isMaximizedVert())) {
+		_gm.height = new_gm.height;
+	}
+
+	// Ensure the window fits in the new head.
 	_gm.width = std::min(_gm.width, new_gm.width);
 	_gm.height = std::min(_gm.height, new_gm.height);
 
@@ -1529,7 +1162,8 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
 	XSizeHints hints = _client->getActiveSizeHints();
 
 	Geometry head;
-	pekwm::rootWo()->getHeadInfoWithEdge(X11Util::getNearestHead(*this), head);
+	pekwm::rootWo()->getHeadInfoWithEdge(X11Util::getNearestHead(*this),
+					     head);
 
 	int max_x, max_r, max_y, max_b;
 	max_x = head.x;
@@ -1604,8 +1238,10 @@ Frame::setStateMaximized(StateAction sa, bool horz, bool vert, bool fill)
 		_client->setMaximizedVert(_maximized_vert);
 	}
 
-	fixGeometry(); // harbour already considered
-	downSize(_gm, true, true); // keep x and keep y ( make conform to inc size )
+	// harbour already considered
+	fixGeometry();
+	// keep x and keep y ( make conform to inc size )
+	downSize(_gm, true, true);
 
 	moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
@@ -1628,10 +1264,16 @@ Frame::setStateFullscreen(StateAction sa)
 	bool lock = _client->setConfigureRequestLock(true);
 
 	if (_fullscreen) {
-		if ((_non_fullscreen_decor_state&DECOR_BORDER) != hasBorder()) {
+		bool state_has_border =
+			(_non_fullscreen_decor_state&DECOR_BORDER)
+			== DECOR_BORDER;
+		if (state_has_border != hasBorder()) {
 			setBorder(STATE_TOGGLE);
 		}
-		if ((_non_fullscreen_decor_state&DECOR_TITLEBAR) != hasTitlebar()) {
+		bool state_has_titlebar =
+			(_non_fullscreen_decor_state&DECOR_TITLEBAR)
+			== DECOR_TITLEBAR;
+		if (state_has_titlebar != hasTitlebar()) {
 			setTitlebar(STATE_TOGGLE);
 		}
 		_gm = _old_gm;
@@ -1657,8 +1299,11 @@ Frame::setStateFullscreen(StateAction sa)
 	moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 
 	// Re-stack window if fullscreen is above other windows.
-	if (pekwm::config()->isFullscreenAbove() && _client->getLayer() != LAYER_DESKTOP) {
-		setLayer(_fullscreen ? LAYER_ABOVE_DOCK : _non_fullscreen_layer);
+	if (pekwm::config()->isFullscreenAbove()
+	    && _client->getLayer() != LAYER_DESKTOP) {
+		setLayer(_fullscreen
+				? LAYER_ABOVE_DOCK
+				: _non_fullscreen_layer);
 		raise();
 	}
 
@@ -1668,6 +1313,12 @@ Frame::setStateFullscreen(StateAction sa)
 	_client->updateEwmhStates();
 }
 
+void
+Frame::clearMaximizedStates(void)
+{
+	clearMaximizedStatesAfterResize();
+}
+
 void
 Frame::setStateSticky(StateAction sa)
 {
@@ -1714,10 +1365,8 @@ Frame::setStateDecorBorder(StateAction sa)
 {
 	bool border = hasBorder();
 
-	setBorder(sa);
-
 	// state changed, update client and atom state
-	if (border != hasBorder()) {
+	if (border != setBorder(sa)) {
 		_client->setBorder(hasBorder());
 
 		// update the _PEKWM_FRAME_DECOR hint
@@ -1733,10 +1382,8 @@ Frame::setStateDecorTitlebar(StateAction sa)
 {
 	bool titlebar = hasTitlebar();
 
-	setTitlebar(sa);
-
 	// state changed, update client and atom state
-	if (titlebar != hasTitlebar()) {
+	if (titlebar != setTitlebar(sa)) {
 		_client->setTitlebar(hasTitlebar());
 
 		X11::setCardinal(_client->getWindow(), PEKWM_FRAME_DECOR,
@@ -1863,28 +1510,36 @@ Frame::getMaxBounds(int &max_x,int &max_r, int &max_y, int &max_b)
 		r = (*it)->getRX();
 		b = (*it)->getBY();
 
-		// update max borders when other frame border lies between
-		// this border and prior max border (originally screen/head edge)
-		if ((r >= max_x) && (r <= _gm.x) && ! ((y >= f_b) || (b <= _gm.y))) {
+		// update max borders when other frame border lies between this
+		// border and prior max border (originally screen/head edge)
+		if ((r >= max_x)
+		    && (r <= _gm.x)
+		    && ! ((y >= f_b) || (b <= _gm.y))) {
 			max_x = r;
 		}
-		if ((x <= max_r) && (x >= f_r) && ! ((y >= f_b) || (b <= _gm.y))) {
+		if ((x <= max_r)
+		    && (x >= f_r)
+		    && ! ((y >= f_b) || (b <= _gm.y))) {
 			max_r = x;
 		}
-		if ((b >= max_y) && (b <= _gm.y) && ! ((x >= f_r) || (r <= _gm.x))) {
+		if ((b >= max_y)
+		    && (b <= _gm.y)
+		    && ! ((x >= f_r) || (r <= _gm.x))) {
 			max_y = b;
 		}
-		if ((y <= max_b) && (y >= f_b) && ! ((x >= f_r) || (r <= _gm.x))) {
+		if ((y <= max_b)
+		    && (y >= f_b)
+		    && ! ((x >= f_r) || (r <= _gm.x))) {
 			max_b = y;
 		}
 	}
 }
 
 void
-Frame::setGeometry(const std::string geometry, int head, bool honour_strut)
+Frame::setGeometry(const std::string& geometry, int head, bool honour_strut)
 {
 	Geometry gm;
-	int mask = X11::parseGeometry(geometry.c_str(), gm);
+	int mask = X11::parseGeometry(geometry, gm);
 	if (! mask) {
 		return;
 	}
@@ -1911,7 +1566,8 @@ void
 Frame::growDirection(uint direction)
 {
 	Geometry head;
-	pekwm::rootWo()->getHeadInfoWithEdge(X11Util::getNearestHead(*this), head);
+	pekwm::rootWo()->getHeadInfoWithEdge(X11Util::getNearestHead(*this),
+					     head);
 
 	switch (direction) {
 	case DIRECTION_UP:
@@ -1932,7 +1588,8 @@ Frame::growDirection(uint direction)
 		break;
 	}
 
-	downSize(_gm, (direction != DIRECTION_LEFT), (direction != DIRECTION_UP));
+	downSize(_gm,
+		 (direction != DIRECTION_LEFT), (direction != DIRECTION_UP));
 
 	moveResize(_gm.x, _gm.y, _gm.width, _gm.height);
 }
@@ -2004,8 +1661,9 @@ Frame::readAutoprops(ApplyOn type)
 		}
 	}
 	if (data->isMask(AP_WORKSPACE)) {
-		// In order to avoid eternal recursion, the workspace is only set here
-		// and then actually called PDecor::setWorkspace from Frame::setWorkspace
+		// In order to avoid eternal recursion, the workspace is only
+		// set here and then actually called PDecor::setWorkspace from
+		// Frame::setWorkspace
 		if (type == APPLY_ON_WORKSPACE) {
 			_workspace = data->workspace;
 		}	else if (_workspace != data->workspace) {
@@ -2159,24 +1817,29 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
 	}
 
 	// Update the stacking (ev->value_mask&CWSibling should not happen)
-	if (! client->isCfgDeny(CFG_DENY_STACKING) && ev->value_mask&CWStackMode) {
+	if (! client->isCfgDeny(CFG_DENY_STACKING)
+	    && ev->value_mask&CWStackMode) {
 		if (ev->detail == Above) {
 			raise();
 		} else if (ev->detail == Below) {
 			lower();
-		} // ignore TopIf, BottomIf, Opposite - PekWM is a reparenting WM
+		} // ignore TopIf, BottomIf, Opposite - PekWM is a reparenting
+		  // WM
 	}
 
 	// Update the geometry if requested
-	bool chg_size = ! _client->isCfgDeny(CFG_DENY_SIZE) && (ev->value_mask&(CWWidth|CWHeight));
-	bool chg_pos  = ! _client->isCfgDeny(CFG_DENY_POSITION) && (ev->value_mask&(CWX|CWY));
+	bool chg_size = ! _client->isCfgDeny(CFG_DENY_SIZE)
+		&& (ev->value_mask&(CWWidth|CWHeight));
+	bool chg_pos  = ! _client->isCfgDeny(CFG_DENY_POSITION)
+		&& (ev->value_mask&(CWX|CWY));
 	if (! (chg_size || chg_pos)) {
 		_client->configureRequestSend();
 		return;
 	}
 
 	if (pekwm::config()->isFullscreenDetect()
-	    && (ev->value_mask&(CWX|CWY|CWWidth|CWHeight)) == (CWX|CWY|CWWidth|CWHeight)
+	    && (ev->value_mask&(CWX|CWY|CWWidth|CWHeight))
+		== (CWX|CWY|CWWidth|CWHeight)
 	    && isRequestGeometryFullscreen(ev)) {
 		if (isFullscreen()) {
 			_client->configureRequestSend();
@@ -2195,8 +1858,8 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
 		gm.height += diff_h;
 
 		if (!chg_pos) {
-			setGravityPosition(_client->getActiveSizeHints().win_gravity,
-					   gm.x, gm.y, diff_w, diff_h);
+			int grav = _client->getActiveSizeHints().win_gravity;
+			setGravityPosition(grav, gm.x, gm.y, diff_w, diff_h);
 		}
 	}
 
@@ -2222,7 +1885,8 @@ Frame::handleConfigureRequest(XConfigureRequestEvent *ev, Client *client)
 bool
 Frame::isRequestGeometryFullscreen(XConfigureRequestEvent *ev)
 {
-	if (_client->isCfgDeny(CFG_DENY_SIZE) || _client->isCfgDeny(CFG_DENY_POSITION)
+	if (_client->isCfgDeny(CFG_DENY_SIZE)
+	    || _client->isCfgDeny(CFG_DENY_POSITION)
 	    || ! _client->allowFullscreen()) {
 		return false;
 	}
@@ -2262,16 +1926,18 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
 				activateChild(client);
 			}
 
-			// If we aren't mapped we check if we make sure we're on the right
-			// workspace and then map the window.
+			// If we aren't mapped we check if we make sure we're
+			// on the right workspace and then map the window.
 			if (! _mapped) {
-				if (_workspace != Workspaces::getActive() &&
-				    !isSticky()) {
-					Workspaces::setWorkspace(_workspace, false);
+				if (_workspace != Workspaces::getActive()
+				    && !isSticky()) {
+					Workspaces::setWorkspace(_workspace,
+								 false);
 				}
 				mapWindow();
 			}
-			// Seems as if raising the window is implied in activating it
+			// Seems as if raising the window is implied in
+			// activating it
 			raise();
 			giveInputFocus();
 		}
@@ -2290,32 +1956,41 @@ Frame::handleClientMessage(XClientMessageEvent *ev, Client *client)
 		   && ev->format == 32) {
 		switch (ev->data.l[2]) {
 		case NET_WM_MOVERESIZE_SIZE_TOPLEFT:
-			doResize(true, true, true, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_TOP_LEFT);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_TOP:
-			doResize(false, false, true, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_TOP);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_TOPRIGHT:
-			doResize(false, true, true, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_TOP_RIGHT);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_RIGHT:
-			doResize(false, true, false, false);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_RIGHT);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_BOTTOMRIGHT:
-			doResize(false, true, false, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_BOTTOM_RIGHT);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_BOTTOM:
-			doResize(false, false, false, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_BOTTOM);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_BOTTOMLEFT:
-			doResize(true, true, false, true);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_BOTTOM_LEFT);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_LEFT:
-			doResize(true, true, false, false);
+			ae = &_ae_resize;
+			ae->action_list[0].setParamI(0, BORDER_LEFT);
 			break;
 		case NET_WM_MOVERESIZE_MOVE:
 			ae = &_ae_move;
-			ae->action_list[0].setParamI(ev->data.l[0], ev->data.l[1]);
+			ae->action_list[0].setParamI(ev->data.l[0],
+						     ev->data.l[1]);
 			break;
 		case NET_WM_MOVERESIZE_SIZE_KEYBOARD:
 		case NET_WM_MOVERESIZE_MOVE_KEYBOARD:
@@ -2376,7 +2051,8 @@ Frame::handleStateAtom(StateAction sa, Atom atom, Client *client)
 		break;
 	case STATE_DEMANDS_ATTENTION:
 		bool ostate = client->demandsAttention();
-		bool nstate = (sa == STATE_SET || (sa == STATE_TOGGLE && ! ostate));
+		bool nstate = (sa == STATE_SET
+			       || (sa == STATE_TOGGLE && ! ostate));
 		client->setDemandsAttention(nstate);
 		if (ostate != nstate) {
 			if (ostate) {
@@ -2447,8 +2123,12 @@ Frame::handlePropertyChange(XPropertyEvent *ev, Client *client)
 	} else if (ev->atom == X11::getAtom(MOTIF_WM_HINTS)) {
 		client->readMwmHints();
 		if (! isFullscreen() && _client == client) {
-			setBorder(_client->hasBorder()?STATE_SET:STATE_UNSET);
-			setTitlebar(_client->hasTitlebar()?STATE_SET:STATE_UNSET);
+			setBorder(_client->hasBorder()
+					? STATE_SET
+					: STATE_UNSET);
+			setTitlebar(_client->hasTitlebar()
+					? STATE_SET
+					: STATE_UNSET);
 		}
 	} else if (ev->atom == XA_WM_NORMAL_HINTS) {
 		client->getWMNormalHints();
