@@ -1,5 +1,6 @@
 //
 // PDecor.cc for pekwm
+// Copyright (C) 2021-2023 Claes Nästén <pekdon@gmail.com>
 // Copyright (C) 2004-2020 the pekwm development team
 //
 // This program is licensed under the GNU GPL.
@@ -27,6 +28,7 @@ extern "C" {
 #include "PTexturePlain.hh" // PTextureSolid
 #include "ActionHandler.hh"
 #include "ManagerWindows.hh"
+#include "PPixmapSurface.hh"
 #include "StatusWindow.hh"
 #include "KeyGrabber.hh"
 #include "Theme.hh"
@@ -38,7 +40,7 @@ extern "C" {
 
 //! @brief PDecor::Button constructor
 PDecor::Button::Button(PWinObj *parent, Theme::PDecorButtonData *data,
-                       uint width, uint height)
+		       uint width, uint height)
 	: PWinObj(true),
 	  _data(data), _state(BUTTON_STATE_UNFOCUSED),
 	  _left(_data->isLeft())
@@ -48,14 +50,10 @@ PDecor::Button::Button(PWinObj *parent, Theme::PDecorButtonData *data,
 	_gm.width = width;
 	_gm.height = height;
 
-	XSetWindowAttributes attr;
-	attr.event_mask = EnterWindowMask|LeaveWindowMask;
-	attr.override_redirect = True;
-	_window =
-		X11::createWindow(_parent->getWindow(),
-				  -_gm.width, -_gm.height, _gm.width, _gm.height, 0,
-				  CopyFromParent, InputOutput, CopyFromParent,
-				  CWEventMask|CWOverrideRedirect, &attr);
+	_window = X11::createWmWindow(_parent->getWindow(),
+				      -_gm.width, -_gm.height,
+				      _gm.width, _gm.height, InputOutput,
+				      EnterWindowMask|LeaveWindowMask);
 	setState(_state);
 }
 
@@ -98,7 +96,8 @@ PDecor::Button::setState(ButtonState state)
 			bool need_free;
 			Pixmap shape = tex->getMask(0, 0, need_free);
 			if (shape != None) {
-				X11::shapeSetMask(_window, ShapeBounding, shape);
+				X11::shapeSetMask(_window, ShapeBounding,
+						  shape);
 				if (need_free) {
 					X11::freePixmap(shape);
 				}
@@ -191,17 +190,13 @@ std::vector<PDecor*> PDecor::_pdecors;
 //! @param dpy Display
 //! @param decor_name String, if not DEFAULT_DECOR_NAME sets _decor_name_saved
 PDecor::PDecor(const std::string &decor_name, const Window child_window,
-               bool init)
+	       bool init)
 	: PWinObj(true),
 	  ThemeGm(nullptr),
 	  _decor_name(decor_name),
 	  _child(0),
 	  _button(0),
 	  _button_press_win(None),
-	  _pointer_x(0),
-	  _pointer_y(0),
-	  _click_x(0),
-	  _click_y(0),
 	  _decor_cfg_child_move_overloaded(false),
 	  _decor_cfg_bpr_replay_pointer(false),
 	  _decor_cfg_bpr_al_child(MOUSE_ACTION_LIST_CHILD_OTHER),
@@ -267,15 +262,15 @@ PDecor::createParentWindow(CreateWindowParams &params, Window child_window)
 
 	if (child_window != None) {
 		XWindowAttributes attr;
-		if (X11::getWindowAttributes(child_window, &attr)
+		if (X11::getWindowAttributes(child_window, attr)
 		    && 32 != X11::getDepth()
 		    && attr.depth == 32) {
 			params.mask |= CWColormap;
 			params.depth = attr.depth;
 			params.visual = attr.visual;
-			params.attr.colormap = XCreateColormap(X11::getDpy(),
-							       X11::getRoot(),
-							       params.visual, AllocNone);
+			params.attr.colormap =
+				XCreateColormap(X11::getDpy(), X11::getRoot(),
+						params.visual, AllocNone);
 		}
 	}
 	params.attr.event_mask = ButtonPressMask|ButtonReleaseMask|
@@ -304,8 +299,9 @@ PDecor::createTitle(CreateWindowParams &params)
 		ButtonMotionMask|EnterWindowMask;
 	Window title = X11::createWindow(_window,
 					 bdLeft(this), bdTop(this), 1, 1, 0,
-					 params.depth, InputOutput, params.visual,
-					 params.mask, &params.attr);
+					 params.depth, InputOutput,
+					 params.visual, params.mask,
+					 &params.attr);
 	_title_wo.setWindow(title);
 	addChildWindow(_title_wo.getWindow());
 	_title_wo.mapWindow();
@@ -325,8 +321,9 @@ PDecor::createBorder(CreateWindowParams &params)
 
 		_border_win[i] =
 			X11::createWindow(_window, -1, -1, 1, 1, 0,
-					  params.depth, InputOutput, params.visual,
-					  params.mask|CWCursor, &params.attr);
+					  params.depth, InputOutput,
+					  params.visual, params.mask|CWCursor,
+					  &params.attr);
 		addChildWindow(_border_win[i]);
 	}
 }
@@ -338,7 +335,7 @@ PDecor::~PDecor(void)
 		       _pdecors.end());
 
 	while (! _children.empty()) {
-		removeChild(_children.back(), false); // Don't call delete this.
+		PDecor::removeChild(_children.back(), false);
 	}
 
 	// Make things look smoother, buttons will be noticed as deleted
@@ -410,7 +407,8 @@ PDecor::move(int x, int y)
 	// update real position
 	PWinObj::move(x, y);
 	if (_child && (_decor_cfg_child_move_overloaded)) {
-		_child->move(x + bdLeft(this), y + bdTop(this) + titleHeight(this));
+		_child->move(x + bdLeft(this),
+			     y + bdTop(this) + titleHeight(this));
 	}
 }
 
@@ -497,14 +495,16 @@ PDecor::moveResize(int x, int y, uint width, uint height)
 	// Update size before moving and shaping the rest as shaping
 	// depends on the child window
 	if (_child) {
-		_child->moveResize(x + bdLeft(this), y + bdTop(this) + titleHeight(this),
+		_child->moveResize(x + bdLeft(this),
+				   y + bdTop(this) + titleHeight(this),
 				   getChildWidth(), getChildHeight());
 
-		// The client window may have its window gravity set to something different
-		// than NorthWestGravity (see Xlib manual chapter 3.2.3). Therefore the
-		// X server may not move its top left corner along with the decoration.
-		// We correct these cases by calling alignChild(). It is called only for
-		// _child and not all members of _children, because activateChild.*()
+		// The client window may have its window gravity set to
+		// something different than NorthWestGravity (see Xlib manual
+		// chapter 3.2.3). Therefore the X server may not move its top
+		// left corner along with the decoration. We correct these
+		// cases by calling alignChild(). It is called only for _child
+		// and not all members of _children, because activateChild.*()
 		// does it itself.
 		alignChild(_child);
 	}
@@ -573,8 +573,9 @@ PDecor::setWorkspace(uint workspace)
 {
 	if (workspace != NET_WM_STICKY_WINDOW) {
 		if (workspace >= Workspaces::size()) {
-			P_LOG("this == " << this << " workspace == " << workspace
-			      << " >= number of workspaces == " << Workspaces::size());
+			P_LOG("this == " << this << " workspace == "
+			      << workspace << " >= number of workspaces == "
+			      << Workspaces::size());
 			workspace = Workspaces::size() - 1;
 		}
 		_workspace = workspace;
@@ -612,61 +613,52 @@ PDecor::giveInputFocus(void)
 ActionEvent*
 PDecor::handleButtonPress(XButtonEvent *ev)
 {
-	ActionEvent *ae = 0;
-	std::vector<ActionEvent> *actions = 0;
-
-	// Remove state modifiers from event
 	X11::stripStateModifiers(&ev->state);
 	X11::stripButtonModifiers(&ev->state);
 
 	// Try to do something about frame buttons
-	if (ev->subwindow != None && (_button = findButton(ev->subwindow)) != 0) {
-		ae = handleButtonPressButton(ev, _button);
+	_button = findButton(ev->subwindow);
+	if (_button) {
+		return handleButtonPressButton(ev, _button);
+	}
+	return handleButtonPressDecor(ev);
+}
 
+ActionEvent*
+PDecor::handleButtonPressDecor(XButtonEvent *ev)
+{
+	// Allow us to get clicks from anywhere on the window.
+	if (_decor_cfg_bpr_replay_pointer && ev->window == _window) {
+		X11::allowEvents(ReplayPointer, CurrentTime);
+	}
+
+	std::vector<ActionEvent> *actions = nullptr;
+	Config *cfg = pekwm::config();
+	if (ev->window == _child->getWindow()
+	    || (ev->state == 0 && ev->subwindow == _child->getWindow())) {
+		// press on the child window (not the actual decor), if the
+		// match was against the subwindow the check against zero
+		// state is required to avoid duplicate actions to trigger.
+		_button_press_win = _child->getWindow();
+		actions = cfg->getMouseActionList(_decor_cfg_bpr_al_child);
+	} else if (_title_wo == ev->window) {
+		// press on the decor title
+		_button_press_win = ev->window;
+		actions = cfg->getMouseActionList(_decor_cfg_bpr_al_title);
 	} else {
-		// Record position, used in the motion handler when doing a window move.
-		_click_x = _gm.x;
-		_click_y = _gm.y;
-		_pointer_x = ev->x_root;
-		_pointer_y = ev->y_root;
-
-		// Allow us to get clicks from anywhere on the window.
-		if (_decor_cfg_bpr_replay_pointer && ev->window == _window) {
-			X11::allowEvents(ReplayPointer, CurrentTime);
-		}
-
-		Config *cfg = pekwm::config();
-		if (ev->window == _child->getWindow()
-		    || (ev->state == 0 && ev->subwindow == _child->getWindow())) {
-			// Clicks on the child window
-			// NOTE: If the we're matching against the subwindow we need to make
-			// sure that the state is 0, meaning we didn't have any modifier
-			// because if we don't care about the modifier we'll get two actions
-			// performed when using modifiers.
-			_button_press_win = _child->getWindow();
-			actions =
-				cfg->getMouseActionList(_decor_cfg_bpr_al_child);
-		} else if (_title_wo == ev->window) {
-			// Clicks on the decor title
+		// press on decor border, default case. Try both window and
+		// sub-window.
+		uint pos = getBorderPosition(ev->window);
+		if (pos != BORDER_NO_POS) {
 			_button_press_win = ev->window;
-			actions = cfg->getMouseActionList(_decor_cfg_bpr_al_title);
-		} else {
-			// Clicks on the decor border, default case. Try both
-			// window and sub-window.
-			uint pos = getBorderPosition(ev->window);
-			if (pos != BORDER_NO_POS) {
-				_button_press_win = ev->window;
-				actions = cfg->getBorderListFromPosition(pos);
-			}
+			actions = cfg->getBorderListFromPosition(pos);
 		}
 	}
 
-	if (! ae && actions) {
-		ae = ActionHandler::findMouseAction(ev->button, ev->state,
-						    MOUSE_EVENT_PRESS, actions);
-	}
-
-	return ae;
+	return actions
+		? ActionHandler::findMouseAction(ev->button, ev->state,
+						 MOUSE_EVENT_PRESS, actions)
+		: nullptr;
 }
 
 /**
@@ -683,10 +675,11 @@ PDecor::handleButtonPressButton(XButtonEvent *ev, PDecor::Button *button)
 	// if the button is used for resizing, we don't want to wait for release
 	if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
 		_button->setState(_focused
-				  ? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
-		_button = 0;
+				  ? BUTTON_STATE_FOCUSED
+				  : BUTTON_STATE_UNFOCUSED);
+		_button = nullptr;
 	} else {
-		ae = 0;
+		ae = nullptr;
 	}
 
 	return ae;
@@ -698,66 +691,61 @@ PDecor::handleButtonPressButton(XButtonEvent *ev, PDecor::Button *button)
 ActionEvent*
 PDecor::handleButtonRelease(XButtonEvent *ev)
 {
-	ActionEvent *ae = 0;
-	std::vector<ActionEvent> *actions = 0;
-	MouseEventType mb = MOUSE_EVENT_RELEASE;
-
 	// Remove state modifiers from event
 	X11::stripStateModifiers(&ev->state);
 	X11::stripButtonModifiers(&ev->state);
 
 	// handle titlebar buttons
 	if (_button) {
-		ae = handleButtonReleaseButton(ev, _button);
+		return handleButtonReleaseButton(ev, _button);
+	}
+	return handleButtonReleaseDecor(ev);
+}
 
-	} else {
-		// Allow us to get clicks from anywhere on the window.
-		if (_decor_cfg_bpr_replay_pointer && ev->window == _window) {
-			X11::allowEvents(ReplayPointer, CurrentTime);
-		}
+ActionEvent*
+PDecor::handleButtonReleaseDecor(XButtonEvent *ev)
+{
+	// ensure that the button was released on the same window as it
+	// was pressed on to allow button release events to be triggered
+	if (ev->window != _button_press_win
+	    && ev->subwindow != _button_press_win) {
+		return nullptr;
+	}
 
-		// clicks on the child window
-		Config *cfg = pekwm::config();
-		if (ev->window == _child->getWindow()
-		    || (ev->state == 0 && ev->subwindow == _child->getWindow())) {
-			// NOTE: If the we're matching against the subwindow we need to make
-			// sure that the state is 0, meaning we didn't have any modifier
-			// because if we don't care about the modifier we'll get two actions
-			// performed when using modifiers.
-			if (_button_press_win == _child->getWindow()) {
-				actions = cfg->getMouseActionList(_decor_cfg_bpr_al_child);
-			}
-
-		} else if (_title_wo == ev->window) {
-			if (_button_press_win == ev->window) {
-				// Handle clicks on the decor title, checking double
-				// clicks first.
-				if (X11::isDoubleClick(ev->window, ev->button - 1, ev->time,
-						       cfg->getDoubleClickTime())) {
-					X11::setLastClickID(ev->window);
-					X11::setLastClickTime(ev->button - 1, 0);
-					mb = MOUSE_EVENT_DOUBLE;
-				} else {
-					X11::setLastClickID(ev->window);
-					X11::setLastClickTime(ev->button - 1, ev->time);
-				}
-
-				actions = cfg->getMouseActionList(_decor_cfg_bpr_al_title);
-			}
+	MouseEventType mb = MOUSE_EVENT_RELEASE;
+	std::vector<ActionEvent> *actions = nullptr;
+	Config *cfg = pekwm::config();
+	if (ev->window == _child->getWindow()
+	    || (ev->state == 0 && ev->subwindow == _child->getWindow())) {
+		// release on the child window (not the actual decor), if the
+		// match was against the subwindow the check against zero
+		// state is required to avoid duplicate actions to trigger.
+		actions = cfg->getMouseActionList(_decor_cfg_bpr_al_child);
+	} else if (_title_wo == ev->window) {
+		// handle release on the decor title, checking double clicks
+		// first.
+		if (X11::isDoubleClick(ev->window, ev->button - 1, ev->time,
+				       cfg->getDoubleClickTime())) {
+			X11::setLastClickID(ev->window);
+			X11::setLastClickTime(ev->button - 1, 0);
+			mb = MOUSE_EVENT_DOUBLE;
 		} else {
-			// Clicks on the decor border, check subwindow then window.
-			uint pos = getBorderPosition(ev->window);
-			if (pos != BORDER_NO_POS && (_button_press_win == ev->window)) {
-				actions = cfg->getBorderListFromPosition(pos);
-			}
+			X11::setLastClickID(ev->window);
+			X11::setLastClickTime(ev->button - 1, ev->time);
+		}
+		actions = cfg->getMouseActionList(_decor_cfg_bpr_al_title);
+	} else {
+		// release on the decor border, check subwindow then window.
+		uint pos = getBorderPosition(ev->window);
+		if (pos != BORDER_NO_POS && (_button_press_win == ev->window)) {
+			actions = cfg->getBorderListFromPosition(pos);
 		}
 	}
 
-	if (! ae && actions) {
-		ae = ActionHandler::findMouseAction(ev->button, ev->state, mb, actions);
-	}
-
-	return ae;
+	return actions
+		? ActionHandler::findMouseAction(ev->button, ev->state,
+						 mb, actions)
+		: nullptr;
 }
 
 /**
@@ -767,9 +755,11 @@ ActionEvent*
 PDecor::handleButtonReleaseButton(XButtonEvent *ev, PDecor::Button *button)
 {
 	// First restore the pressed buttons state
-	_button->setState(_focused ? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
+	_button->setState(_focused
+			  ? BUTTON_STATE_FOCUSED
+			  : BUTTON_STATE_UNFOCUSED);
 
-	ActionEvent *ae = 0;
+	ActionEvent *ae = nullptr;
 
 	// Then see if the button was released over ( to execute an action )
 	if (*_button == ev->subwindow) {
@@ -777,11 +767,11 @@ PDecor::handleButtonReleaseButton(XButtonEvent *ev, PDecor::Button *button)
 
 		// This is a hack to avoid resizing on both press and release
 		if (ae && ae->isOnlyAction(ACTION_RESIZE)) {
-			ae = 0;
+			ae = nullptr;
 		}
 	}
 
-	_button = 0;
+	_button = nullptr;
 
 	return ae;
 }
@@ -921,12 +911,15 @@ PDecor::loadDecor(void)
 	for (; b_it != _data->buttonEnd(); ++b_it) {
 		uint width = std::max(static_cast<uint>(1),
 				      (*b_it)->getWidth()
-				      ? (*b_it)->getWidth() : titleHeight(this));
+				      ? (*b_it)->getWidth()
+				      : titleHeight(this));
 		uint height = std::max(static_cast<uint>(1),
 				       (*b_it)->getHeight()
-				       ? (*b_it)->getHeight() : titleHeight(this));
+				       ? (*b_it)->getHeight()
+				       : titleHeight(this));
 
-		_buttons.push_back(new PDecor::Button(&_title_wo, *b_it, width, height));
+		_buttons.push_back(new PDecor::Button(&_title_wo, *b_it,
+						      width, height));
 		_buttons.back()->mapWindow();
 		addChildWindow(_buttons.back()->getWindow());
 	}
@@ -997,6 +990,10 @@ PDecor::unloadDecor(void)
 PDecor::Button*
 PDecor::findButton(Window win)
 {
+	if (win == None) {
+		return nullptr;
+	}
+
 	std::vector<PDecor::Button*>::iterator it = _buttons.begin();
 	for (; it != _buttons.end(); ++it) {
 		if (*(*it) == win) {
@@ -1084,11 +1081,11 @@ PDecor::resizeChild(uint width, uint height)
 }
 
 //! @brief Sets border state of the decor
-void
+bool
 PDecor::setBorder(StateAction sa)
 {
 	if (! ActionUtil::needToggle(sa, _border)) {
-		return;
+		return _border;
 	}
 
 	// If we are going to remove the border, we need to check carefully
@@ -1107,14 +1104,17 @@ PDecor::setBorder(StateAction sa)
 		resizeChild(_child->getWidth(), _child->getHeight());
 	}
 	decorUpdated();
+
+	return _border;
 }
 
 //! @brief Sets titlebar state of the decor
-void
+bool
 PDecor::setTitlebar(StateAction sa)
 {
-	if (! ActionUtil::needToggle(sa, _titlebar))
-		return;
+	if (! ActionUtil::needToggle(sa, _titlebar)) {
+		return _titlebar;
+	}
 
 	// If we are going to remove the titlebar, we need to check carefully
 	// that we don't try to make the window disappear.
@@ -1134,6 +1134,8 @@ PDecor::setTitlebar(StateAction sa)
 		resizeChild(_child->getWidth(), _child->getHeight());
 	}
 	decorUpdated();
+
+	return _titlebar;
 }
 
 //! @brief Adds a child to the decor, reparenting the window
@@ -1172,7 +1174,7 @@ PDecor::removeChild(PWinObj *child, bool do_delete)
 			_gm.y + bdTop(this) + titleHeight(this));
 
 	std::vector<PWinObj*>::iterator it
-		= find(_children.begin(), _children.end(), child);
+		= std::find(_children.begin(), _children.end(), child);
 	if (it == _children.end()) {
 		P_LOG("this == " << this << " child == " << child
 		      << " - child not in _child_list, bailing out");
@@ -1217,7 +1219,7 @@ PDecor::activateChild(PWinObj *child)
 void
 PDecor::getDecorInfo(char *buf, uint size, const Geometry& gm)
 {
-	snprintf(buf, size, "%dx%d+%d+%d", gm.width, gm.height, gm.x, gm.y);
+	snprintf(buf, size, "%ux%u+%d+%d", gm.width, gm.height, gm.x, gm.y);
 }
 
 void
@@ -1235,21 +1237,21 @@ PDecor::activateChildNum(uint num)
 void
 PDecor::activateChildRel(int off)
 {
-	std::vector<PWinObj*>::size_type cur=0, size = _children.size();
-	for (; cur < size; ++cur) {
+	std::vector<PWinObj*>::size_type cur=0, num_children = _children.size();
+	for (; cur < num_children; ++cur) {
 		if (_child == _children[cur]) {
 			break;
 		}
 	}
 
-	if (cur == size) {
+	if (cur == num_children) {
 		_child = _children.front();
 		cur = 0;
 	}
 
-	off = (off+signed(cur))%signed(size);
+	off = (off+signed(cur))%signed(num_children);
 	if (off < 0) {
-		off += size;
+		off += num_children;
 	}
 	activateChild(_children[off]);
 }
@@ -1259,51 +1261,54 @@ PDecor::activateChildRel(int off)
 void
 PDecor::moveChildRel(int off)
 {
-	std::vector<PWinObj*>::size_type idx=0, size = _children.size();
-	for (; idx < size; ++idx) {
+	std::vector<PWinObj*>::size_type idx=0, num_children = _children.size();
+	for (; idx < num_children; ++idx) {
 		if (_child == _children[idx]) {
 			break;
 		}
 	}
 
-	if (idx == size) {
+	if (idx == num_children) {
 		if (_children.empty()) {
-			P_ERR("_children is empty! off == " << off << " Please report.");
+			P_ERR("_children is empty! off == " << off
+			      << " Please report.");
 			return;
 		}
 		_child = _children.front();
 		idx = 0;
 	}
 
-	off = (off+idx)%size;
+	off = (off+idx)%num_children;
 	if (off < 0) {
-		off += size;
+		off += num_children;
 	}
 
-	_children.erase(std::remove(_children.begin(), _children.end(), _child),
+	_children.erase(std::remove(_children.begin(), _children.end(),
+				    _child),
 			_children.end());
 	_children.insert(_children.begin()+off, _child);
 	updatedChildOrder();
 }
 
 //! @brief Sets shaded state
-void
+bool
 PDecor::setShaded(StateAction sa)
 {
-	if (! ActionUtil::needToggle(sa, _shaded))
-		return;
+	if (! ActionUtil::needToggle(sa, _shaded)) {
+		return _shaded;
+	}
 
 	// If we are going to shade the window, we need to check carefully
 	// that we don't try to make the window disappear.
 	if (_shaded) {
 		_gm.height = _real_height;
 		_shaded = false;
-		// If we have a titlebar OR border (border will only be visible in
-		// full-width mode only)
+		// If we have a titlebar OR border (border will only be
+		// visible in full-width mode only)
 	} else if (_titlebar || (_border && ! _data->getTitleWidthMin())) {
 		_real_height = _gm.height;
 		_gm.height = titleHeight(this);
-		// shading in non full width title mode will make border go away
+		// Shading in non full width title mode (border not visible)
 		if (! _data->getTitleWidthMin()) {
 			_gm.height += bdTop(this) + bdBottom(this);
 		}
@@ -1314,6 +1319,8 @@ PDecor::setShaded(StateAction sa)
 	placeBorder();
 	restackBorder();
 	PWinObj::resize(_gm.width, _gm.height);
+
+	return _shaded;
 }
 
 //! @brief Sets skip state.
@@ -1367,29 +1374,28 @@ PDecor::renderTitle(void)
 	}
 
 	PTexture *t_sep = _data->getTextureSeparator(getFocusedState(false));
-	Pixmap title_bg = X11::createPixmap(_title_wo.getWidth(),
-					    _title_wo.getHeight());
+	PPixmapSurface title_bg(_title_wo.getWidth(), _title_wo.getHeight());
+
 	// Render main background on pixmap
 	PTexture *t_main = _data->getTextureMain(getFocusedState(false));
-	t_main->render(title_bg, 0, 0,
+	t_main->render(&title_bg, 0, 0,
 		       _title_wo.getWidth(), _title_wo.getHeight());
 
-	PFont *font;
-	bool sel; // Current tab selected flag
 	uint x = _titles_left; // Position
 	// Amount of horizontal padding
 	uint pad_horiz =  _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
 
-	uint size = _titles.size();
-	for (uint i = 0; i < size; ++i) {
-		sel = (_title_active == i);
+	uint num_titles = _titles.size();
+	for (uint i = 0; i < num_titles; ++i) {
+		// Current tab selected flag
+		bool sel = (_title_active == i);
 
 		// render tab
 		PTexture *tab = _data->getTextureTab(getFocusedState(sel));
-		tab->render(title_bg, x, 0,
+		tab->render(&title_bg, x, 0,
 			    _titles[i]->getWidth(), _title_wo.getHeight());
 
-		font = getFont(getFocusedState(sel));
+		PFont *font = getFont(getFocusedState(sel));
 		font->setColor(_data->getFontColor(getFocusedState(sel)));
 
 		PFont::TrimType trim = PFont::FONT_TRIM_MIDDLE;
@@ -1397,26 +1403,26 @@ PDecor::renderTitle(void)
 			trim = PFont::FONT_TRIM_END;
 		}
 
-		font->draw(title_bg,
+		font->draw(&title_bg,
 			   x + _data->getPad(PAD_LEFT), // X position
 			   _data->getPad(PAD_UP), // Y position
 			   _titles[i]->getVisible(), 0, // Text and max chars
-			   _titles[i]->getWidth() - pad_horiz, // Available width
+			   _titles[i]->getWidth() - pad_horiz,
 			   trim); // Type of trim
 
 		// move to next tab (or separator if any)
 		x += _titles[i]->getWidth();
 
 		// draw separator
-		if (size > 1 && i < size - 1) {
-			t_sep->render(title_bg, x, 0, 0, 0);
+		if (num_titles > 1 && i < (num_titles - 1)) {
+			t_sep->render(&title_bg, x, 0, 0, 0);
 			x += t_sep->getWidth();
 		}
 	}
 
-	X11::setWindowBackgroundPixmap(_title_wo.getWindow(), title_bg);
+	X11::setWindowBackgroundPixmap(_title_wo.getWindow(),
+				       title_bg.getDrawable());
 	X11::clearWindow(_title_wo.getWindow());
-	X11::freePixmap(title_bg);
 }
 
 void
@@ -1425,7 +1431,8 @@ PDecor::renderButtons(void)
 	std::vector<PDecor::Button*>::iterator it = _buttons.begin();
 	for (; it != _buttons.end(); ++it) {
 		(*it)->setState(_focused
-				? BUTTON_STATE_FOCUSED : BUTTON_STATE_UNFOCUSED);
+				? BUTTON_STATE_FOCUSED
+				: BUTTON_STATE_UNFOCUSED);
 	}
 }
 
@@ -1439,8 +1446,9 @@ PDecor::renderBorder(void)
 	uint width, height;
 	FocusedState state = getFocusedState(false);
 	for (int i=0; i < BORDER_NO_POS; ++i) {
+		BorderPosition bp = static_cast<BorderPosition>(i);
 		PTexture *tex =
-			_data->getBorderTexture(state, static_cast<BorderPosition>(i));
+			_data->getBorderTexture(state, bp);
 		getBorderSize(static_cast<BorderPosition>(i), width, height);
 		tex->setBackground(_border_win[i], 0, 0, width, height);
 		X11::clearWindow(_border_win[i]);
@@ -1464,7 +1472,8 @@ PDecor::setBorderShape(void)
 		// Get the size of the border at position
 		getBorderSize(static_cast<BorderPosition>(i), width, height);
 
-		tex = _data->getBorderTexture(state, static_cast<BorderPosition>(i));
+		tex = _data->getBorderTexture(state,
+					      static_cast<BorderPosition>(i));
 		pix = tex->getMask(width, height, do_free);
 
 		if (pix != None) {
@@ -1509,7 +1518,7 @@ isBetween(int x1, int x2, int t1, int t2)
 
 //! @todo PDecor/PWinObj doesn't have _skip property
 void
-PDecor::checkWOSnap(PWinObj *skip_wo, Geometry &gm)
+PDecor::checkWOSnap(const PWinObj *skip_wo, Geometry &gm)
 {
 	PDecor *decor;
 	Geometry orig_gm = gm;
@@ -1541,27 +1550,32 @@ PDecor::checkWOSnap(PWinObj *skip_wo, Geometry &gm)
 		// check snap
 		if ((x >= ((*it)->getX() - attract))
 		    && (x <= ((*it)->getX() + resist))) {
-			if (isBetween(gm.y, y, (*it)->getY(), (*it)->getBY())) {
+			if (isBetween(gm.y, y,
+				      (*it)->getY(), (*it)->getBY())) {
 				gm.x = (*it)->getX() - orig_gm.width;
 				snapped = true;
 			}
 		} else if ((gm.x >= signed((*it)->getRX() - resist)) &&
 			   (gm.x <= signed((*it)->getRX() + attract))) {
-			if (isBetween(gm.y, y, (*it)->getY(), (*it)->getBY())) {
+			if (isBetween(gm.y, y,
+				      (*it)->getY(), (*it)->getBY())) {
 				gm.x = (*it)->getRX();
 				snapped = true;
 			}
 		}
 
-		if (y >= ((*it)->getY() - attract) && (y <= (*it)->getY() + resist)) {
-			if (isBetween(gm.x, x, (*it)->getX(), (*it)->getRX())) {
+		if (y >= ((*it)->getY() - attract)
+		    && (y <= (*it)->getY() + resist)) {
+			if (isBetween(gm.x, x,
+				      (*it)->getX(), (*it)->getRX())) {
 				gm.y = (*it)->getY() - orig_gm.height;
 				if (snapped)
 					break;
 			}
 		} else if ((gm.y >= signed((*it)->getBY() - resist)) &&
 			   (gm.y <= signed((*it)->getBY() + attract))) {
-			if (isBetween(gm.x, x, (*it)->getX(), (*it)->getRX())) {
+			if (isBetween(gm.x, x,
+				      (*it)->getX(), (*it)->getRX())) {
 				gm.y = (*it)->getBY();
 				if (snapped)
 					break;
@@ -1610,14 +1624,17 @@ PDecor::alignChild(PWinObj *child)
 	}
 }
 
-//! @brief Draws outline of the decor with geometry gm
+/**
+ * Draws outline for the provided geometry, if shaded is non-zero use
+ * that as height instead of gm.height
+ */
 void
-PDecor::drawOutline(const Geometry &gm)
+PDecor::drawOutline(const Geometry &gm, uint shaded)
 {
 	XDrawRectangle(X11::getDpy(), X11::getRoot(),
 		       pekwm::theme()->getInvertGC(),
 		       gm.x, gm.y, gm.width,
-		       _shaded ? _gm.height : gm.height);
+		       shaded ? shaded : gm.height);
 }
 
 //! @brief Places decor buttons
@@ -1651,7 +1668,9 @@ PDecor::placeBorder(void)
 		X11::moveResizeWindow(_border_win[BORDER_TOP],
 				      bdTopLeft(this),
 				      bt_off,
-				      _gm.width - bdTopLeft(this) - bdTopRight(this),
+				      _gm.width
+					  - bdTopLeft(this)
+					  - bdTopRight(this),
 				      bdTop(this));
 
 		X11::moveResizeWindow(_border_win[BORDER_TOP_LEFT],
@@ -1669,7 +1688,9 @@ PDecor::placeBorder(void)
 					      0,
 					      bdTopLeftHeight(this) + bt_off,
 					      bdLeft(this),
-					      _gm.height - bdTopLeftHeight(this) - bdBottomLeftHeight(this));
+					      _gm.height
+						  - bdTopLeftHeight(this)
+						  - bdBottomLeftHeight(this));
 		}
 
 		if (bdRight(this)) {
@@ -1677,7 +1698,9 @@ PDecor::placeBorder(void)
 					      _gm.width - bdRight(this),
 					      bdTopRightHeight(this) + bt_off,
 					      bdRight(this),
-					      _gm.height - bdTopRightHeight(this) - bdBottomRightHeight(this));
+					      _gm.height
+						  - bdTopRightHeight(this)
+						  - bdBottomRightHeight(this));
 		}
 	} else {
 		if (bdLeft(this)) {
@@ -1685,7 +1708,9 @@ PDecor::placeBorder(void)
 					      0,
 					      titleHeight(this),
 					      bdLeft(this),
-					      _gm.height - titleHeight(this) - bdBottom(this));
+					      _gm.height
+						  - titleHeight(this)
+						  - bdBottom(this));
 		}
 
 		if (bdRight(this)) {
@@ -1693,7 +1718,9 @@ PDecor::placeBorder(void)
 					      _gm.width - bdRight(this),
 					      titleHeight(this),
 					      bdRight(this),
-					      _gm.height - titleHeight(this) - bdBottom(this));
+					      _gm.height
+						  - titleHeight(this)
+						  - bdBottom(this));
 		}
 	}
 
@@ -1701,7 +1728,9 @@ PDecor::placeBorder(void)
 		X11::moveResizeWindow(_border_win[BORDER_BOTTOM],
 				      bdBottomLeft(this),
 				      _gm.height - bdBottom(this),
-				      _gm.width - bdBottomLeft(this) - bdBottomRight(this),
+				      _gm.width
+					  - bdBottomLeft(this)
+					  - bdBottomRight(this),
 				      bdBottom(this));
 		X11::moveResizeWindow(_border_win[BORDER_BOTTOM_LEFT],
 				      0,
@@ -1742,12 +1771,13 @@ PDecor::applyBorderShape(int kind)
 void
 PDecor::applyBorderShapeNormal(int kind, bool client_shape)
 {
-	Window shape = X11::createSimpleWindow(X11::getRoot(),
-					       0, 0, _gm.width, _gm.height,
-					       0, 0, 0);
+	Window shape = X11::createWmWindow(X11::getRoot(),
+					   0, 0, _gm.width, _gm.height,
+					   InputOutput, None);
 	if (_child) {
 		X11::shapeCombine(shape, kind,
-				  bdLeft(this), bdTop(this) + titleHeight(this),
+				  bdLeft(this),
+				  bdTop(this) + titleHeight(this),
 				  _child->getWindow(), ShapeSet);
 	}
 
@@ -1780,15 +1810,15 @@ PDecor::applyBorderShapeShaded(int kind)
 {
 	Window shape;
 	if (_data->getTitleWidthMin()) {
-		shape =
-			X11::createSimpleWindow(X11::getRoot(),
-						0, 0,
-						_title_wo.getWidth(), _title_wo.getHeight(),
-						0, 0, 0);
+		shape = X11::createWmWindow(X11::getRoot(),
+					    0, 0,
+					    _title_wo.getWidth(),
+					    _title_wo.getHeight(), InputOutput,
+					    None);
 	} else {
-		shape =
-			X11::createSimpleWindow(X11::getRoot(),
-						0, 0, _gm.width, _gm.height, 0, 0, 0);
+		shape = X11::createWmWindow(X11::getRoot(),
+					    0, 0, _gm.width, _gm.height,
+					    InputOutput, None);
 		if (_border) {
 			applyBorderShapeBorder(kind, shape);
 		}
@@ -1818,15 +1848,17 @@ PDecor::applyBorderShapeBorder(int kind, Window shape)
 	if (bdLeft(this) > 0) {
 		X11::shapeCombine(shape, kind,
 				  0,
-				  use_bt_off ? bd_top_offset + bdTopLeftHeight(this)
-				  : titleHeight(this),
+				  use_bt_off
+				      ? bd_top_offset + bdTopLeftHeight(this)
+				      : titleHeight(this),
 				  _border_win[BORDER_LEFT], ShapeUnion);
 	}
 
 	if (bdRight(this) > 0) {
 		X11::shapeCombine(shape, kind, _gm.width - bdRight(this),
-				  use_bt_off ? bd_top_offset + bdTopRightHeight(this)
-				  : titleHeight(this),
+				  use_bt_off
+				      ? bd_top_offset + bdTopRightHeight(this)
+				      : titleHeight(this),
 				  _border_win[BORDER_RIGHT], ShapeUnion);
 	}
 
@@ -1835,7 +1867,8 @@ PDecor::applyBorderShapeBorder(int kind, Window shape)
 				  0, _gm.height - bdBottomLeftHeight(this),
 				  _border_win[BORDER_BOTTOM_LEFT], ShapeUnion);
 		X11::shapeCombine(shape, kind,
-				  bdBottomLeft(this), _gm.height - bdBottom(this),
+				  bdBottomLeft(this),
+				  _gm.height - bdBottom(this),
 				  _border_win[BORDER_BOTTOM], ShapeUnion);
 		X11::shapeCombine(shape, kind,
 				  _gm.width - bdBottomRight(this),
@@ -1903,8 +1936,11 @@ PDecor::getBorderSize(BorderPosition pos, uint &width, uint &height)
 		height = _data->getBorderTexture(state, pos)->getHeight();
 		break;
 	case BORDER_BOTTOM:
-		if ((bdBottomLeft(this) + bdBottomRight(this)) < _gm.width) {
-			width = _gm.width - bdBottomLeft(this) - bdBottomRight(this);
+		if ((bdBottomLeft(this)
+		     + bdBottomRight(this)) < _gm.width) {
+			width = _gm.width
+				- bdBottomLeft(this)
+				- bdBottomRight(this);
 		} else {
 			width = 1;
 		}
@@ -1912,16 +1948,22 @@ PDecor::getBorderSize(BorderPosition pos, uint &width, uint &height)
 		break;
 	case BORDER_LEFT:
 		width = _data->getBorderTexture(state, pos)->getWidth();
-		if ((bdTopLeftHeight(this) + bdBottomLeftHeight(this)) < _gm.height) {
-			height = _gm.height - bdTopLeftHeight(this) - bdBottomLeftHeight(this);
+		if ((bdTopLeftHeight(this)
+		     + bdBottomLeftHeight(this)) < _gm.height) {
+			height = _gm.height
+				- bdTopLeftHeight(this)
+				- bdBottomLeftHeight(this);
 		} else {
 			height = 1;
 		}
 		break;
 	case BORDER_RIGHT:
 		width = _data->getBorderTexture(state, pos)->getWidth();
-		if ((bdTopRightHeight(this) + bdBottomRightHeight(this)) < _gm.height) {
-			height = _gm.height - bdTopRightHeight(this) - bdBottomRightHeight(this);
+		if ((bdTopRightHeight(this)
+		     + bdBottomRightHeight(this)) < _gm.height) {
+			height = _gm.height
+				- bdTopRightHeight(this)
+				- bdBottomRightHeight(this);
 		} else {
 			height = 1;
 		}
@@ -1956,14 +1998,15 @@ PDecor::calcTitleWidth(void)
 uint
 PDecor::calcTitleWidthDynamic(void)
 {
+	FocusedState fs = getFocusedState(false);
 	uint width = 0;
-	uint width_max = 0;
-	// FIXME: what about selected tabs?
-	PFont *font = getFont(getFocusedState(false));
+	PFont *font = getFont(fs);
 
 	if (_data->isTitleWidthSymetric()) {
-		// Symetric mode, get max tab width, multiply with number of tabs
+		// Symetric mode, get max tab width, multiply with number
+		// of tabs
 		std::vector<PDecor::TitleItem*>::iterator it = _titles.begin();
+		uint width_max = 0;
 		for (; it != _titles.end(); ++it) {
 			width = font->getWidth((*it)->getVisible());
 			if (width > width_max) {
@@ -1971,7 +2014,9 @@ PDecor::calcTitleWidthDynamic(void)
 			}
 		}
 
-		width = width_max + _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
+		width = width_max
+			+ _data->getPad(PAD_LEFT)
+			+ _data->getPad(PAD_RIGHT);
 		width *= _titles.size();
 
 	} else {
@@ -1979,13 +2024,14 @@ PDecor::calcTitleWidthDynamic(void)
 		std::vector<PDecor::TitleItem*>::iterator it = _titles.begin();
 		for (; it != _titles.end(); ++it) {
 			width += font->getWidth((*it)->getVisible())
-				+ _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT);
+				+ _data->getPad(PAD_LEFT)
+				+ _data->getPad(PAD_RIGHT);
 		}
 	}
 
 	// Add width of separators and buttons
 	width += (_titles.size() - 1)
-		* _data->getTextureSeparator(getFocusedState(false))->getWidth();
+		* _data->getTextureSeparator(fs)->getWidth();
 	width += _titles_left + _titles_right;
 
 	// Validate sizes, make sure it is not less than width min
@@ -2024,7 +2070,8 @@ PDecor::calcTabsWidth(void)
  * @param off Number of pixels left with tab_width pixels wide tabs.
  */
 void
-PDecor::calcTabsGetAvailAndTabWidth(uint &width_avail, uint &tab_width, int &off)
+PDecor::calcTabsGetAvailAndTabWidth(uint& width_avail, uint& tab_width,
+				    int &off)
 {
 	// Calculate width
 	width_avail = _title_wo.getWidth();
@@ -2035,8 +2082,9 @@ PDecor::calcTabsGetAvailAndTabWidth(uint &width_avail, uint &tab_width, int &off
 	}
 
 	// Remove separators if enough space is available
+	FocusedState fs = getFocusedState(false);
 	uint sep_width = (_titles.size() - 1)
-		* _data->getTextureSeparator(getFocusedState(false))->getWidth();
+		* _data->getTextureSeparator(fs)->getWidth();
 	if (width_avail > sep_width) {
 		width_avail -= sep_width;
 	}
@@ -2073,7 +2121,7 @@ void
 PDecor::calcTabsWidthAsymetric(void)
 {
 	int off;
-	uint width, width_avail, tab_width;
+	uint width_avail, tab_width;
 	calcTabsGetAvailAndTabWidth(width_avail, tab_width, off);
 
 	// Convenience
@@ -2084,8 +2132,10 @@ PDecor::calcTabsWidthAsymetric(void)
 	std::vector<PDecor::TitleItem*>::iterator it = _titles.begin();
 	for (; it != _titles.end(); ++it) {
 		// This should set the tab width to be only the size needed
-		width = font->getWidth((*it)->getVisible().c_str())
-			+ _data->getPad(PAD_LEFT) + _data->getPad(PAD_RIGHT) + ((off-- > 0) ? 1 : 0);
+		uint width = font->getWidth((*it)->getVisible().c_str())
+			+ _data->getPad(PAD_LEFT)
+			+ _data->getPad(PAD_RIGHT)
+			+ ((off-- > 0) ? 1 : 0);
 		width_total += width;
 		(*it)->setWidth(width);
 	}
@@ -2109,7 +2159,8 @@ PDecor::calcTabsWidthAsymetricShrink(uint width_avail, uint tab_width)
 	std::vector<PDecor::TitleItem*>::iterator it = _titles.begin();
 	for (; it != _titles.end(); ++it) {
 		if ((*it)->getWidth() < tab_width) {
-			// 3. Add width of tabs requiring less space to the average.
+			// 3. Add width of tabs requiring less space than the
+			//    the average
 			tabs_left--;
 			width_avail -= (*it)->getWidth();
 		}
