@@ -1,5 +1,6 @@
 //
 // PMenu.cc for pekwm
+// Copyright (C) 2022-2023 Claes Nästén <pekdon@gmail.com>
 // Copyright (C) 2004-2020 the pekwm development team
 //
 // This program is licensed under the GNU GPL.
@@ -28,8 +29,6 @@
 
 PMenu::Item::Item(const std::string &name, PWinObj *wo_ref, PTexture *icon)
 	: PWinObjReference(wo_ref),
-	  _x(0),
-	  _y(0),
 	  _name(name),
 	  _type(MENU_ITEM_NORMAL),
 	  _icon(icon),
@@ -51,19 +50,19 @@ std::map<Window,PMenu*> PMenu::_menu_map = std::map<Window,PMenu*>();
 
 //! @brief Constructor for PMenu class
 PMenu::PMenu(const std::string &title,
-             const std::string &name, const std::string decor_name,
-             bool init)
+	     const std::string &name, const std::string& decor_name,
+	     bool init)
 	: PDecor(decor_name, None, init),
 	  _name(name),
 	  _menu_parent(0), _class_hint("pekwm", "Menu", "", "", ""),
 	  _item_curr(0),
 	  _menu_wo(0),
-	  _menu_bg_fo(None),
-	  _menu_bg_un(None),
-	  _menu_bg_se(None),
 	  _menu_width(0),
-	  _item_height(0), _item_width_max(0), _item_width_max_avail(0),
-	  _icon_width(0), _icon_height(0),
+	  _item_height(0),
+	  _item_width_max(0),
+	  _item_pad_horz(0),
+	  _icon_width(0),
+	  _icon_height(0),
 	  _separator_height(0),
 	  _rows(0),
 	  _cols(0),
@@ -73,21 +72,18 @@ PMenu::PMenu(const std::string &title,
 	// PWinObj attributes
 	_type = PWinObj::WO_MENU;
 	setLayer(LAYER_MENU);
-	_hidden = true; // don't care about it when changing worskpace etc
+	_hidden = true;
 
 	// create menu content child
 	_menu_wo = new PWinObj(false);
-	XSetWindowAttributes attr;
-	attr.override_redirect = True;
-	attr.event_mask =
+
+	long event_mask =
 		ButtonPressMask|ButtonReleaseMask|ButtonMotionMask|
 		ExposureMask|FocusChangeMask|KeyPressMask|KeyReleaseMask|
 		PointerMotionMask;
-	_menu_wo->setWindow(X11::createWindow(_window, 0, 0, 1, 1, 0,
-					      CopyFromParent, InputOutput,
-					      CopyFromParent,
-					      CWOverrideRedirect|CWEventMask,
-					      &attr));
+	Window window = X11::createWmWindow(_window, 0, 0, 1, 1,
+					    InputOutput, event_mask);
+	_menu_wo->setWindow(window);
 
 	titleAdd(&_title);
 	titleSetActive(0);
@@ -128,10 +124,6 @@ PMenu::~PMenu(void)
 	for (; it != _items.end(); ++it) {
 		delete *it;
 	}
-
-	X11::freePixmap(_menu_bg_fo);
-	X11::freePixmap(_menu_bg_un);
-	X11::freePixmap(_menu_bg_se);
 }
 
 // START - PWinObj interface.
@@ -152,12 +144,16 @@ PMenu::setFocused(bool focused)
 	if (_focused != focused) {
 		PDecor::setFocused(focused);
 
-		X11::setWindowBackgroundPixmap(_menu_wo->getWindow(),
-					       _focused ? _menu_bg_fo : _menu_bg_un);
+		X11::setWindowBackgroundPixmap(
+				_menu_wo->getWindow(),
+				_focused
+				   ? _menu_bg_fo.getDrawable()
+				   : _menu_bg_un.getDrawable());
 		X11::clearWindow(_menu_wo->getWindow());
 		if (_item_curr < _items.size()) {
 			item_it item(_items.begin() + _item_curr);
-			_item_curr = _items.size(); // Force selectItem(item) to redraw
+			// Force selectItem(item) to redraw
+			_item_curr = _items.size();
 			selectItem(item);
 		}
 	}
@@ -183,8 +179,9 @@ PMenu::handleButtonPress(XButtonEvent *ev)
 		_pointer_x = ev->x_root;
 		_pointer_y = ev->y_root;
 
+		Config* cfg = pekwm::config();
 		std::vector<ActionEvent> *malm =
-			pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_MENU);
+			cfg->getMouseActionList(MOUSE_ACTION_LIST_MENU);
 		return ActionHandler::findMouseAction(ev->button, ev->state,
 						      MOUSE_EVENT_PRESS, malm);
 	} else {
@@ -225,7 +222,8 @@ PMenu::handleButtonRelease(XButtonEvent *ev)
 
 		std::vector<ActionEvent> *malm =
 			cfg->getMouseActionList(MOUSE_ACTION_LIST_MENU);
-		return ActionHandler::findMouseAction(ev->button, ev->state, mb, malm);
+		return ActionHandler::findMouseAction(ev->button, ev->state,
+						      mb, malm);
 	} else {
 		return PDecor::handleButtonRelease(ev);
 	}
@@ -258,21 +256,24 @@ PMenu::handleMotionEvent(XMotionEvent *ev)
 	if (*_menu_wo == ev->window) {
 		uint button = X11::getButtonFromState(ev->state);
 		handleItemEvent(button
-				? MOUSE_EVENT_MOTION_PRESSED : MOUSE_EVENT_MOTION,
+				? MOUSE_EVENT_MOTION_PRESSED
+				: MOUSE_EVENT_MOTION,
 				ev->x, ev->y);
 
 		ActionEvent *ae;
 		X11::stripButtonModifiers(&ev->state);
+		Config* cfg = pekwm::config();
 		std::vector<ActionEvent> *malm =
-			pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_MENU);
+			cfg->getMouseActionList(MOUSE_ACTION_LIST_MENU);
 		ae = ActionHandler::findMouseAction(button, ev->state,
 						    MOUSE_EVENT_MOTION, malm);
 
 		// check motion threshold
 		if (ae && (ae->threshold > 0)) {
-			if (! ActionHandler::checkAEThreshold(ev->x_root, ev->y_root,
-							      _pointer_x, _pointer_y,
-							      ae->threshold)) {
+			if (! ActionHandler::checkAEThreshold(
+						ev->x_root, ev->y_root,
+						_pointer_x, _pointer_y,
+						ae->threshold)) {
 				ae = 0;
 			}
 		}
@@ -286,8 +287,9 @@ ActionEvent*
 PMenu::handleEnterEvent(XCrossingEvent *ev)
 {
 	if (*_menu_wo == ev->window) {
+		Config* cfg = pekwm::config();
 		std::vector<ActionEvent> *malm =
-			pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_MENU);
+			cfg->getMouseActionList(MOUSE_ACTION_LIST_MENU);
 		return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
 						      MOUSE_EVENT_ENTER, malm);
 	} else {
@@ -299,8 +301,9 @@ ActionEvent*
 PMenu::handleLeaveEvent(XCrossingEvent *ev)
 {
 	if (*_menu_wo == ev->window) {
+		Config* cfg = pekwm::config();
 		std::vector<ActionEvent> *malm =
-			pekwm::config()->getMouseActionList(MOUSE_ACTION_LIST_MENU);
+			cfg->getMouseActionList(MOUSE_ACTION_LIST_MENU);
 		return ActionHandler::findMouseAction(BUTTON_ANY, ev->state,
 						      MOUSE_EVENT_LEAVE, malm);
 	} else {
@@ -339,19 +342,21 @@ PMenu::handleItemEvent(MouseEventType type, int x, int y)
 	}
 
 	if (pekwm::config()->isMenuEnterOn(type)) {
-		if (item->getWORef()
-		    && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
+		PWinObj* wo_ref = item->getWORef();
+		if (wo_ref
+		    && (wo_ref->getType() == PWinObj::WO_MENU)) {
+			PMenu* ref_menu = static_cast<PMenu*>(wo_ref);
 			// Special case for motion to avoid flickering
 			if (type != MOUSE_EVENT_MOTION
 			    && type != MOUSE_EVENT_MOTION_PRESSED
-			    && item->getWORef()->isMapped()) {
-				static_cast<PMenu*>(item->getWORef())->unmapSubmenus();
-				item->getWORef()->unmapWindow();
+			    && wo_ref->isMapped()) {
+				ref_menu->unmapSubmenus();
+				ref_menu->unmapWindow();
 
-			} else if (! item->getWORef()->isMapped()) {
+			} else if (! wo_ref->isMapped()) {
 				// unmap previous opened submenu if any
 				unmapSubmenus();
-				mapSubmenu(static_cast<PMenu*>(item->getWORef()));
+				mapSubmenu(ref_menu);
 			}
 		}
 	}
@@ -392,7 +397,8 @@ PMenu::buildMenuCalculate(void)
 	for (_size = 0; it != _items.end(); ++it) {
 		if ((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
 			++_size;
-		} else if ((*it)->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) {
+		} else if ((*it)->getType()
+			   == PMenu::Item::MENU_ITEM_SEPARATOR) {
 			++sep;
 		}
 	}
@@ -405,35 +411,26 @@ PMenu::buildMenuCalculate(void)
 		_item_width_max = title_width;
 	}
 
-	// This is the available width for drawing text on, the rest is reserved
-	// for submenu indicator, padding etc.
-	_item_width_max_avail = _item_width_max;
-
 	// Continue add padding etc.
 	Theme::PMenuData *md = pekwm::theme()->getMenuData();
-	_item_width_max += md->getPad(PAD_LEFT)
-		+ md->getPad(PAD_RIGHT);
+	_item_pad_horz = md->getPad(PAD_LEFT) + md->getPad(PAD_RIGHT);
 	if (pekwm::config()->isDisplayMenuIcons()) {
-		_item_width_max += _icon_width;
+		_item_pad_horz += _icon_width;
 	}
 
-	// If we have any submenus, increase the maximum width with arrow width +
-	// right pad as we are going to pad the arrow from the text too.
+	// If we have any submenus, increase the maximum width with arrow width
+	// + right pad as we are going to pad the arrow from the text too.
 	if (_has_submenu) {
-		_item_width_max += md->getPad(PAD_RIGHT)
+		_item_pad_horz += md->getPad(PAD_RIGHT)
 			+ md->getTextureArrow(OBJECT_STATE_FOCUSED)->getWidth();
 	}
 
-	// Remove padding etc from avail and item width.
-	if (_menu_width) {
-		uint padding = _item_width_max - _item_width_max_avail;
-		_item_width_max -= padding;
-		_item_width_max_avail -= padding;
-	}
+	_item_width_max += _item_pad_horz;
 
 	// Calculate item height
 	_item_height =
-		std::max(md->getFont(OBJECT_STATE_FOCUSED)->getHeight(), _icon_height)
+		std::max(md->getFont(OBJECT_STATE_FOCUSED)->getHeight(),
+			 _icon_height)
 		+ md->getPad(PAD_UP)
 		+ md->getPad(PAD_DOWN);
 	_separator_height =
@@ -459,7 +456,7 @@ PMenu::buildMenuCalculate(void)
  */
 void
 PMenu::buildMenuCalculateMaxWidth(uint &max_width,
-                                  uint &icon_width, uint &icon_height)
+				  uint &icon_width, uint &icon_height)
 {
 	// Calculate max item width, to be used if/when splitting a menu
 	// up in rows because of limited vertical space.
@@ -496,18 +493,15 @@ PMenu::buildMenuCalculateMaxWidth(uint &max_width,
 	// Make sure icon width and height are not larger than configured.
 	if (icon_width) {
 		Config *cfg = pekwm::config();
-		icon_width =
-			Util::between<uint>(icon_width,
-					    cfg->getMenuIconLimit(icon_width, WIDTH_MIN,
-								  _name),
-					    cfg->getMenuIconLimit(icon_width, WIDTH_MAX,
-								  _name));
-		icon_height =
-			Util::between<uint>(icon_height,
-					    cfg->getMenuIconLimit(icon_height, HEIGHT_MIN,
-								  _name),
-					    cfg->getMenuIconLimit(icon_height, HEIGHT_MAX,
-								  _name));
+		uint lmin, lmax;
+
+		lmin = cfg->getMenuIconLimit(icon_width, WIDTH_MIN, _name),
+		lmax = cfg->getMenuIconLimit(icon_width, WIDTH_MAX, _name);
+		icon_width = Util::between<uint>(icon_width, lmin, lmax);
+
+		lmin = cfg->getMenuIconLimit(icon_height, HEIGHT_MIN, _name),
+		lmax = cfg->getMenuIconLimit(icon_height, HEIGHT_MAX, _name);
+		icon_height = Util::between<uint>(icon_height, lmin, lmax);
 	}
 }
 
@@ -539,13 +533,15 @@ PMenu::buildMenuCalculateColumns(unsigned int &width, unsigned int &height)
 	width = _cols * _item_width_max;
 	// need to calculate max height, the one with most separators if any
 	if (_cols > 1) {
-		uint i, j, row_height;
+		uint i, j;
 		height = 0;
 
 		item_it it(_items.begin());
 		for (i = 0, it = _items.begin(); i < _cols; ++i) {
-			row_height = 0;
-			for (j = 0; j < _rows && it != _items.end(); ++it, ++j) {
+			uint row_height = 0;
+			for (j = 0;
+			     j < _rows && it != _items.end();
+			     ++it, ++j) {
 				switch ((*it)->getType()) {
 				case PMenu::Item::MENU_ITEM_NORMAL:
 					row_height += _item_height;
@@ -570,25 +566,32 @@ PMenu::buildMenuCalculateColumns(unsigned int &width, unsigned int &height)
 void
 PMenu::buildMenuPlace(void)
 {
-	uint x, y;
+	uint x;
 	std::vector<PMenu::Item*>::const_iterator it;
 
 	x = 0;
 	it = _items.begin();
 	// cols
 	for (uint i = 0; i < _cols; ++i) {
-		y = 0;
+		uint y = 0;
 		// rows
 		for (uint j = 0; j < _rows && it != _items.end(); ++it) {
-			if ((*it)->getType() != PMenu::Item::MENU_ITEM_HIDDEN) {
-				(*it)->setX(x);
-				(*it)->setY(y);
-				if ((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
-					y += _item_height;
-					++j; // only count real menu items
-				} else if ((*it)->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) {
-					y += _separator_height;
-				}
+			if ((*it)->getType()
+			    == PMenu::Item::MENU_ITEM_HIDDEN) {
+				continue;
+			}
+			(*it)->setX(x);
+			(*it)->setY(y);
+			(*it)->setWidth(_item_width_max);
+			if ((*it)->getType()
+			    == PMenu::Item::MENU_ITEM_NORMAL) {
+				y += _item_height;
+				++j; // only count real menu items
+				(*it)->setHeight(_item_height);
+			} else if ((*it)->getType()
+				   == PMenu::Item::MENU_ITEM_SEPARATOR) {
+				y += _separator_height;
+				(*it)->setHeight(_separator_height);
 			}
 		}
 		x += _item_width_max;
@@ -599,125 +602,142 @@ PMenu::buildMenuPlace(void)
 void
 PMenu::buildMenuRender(void)
 {
-	buildMenuRenderState(_menu_bg_fo, OBJECT_STATE_FOCUSED);
-	buildMenuRenderState(_menu_bg_un, OBJECT_STATE_UNFOCUSED);
-	buildMenuRenderState(_menu_bg_se, OBJECT_STATE_SELECTED);
+	_menu_bg_fo.resize(getChildWidth(), getChildHeight());
+	buildMenuRenderState(&_menu_bg_fo, OBJECT_STATE_FOCUSED);
+	_menu_bg_un.resize(getChildWidth(), getChildHeight());
+	buildMenuRenderState(&_menu_bg_un, OBJECT_STATE_UNFOCUSED);
+	_menu_bg_se.resize(getChildWidth(), getChildHeight());
+	buildMenuRenderState(&_menu_bg_se, OBJECT_STATE_SELECTED);
 
 	X11::setWindowBackgroundPixmap(_menu_wo->getWindow(),
-				       _focused ? _menu_bg_fo : _menu_bg_un);
+				       _focused ? _menu_bg_fo.getDrawable()
+						: _menu_bg_un.getDrawable());
 	X11::clearWindow(_menu_wo->getWindow());
 }
 
 //! @brief Renders menu content on pix, with state state
 void
-PMenu::buildMenuRenderState(Pixmap &pix, ObjectState state)
+PMenu::buildMenuRenderState(PSurface *surf, ObjectState state)
 {
-	// get a fresh pixmap for the menu
-	X11::freePixmap(pix);
-	pix = X11::createPixmap(getChildWidth(), getChildHeight());
-
 	Theme::PMenuData *md = pekwm::theme()->getMenuData();
 	PTexture *tex = md->getTextureMenu(state);
-	tex->render(pix, 0, 0, getChildWidth(), getChildHeight());
+	tex->render(surf, 0, 0, getChildWidth(), getChildHeight());
 	PFont *font = md->getFont(state);
 	font->setColor(md->getColor(state));
 
 	item_it it = _items.begin();
 	for (; it != _items.end(); ++it) {
 		if ((*it)->getType() != PMenu::Item::MENU_ITEM_HIDDEN) {
-			buildMenuRenderItem(pix, state, *it);
+			buildMenuRenderItem(surf, state, *it);
 		}
 	}
 }
 
 //! @brief Renders item on pix, with state state
 void
-PMenu::buildMenuRenderItem(Pixmap pix, ObjectState state, PMenu::Item *item)
+PMenu::buildMenuRenderItem(PSurface *surf, ObjectState state, PMenu::Item *item)
+{
+	if (item->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
+		buildMenuRenderItemNormal(surf, state, item);
+	} else if (item->getType() == PMenu::Item::MENU_ITEM_SEPARATOR
+		   && state < OBJECT_STATE_SELECTED) {
+		buildMenuRenderItemSeparator(surf, state, item);
+	}
+}
+
+void
+PMenu::buildMenuRenderItemNormal(PSurface *surf, ObjectState state,
+				 PMenu::Item* item)
 {
 	Theme::PMenuData *md = pekwm::theme()->getMenuData();
 	Config *cfg = pekwm::config();
 
-	if (item->getType() == PMenu::Item::MENU_ITEM_NORMAL) {
-		PTexture *tex = md->getTextureItem(state);
-		tex->render(pix, item->getX(), item->getY(),
-			    _item_width_max, _item_height);
+	PTexture *tex = md->getTextureItem(state);
+	tex->render(surf,
+		    item->getX(), item->getY(),
+		    item->getWidth(), item->getHeight());
 
-		uint start_x, start_y, icon_width, icon_height;
-		// If entry has an icon, draw it
-		if (item->getIcon() && cfg->isDisplayMenuIcons()) {
-			icon_width =
-				Util::between<uint>(item->getIcon()->getWidth(),
-						    cfg->getMenuIconLimit(_icon_width,
-									  WIDTH_MIN, _name),
-						    cfg->getMenuIconLimit(_icon_width,
-									  WIDTH_MAX, _name));
+	uint start_x, start_y;
+	// If entry has an icon, draw it
+	if (item->getIcon() && cfg->isDisplayMenuIcons()) {
+		uint lmin, lmax;
+		lmin = cfg->getMenuIconLimit(_icon_width, WIDTH_MIN, _name);
+		lmax = cfg->getMenuIconLimit(_icon_width, WIDTH_MAX, _name);
+		uint icon_width =
+			Util::between<uint>(item->getIcon()->getWidth(),
+					    lmin, lmax);
 
-			icon_height =
-				Util::between<uint>(item->getIcon()->getHeight(),
-						    cfg->getMenuIconLimit(_icon_height,
-									  HEIGHT_MIN, _name),
-						    cfg->getMenuIconLimit(_icon_height,
-									  HEIGHT_MAX, _name));
+		lmin = cfg->getMenuIconLimit(_icon_height, HEIGHT_MIN, _name);
+		lmax = cfg->getMenuIconLimit(_icon_height, HEIGHT_MAX, _name);
+		uint icon_height =
+			Util::between<uint>(item->getIcon()->getHeight(),
+					    lmin, lmax);
 
-			start_x = item->getX() + md->getPad(PAD_LEFT)
-				+ (_icon_width - icon_width) / 2;
-			start_y = item->getY() + (_item_height - icon_height) / 2;
-			item->getIcon()->render(pix, start_x, start_y,
-						icon_width, icon_height);
-		} else {
-			icon_width = 0;
-			icon_height = 0;
-		}
-
-		// If entry has a submenu, lets draw our submenu "arrow"
-		if (item->getWORef()
-		    && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
-			tex = md->getTextureArrow(state);
-			uint arrow_width = tex->getWidth();
-			uint arrow_height = tex->getHeight();
-			uint arrow_y =
-				static_cast<uint>((_item_height / 2) - (arrow_height / 2));
-
-			start_x = item->getX() + _item_width_max
-				- arrow_width - md->getPad(PAD_RIGHT);
-			start_y = item->getY() + arrow_y;
-			tex->render(pix, start_x, start_y, arrow_width, arrow_height);
-		}
-
-		PFont *font = md->getFont(state);
-		start_x = item->getX() + md->getPad(PAD_LEFT);
-		// Add icon width to starting x position if frame icons are enabled.
-		if (cfg->isDisplayMenuIcons()) {
-			start_x += _icon_width;
-		}
-
-		start_y = item->getY() + md->getPad(PAD_UP)
-			+ (_item_height - font->getHeight()
-			   - md->getPad(PAD_UP) - md->getPad(PAD_DOWN)) / 2;
-
-		// Render item text.
-		font->draw(pix, start_x, start_y, item->getName().c_str(),
-			   0, _item_width_max_avail);
-
-	} else if ((item->getType() == PMenu::Item::MENU_ITEM_SEPARATOR) &&
-		   (state < OBJECT_STATE_SELECTED)) {
-		PTexture *tex = md->getTextureSeparator(state);
-		tex->render(pix, item->getX(), item->getY(),
-			    _item_width_max, _separator_height);
+		start_x = item->getX() + md->getPad(PAD_LEFT)
+			+ (_icon_width - icon_width) / 2;
+		start_y = item->getY()
+			+ (_item_height - icon_height) / 2;
+		item->getIcon()->render(surf, start_x, start_y,
+					icon_width, icon_height);
 	}
+
+	// If entry has a submenu, lets draw our submenu "arrow"
+	if (item->getWORef()
+	    && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
+		tex = md->getTextureArrow(state);
+		uint arrow_width = tex->getWidth();
+		uint arrow_height = tex->getHeight();
+		uint arrow_y =
+			static_cast<uint>((_item_height / 2)
+					  - (arrow_height / 2));
+
+		start_x = item->getX() + item->getWidth()
+			- arrow_width - md->getPad(PAD_RIGHT);
+		start_y = item->getY() + arrow_y;
+		tex->render(surf, start_x, start_y,
+			    arrow_width, arrow_height);
+	}
+
+	PFont *font = md->getFont(state);
+	start_x = item->getX() + md->getPad(PAD_LEFT);
+	// Add icon width to starting x position if frame icons are
+	// enabled.
+	if (cfg->isDisplayMenuIcons()) {
+		start_x += _icon_width;
+	}
+
+	start_y = item->getY() + md->getPad(PAD_UP)
+		+ (_item_height - font->getHeight()
+		   - md->getPad(PAD_UP) - md->getPad(PAD_DOWN)) / 2;
+
+	// Render item text.
+	font->draw(surf, start_x, start_y, item->getName().c_str(),
+		   0, item->getWidth() - _item_pad_horz);
 }
 
-#define COPY_ITEM_AREA(ITEM, PIX)					\
-	XCopyArea(X11::getDpy(), PIX, _menu_wo->getWindow(), X11::getGC(), \
-		  (ITEM)->getX(), (ITEM)->getY(), _item_width_max, _item_height, \
-		  (ITEM)->getX(), (ITEM)->getY());
+void
+PMenu::buildMenuRenderItemSeparator(PSurface *surf, ObjectState state,
+				    PMenu::Item* item)
+{
+	Theme::PMenuData *md = pekwm::theme()->getMenuData();
+	PTexture *tex = md->getTextureSeparator(state);
+	tex->render(surf,
+		    item->getX(), item->getY(),
+		    item->getWidth(), item->getHeight());
+}
+
+#define COPY_ITEM_AREA(ITEM, PIX)		  \
+	X11::copyArea(PIX, _menu_wo->getWindow(), \
+		      (ITEM)->getX(), (ITEM)->getY(), \
+		      (ITEM)->getWidth(), (ITEM)->getHeight(), \
+		      (ITEM)->getX(), (ITEM)->getY());
 
 //! @brief Renders item as selected
 //! @param item Item to select
 //! @param unmap_submenu Defaults to true
 void
 PMenu::selectItem(std::vector<PMenu::Item*>::const_iterator item,
-                  bool unmap_submenu)
+		  bool unmap_submenu)
 {
 	if (_item_curr < _items.size() && _items[_item_curr] == *item) {
 		return;
@@ -733,11 +753,12 @@ PMenu::selectItem(std::vector<PMenu::Item*>::const_iterator item,
 void
 PMenu::renderSelectedItem(void)
 {
-	if (_mapped && _item_curr < _items.size()) {
-		PMenu::Item *item = _items[_item_curr];
-		if (item->getType() != PMenu::Item::MENU_ITEM_HIDDEN) {
-			COPY_ITEM_AREA(item, _menu_bg_se);
-		}
+	if (! _mapped || _item_curr >= _items.size()) {
+		return;
+	}
+	PMenu::Item *item = _items[_item_curr];
+	if (item->getType() != PMenu::Item::MENU_ITEM_HIDDEN) {
+		COPY_ITEM_AREA(item, _menu_bg_se.getDrawable());
 	}
 }
 
@@ -747,16 +768,28 @@ void
 PMenu::deselectItem(bool unmap_submenu)
 {
 	// deselect previous item
-	if (_item_curr < _items.size()
-	    && _items[_item_curr]->getType() != PMenu::Item::MENU_ITEM_HIDDEN) {
-		if (_mapped)
-			COPY_ITEM_AREA(_items[_item_curr], (_focused ? _menu_bg_fo : _menu_bg_un));
+	if (_item_curr >= _items.size()) {
+		return;
+	}
 
-		if (unmap_submenu && _items[_item_curr]->getWORef()
-		    && (_items[_item_curr]->getWORef()->getType() == PWinObj::WO_MENU)) {
-			static_cast<PMenu*>(_items[_item_curr]->getWORef())->unmapSubmenus();
-			_items[_item_curr]->getWORef()->unmapWindow();
-		}
+	PMenu::Item* item = _items[_item_curr];
+	if (item->getType() == PMenu::Item::MENU_ITEM_HIDDEN) {
+		return;
+	}
+
+	if (_mapped) {
+		COPY_ITEM_AREA(_items[_item_curr],
+			       (_focused ? _menu_bg_fo.getDrawable()
+					 : _menu_bg_un.getDrawable()));
+	}
+
+	PWinObj* wo_ref = item->getWORef();
+	if (unmap_submenu
+	    && wo_ref
+	    && (wo_ref->getType() == PWinObj::WO_MENU)) {
+		PMenu* ref_menu = static_cast<PMenu*>(wo_ref);
+		ref_menu->unmapSubmenus();
+		wo_ref->unmapWindow();
 	}
 }
 
@@ -889,7 +922,7 @@ PMenu::insert(const std::string &name, PWinObj *wo_ref, PTexture *icon)
 //! @param wo_ref PWinObj to refer to, defaults to 0
 void
 PMenu::insert(const std::string &name, const ActionEvent &ae,
-              PWinObj *wo_ref, PTexture *icon)
+	      PWinObj *wo_ref, PTexture *icon)
 {
 	PMenu::Item *item = new PMenu::Item(name, wo_ref, icon);
 	item->setAE(ae);
@@ -909,7 +942,8 @@ PMenu::remove(PMenu::Item *item)
 		_item_curr = _items.size();
 	}
 
-	if (item->getWORef() && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
+	if (item->getWORef()
+	    && (item->getWORef()->getType() == PWinObj::WO_MENU)) {
 		_has_submenu--;
 	}
 
@@ -940,8 +974,8 @@ PMenu::mapUnderMouse(void)
 	X11::getMousePosition(x, y);
 
 	// this might seem a bit silly but the menu won't get updated before
-	// it has been mapped (if dynamic) so we're doing it twice to reduce the
-	// "flickering" risk but it's not 100% so it's done twice.
+	// it has been mapped (if dynamic) so we're doing it twice to reduce
+	// the "flickering" risk but it's not 100% so it's done twice.
 	makeInsideScreen(x, y);
 	mapWindowRaised();
 	makeInsideScreen(x, y);
@@ -963,8 +997,8 @@ PMenu::mapSubmenu(PMenu *menu, bool focus)
 	}
 
 	// this might seem a bit silly but the menu won't get updated before
-	// it has been mapped (if dynamic) so we're doing it twice to reduce the
-	// "flickering" risk but it's not 100% so it's done twice.
+	// it has been mapped (if dynamic) so we're doing it twice to reduce
+	// the "flickering" risk but it's not 100% so it's done twice.
 	menu->makeInsideScreen(x, y);
 	menu->mapWindowRaised();
 	menu->makeInsideScreen(x, y);
@@ -983,8 +1017,8 @@ PMenu::unmapSubmenus(void)
 	for (; it != _items.end(); ++it) {
 		if ((*it)->getWORef()
 		    && (*it)->getWORef()->getType() == PWinObj::WO_MENU) {
-			// Sub-menus will be deleted when unmapping this, so no need
-			// to continue.
+			// Sub-menus will be deleted when unmapping this, so no
+			// need to continue.
 			static_cast<PMenu*>((*it)->getWORef())->unmapSubmenus();
 			(*it)->getWORef()->unmapWindow();
 		}
@@ -1021,7 +1055,8 @@ PMenu::gotoParentMenu(void)
 void
 PMenu::select(PMenu::Item *item, bool unmap_submenu)
 {
-	selectItem(find(_items.begin(), _items.end(), item), unmap_submenu);
+	selectItem(std::find(_items.begin(), _items.end(), item),
+		   unmap_submenu);
 }
 
 //! @brief Selects item number num in menu
@@ -1040,7 +1075,8 @@ void
 PMenu::selectItemRel(int off)
 {
 	if (off == 0) {
-		P_WARN("trying to select non existing relative (current) item number");
+		P_WARN("trying to select non existing relative (current) "
+		       "item number");
 		return;
 	}
 
@@ -1100,9 +1136,9 @@ PMenu::findItem(int x, int y)
 	for (; it != _items.end(); ++it) {
 		if (((*it)->getType() == PMenu::Item::MENU_ITEM_NORMAL) &&
 		    (x >= (*it)->getX())
-		    && (x <= signed((*it)->getX() + _item_width_max)) &&
+		    && (x <= signed((*it)->getX() + (*it)->getWidth())) &&
 		    (y >= (*it)->getY())
-		    && (y <= signed((*it)->getY() + _item_height))) {
+		    && (y <= signed((*it)->getY() + (*it)->getHeight()))) {
 			return *it;
 		}
 	}
