@@ -1,6 +1,6 @@
 //
 // PWinObj.cc for pekwm
-// Copyright (C) 2003-2020 Claes Nästen <pekdon@gmail.com>
+// Copyright (C) 2003-2023 Claes Nästen <pekdon@gmail.com>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
@@ -26,10 +26,17 @@ std::map<Window, PWinObj*> PWinObj::_wo_map = std::map<Window, PWinObj*>();
 //! @brief PWinObj constructor.
 PWinObj::PWinObj(bool keyboard_input)
 	: _window(None),
-	  _parent(0), _type(WO_NO_TYPE), _lastActivity(X11::getLastEventTime()),
-	  _opaque(true), _workspace(0), _layer(LAYER_NORMAL),
-	  _mapped(false), _iconified(false),
-	  _hidden(false), _focused(false), _sticky(false),
+	  _parent(0),
+	  _type(WO_NO_TYPE),
+	  _lastActivity(X11::getLastEventTime()),
+	  _opaque(true),
+	  _workspace(0),
+	  _layer(LAYER_NORMAL),
+	  _mapped(false),
+	  _iconified(false),
+	  _hidden(false),
+	  _focused(false),
+	  _sticky(false),
 	  _focusable(true),
 	  _shape_bounding(false),
 	  _keyboard_input(keyboard_input)
@@ -110,7 +117,7 @@ PWinObj::unmapWindow(void)
 	if (! _mapped) {
 		return;
 	}
-    
+
 	_mapped = false;
 
 	// Make sure unmapped windows drops focus
@@ -126,7 +133,7 @@ PWinObj::iconify(void)
 	if (_iconified) {
 		return;
 	}
-    
+
 	_iconified = true;
 }
 
@@ -163,7 +170,8 @@ void
 PWinObj::resize(uint width, uint height)
 {
 	if (! width || ! height) {
-		P_WARN("width " << width << " height " << height << ", invalid geometry");
+		P_WARN("width " << width << " height " << height
+		       << ", invalid geometry");
 		return;
 	}
 
@@ -231,20 +239,13 @@ PWinObj::setOpaque(bool opaque)
 	updateOpacity();
 }
 
-//! @brief Only sets _hidden to hidden.
-void
-PWinObj::setHidden(bool hidden)
-{
-	_hidden = hidden;
-}
-
 //! @brief Executes XSetInputFocus on the appropriate window.
 void
 PWinObj::giveInputFocus(void)
 {
 	if (! _mapped  || ! _focusable) {
-		P_WARN("trying to focus non focusable window. mapped " << _mapped
-		       << " focusable " << _focusable);
+		P_WARN("trying to focus non focusable window. mapped "
+		       << _mapped << " focusable " << _focusable);
 		return;
 	}
 
@@ -280,7 +281,7 @@ void
 PWinObj::woListRemove(PWinObj *wo)
 {
 	std::vector<PWinObj*>::iterator it
-		= find(_wo_list.begin(), _wo_list.end(), wo);
+		= std::find(_wo_list.begin(), _wo_list.end(), wo);
 	if (it != _wo_list.end()) {
 		_wo_list.erase(it);
 	}
