@@ -1,5 +1,6 @@
 //
 // WindowManager.cc for pekwm
+// Copyright (C) 2023 Claes Nästén <pekdon@gmail.com>
 // Copyright (C) 2002-2021 the pekwm development team
 //
 // windowmanager.cc for aewm++
@@ -12,6 +13,7 @@
 
 #include "config.h"
 
+#include "Color.hh"
 #include "Debug.hh"
 #include "PWinObj.hh"
 #include "PDecor.hh"
@@ -305,22 +307,22 @@ WindowManager::scanWindows(void)
 	// We filter out all windows with the the IconWindowHint
 	// set not pointing to themselves, making DockApps
 	// work as they are supposed to.
+	XWMHints wm_hints;
 	for (; it != win_list.end(); ++it) {
 		if (*it == None) {
 			continue;
 		}
 
-		XWMHints *wm_hints = XGetWMHints(X11::getDpy(), *it);
-		if (wm_hints) {
-			if ((wm_hints->flags&IconWindowHint) &&
-			    (wm_hints->icon_window != *it)) {
-				std::vector<Window>::iterator
-					i_it(find(win_list.begin(), win_list.end(),
-						  wm_hints->icon_window));
-				if (i_it != win_list.end())
-					*i_it = None;
+		if (X11::getWMHints(*it, wm_hints)
+		    && (wm_hints.flags&IconWindowHint)
+		    && (wm_hints.icon_window != *it)) {
+			std::vector<Window>::iterator
+				i_it(std::find(win_list.begin(),
+					       win_list.end(),
+					       wm_hints.icon_window));
+			if (i_it != win_list.end()) {
+				*i_it = None;
 			}
-			X11::free(wm_hints);
 		}
 	}
 
@@ -331,8 +333,8 @@ WindowManager::scanWindows(void)
 	}
 
 	// Try to focus the ontop window, if no window we give root focus
-	PWinObj *wo = Workspaces::getTopWO(PWinObj::WO_FRAME);
-	if (wo && wo->isMapped()) {
+	PWinObj *wo = Workspaces::getTopFocusableWO(PWinObj::WO_FRAME);
+	if (wo) {
 		wo->giveInputFocus();
 	} else {
 		pekwm::rootWo()->giveInputFocus();
@@ -366,19 +368,23 @@ WindowManager::screenEdgeCreate(void)
 	RootWO *root_wo = pekwm::rootWo();
 	_screen_edges[0] =
 		new EdgeWO(root_wo, SCREEN_EDGE_LEFT,
-			   indent && (cfg->getScreenEdgeSize(SCREEN_EDGE_LEFT) > 0),
+			   indent
+			   && (cfg->getScreenEdgeSize(SCREEN_EDGE_LEFT) > 0),
 			   cfg);
 	_screen_edges[1] =
 		new EdgeWO(root_wo, SCREEN_EDGE_RIGHT,
-			   indent && (cfg->getScreenEdgeSize(SCREEN_EDGE_RIGHT) > 0),
+			   indent
+			   && (cfg->getScreenEdgeSize(SCREEN_EDGE_RIGHT) > 0),
 			   cfg);
 	_screen_edges[2] =
 		new EdgeWO(root_wo, SCREEN_EDGE_TOP,
-			   indent && (cfg->getScreenEdgeSize(SCREEN_EDGE_TOP) > 0),
+			   indent
+			   && (cfg->getScreenEdgeSize(SCREEN_EDGE_TOP) > 0),
 			   cfg);
 	_screen_edges[3] =
 		new EdgeWO(root_wo, SCREEN_EDGE_BOTTOM,
-			   indent && (cfg->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) > 0),
+			   indent
+			   && (cfg->getScreenEdgeSize(SCREEN_EDGE_BOTTOM) > 0),
 			   cfg);
 
 	// make sure the edge stays ontop
@@ -426,12 +432,12 @@ WindowManager::screenEdgeResize(void)
 void
 WindowManager::screenEdgeMapUnmap(void)
 {
-	EdgeWO* edge;
 	for (int i=0; i<4; ++i) {
-		edge = _screen_edges[i];
+		EdgeWO *edge = _screen_edges[i];
 		Config *cfg = pekwm::config();
 		if (cfg->getScreenEdgeSize(edge->getEdge()) > 0
-		    && cfg->getEdgeListFromPosition(edge->getEdge())->size() > 0) {
+		    && cfg->getEdgeListFromPosition(edge->getEdge())->size()
+		       > 0) {
 			edge->mapWindow();
 		} else {
 			edge->unmapWindow();
@@ -467,10 +473,10 @@ WindowManager::doReloadConfig(void)
 	Config *cfg = pekwm::config();
 	// If any of these changes, re-fetch of all names is required
 	bool old_client_unique_name = cfg->getClientUniqueName();
-	const std::string &old_client_unique_name_pre =
-		cfg->getClientUniqueNamePre();
-	const std::string &old_client_unique_name_post =
-		cfg->getClientUniqueNamePost();
+	std::string
+		old_client_unique_name_pre(cfg->getClientUniqueNamePre());
+	std::string
+		old_client_unique_name_post(cfg->getClientUniqueNamePost());
 
 	// Reload configuration
 	if (! cfg->load(cfg->getConfigFile())) {
@@ -503,17 +509,25 @@ WindowManager::doReloadConfig(void)
  * Reload theme file and update decorations.
  */
 void
-WindowManager::doReloadTheme(void)
+WindowManager::doReloadTheme(bool force)
 {
 	// Reload the theme
 	if (! pekwm::theme()->load(pekwm::config()->getThemeFile(),
-				   pekwm::config()->getThemeVariant())) {
+				   pekwm::config()->getThemeVariant(),
+				   force)) {
+		P_TRACE("not reloading decors, theme not changed");
 		return;
 	}
 
 	startBackground(pekwm::theme()->getThemeDir(),
 			pekwm::theme()->getBackground());
 
+	doReloadThemeDecors();
+}
+
+void
+WindowManager::doReloadThemeDecors()
+{
 	// Reload the themes on all decors
 	std::vector<PDecor*>::const_iterator it = PDecor::pdecor_begin();
 	for (; it != PDecor::pdecor_end(); ++it) {
@@ -622,6 +636,39 @@ WindowManager::doReloadHarbour(void)
 	pekwm::harbour()->updateHarbourSize();
 }
 
+/**
+ * Reload Xrm Resources and update resource dependent parts (if any at
+ * the time).
+ */
+void
+WindowManager::doReloadResources(void)
+{
+	P_TRACE("RESOURCE_MANAGER changed, reloading resources");
+	X11::loadXrmResources();
+	if (isResourcesChanged()) {
+		P_TRACE("used resources changed, reloading theme");
+		doReloadTheme(true);
+	}
+}
+
+bool
+WindowManager::isResourcesChanged(void)
+{
+	const std::map<std::string, std::string> &resources =
+		X11::getRefResources();
+	std::map<std::string, std::string>::const_iterator it =
+		resources.begin();
+	for (; it != resources.end(); ++it) {
+		P_TRACE("check referenced resource " << it->first);
+		std::string val;
+		X11::getXrmString(it->first, val);
+		if (val != it->second) {
+			return true;
+		}
+	}
+	return false;
+}
+
 /**
  * Exit pekwm and restart with the command command
  */
@@ -662,12 +709,14 @@ WindowManager::handleSignals(void)
 			pid = waitpid(WAIT_ANY, nullptr, WNOHANG);
 			if (pid == -1) {
 				if (errno == EINTR) {
-					P_TRACE("waitpid interrupted, retrying");
+					P_TRACE("waitpid interrupted, "
+						"retrying");
 				}
 			} else if (pid == 0) {
 				P_TRACE("no more finished child processes");
 			} else {
-				P_TRACE("child process " << pid << " finished");
+				P_TRACE("child process " << pid
+					<< " finished");
 			}
 		} while (pid > 0 || (pid == -1 && errno == EINTR));
 
@@ -828,7 +877,8 @@ WindowManager::handleEvent(XEvent &ev)
 			screenEdgeResize();
 
 			// Make sure windows are visible after resize
-			std::vector<PDecor*>::const_iterator it(PDecor::pdecor_begin());
+			std::vector<PDecor*>::const_iterator it =
+				PDecor::pdecor_begin();
 			for (; it != PDecor::pdecor_end(); ++it) {
 				Workspaces::placeWoInsideScreen(*it);
 			}
@@ -912,63 +962,44 @@ WindowManager::handleKeyEventAction(XKeyEvent *ev, ActionEvent *ae,
 		close_ae.action_list.back().setAction(ACTION_CLOSE);
 
 		ActionPerformed close_ap(wo_orig, close_ae);
-		pekwm::actionHandler()->handleAction(close_ap);
+		pekwm::actionHandler()->handleAction(&close_ap);
 	}
 
 	ActionPerformed ap(wo, *ae);
 	ap.type = ev->type;
 	ap.event.key = ev;
-	pekwm::actionHandler()->handleAction(ap);
+	pekwm::actionHandler()->handleAction(&ap);
 }
 
 void
 WindowManager::handleButtonPressEvent(XButtonEvent *ev)
 {
-	// Clear event queue
-	while (X11::checkTypedEvent(ButtonPress, (XEvent *) ev)) {
-		if (! ev->send_event) {
-			X11::setLastEventTime(ev->time);
-		}
+	ActionEvent *ae = nullptr;
+	PWinObj *wo = PWinObj::findPWinObj(ev->window);
+	if (wo == pekwm::rootWo() && ev->subwindow != None) {
+		wo = PWinObj::findPWinObj(ev->subwindow);
 	}
 
-	ActionEvent *ae = 0;
-	PWinObj *wo = 0;
-
-	wo = PWinObj::findPWinObj(ev->window);
 	if (wo) {
-		// Update all objects (and again if child found)
 		wo->setLastActivity(ev->time);
 
-		ae = wo->handleButtonPress(ev);
-
-		if (wo->getType() == PWinObj::WO_FRAME) {
-			// this is done so that clicking the titlebar executes
-			// action on the client clicked on, doesn't apply when
-			// subwindow is set (meaning a titlebar button beeing
-			// pressed)
-			if ((ev->subwindow == None)
-			    && (ev->window == static_cast<Frame*>(wo)->getTitleWindow())) {
-				wo = static_cast<Frame*>(wo)->getChildFromPos(ev->x);
-			} else {
-				wo = static_cast<Frame*>(wo)->getActiveChild();
-			}
-
-			if (wo != 0) {
-				wo->setLastActivity(ev->time);
-			}
-		}
+		// in case the event causes the wo to go away, update it
+		// before handling the event.
+		PWinObj *orig_wo = wo;
+		wo = updateWoForFrameClick(ev, orig_wo);
+		ae = orig_wo->handleButtonPress(ev);
 	}
 
 	if (ae) {
 		ActionPerformed ap(wo, *ae);
 		ap.type = ev->type;
 		ap.event.button = ev;
-
-		pekwm::actionHandler()->handleAction(ap);
+		pekwm::actionHandler()->handleAction(&ap);
 	} else {
-		DockApp *da = pekwm::harbour()->findDockAppFromFrame(ev->window);
+		Harbour *harbour = pekwm::harbour();
+		DockApp *da = harbour->findDockAppFromFrame(ev->window);
 		if (da) {
-			pekwm::harbour()->handleButtonEvent(ev, da);
+			harbour->handleButtonEvent(ev, da);
 		}
 	}
 }
@@ -976,67 +1007,77 @@ WindowManager::handleButtonPressEvent(XButtonEvent *ev)
 void
 WindowManager::handleButtonReleaseEvent(XButtonEvent *ev)
 {
-	// Flush ButtonReleases
-	while (X11::checkTypedEvent(ButtonRelease, (XEvent *) ev)) {
-		if (! ev->send_event) {
-			X11::setLastEventTime(ev->time);
-		}
-	}
-
-	ActionEvent *ae = 0;
+	ActionEvent *ae = nullptr;
 	PWinObj *wo = PWinObj::findPWinObj(ev->window);
 	if (wo == pekwm::rootWo() && ev->subwindow != None) {
 		wo = PWinObj::findPWinObj(ev->subwindow);
 	}
 
 	if (wo) {
-		// Kludge for the case that wo is freed by handleButtonRelease(.).
-		PWinObj::Type wotype = wo->getType();
-		ae = wo->handleButtonRelease(ev);
+		wo->setLastActivity(ev->time);
 
-		if (wotype == PWinObj::WO_FRAME) {
-			// this is done so that clicking the titlebar executes
-			// action on the client clicked on, doesn't apply when
-			// subwindow is set (meaning a titlebar button beeing
-			// pressed)
-			if ((ev->subwindow == None)
-			    && (ev->window == static_cast<Frame*>(wo)->getTitleWindow())) {
-				wo = static_cast<Frame*>(wo)->getChildFromPos(ev->x);
-			} else {
-				wo = static_cast<Frame*>(wo)->getActiveChild();
-			}
-		}
+		// in case the event causes the wo to go away, update it
+		// before handling the event.
+		PWinObj *orig_wo = wo;
+		wo = updateWoForFrameClick(ev, orig_wo);
+		ae = orig_wo->handleButtonRelease(ev);
+	}
 
-		if (ae) {
-			ActionPerformed ap(wo, *ae);
-			ap.type = ev->type;
-			ap.event.button = ev;
-
-			pekwm::actionHandler()->handleAction(ap);
-		}
+	if (ae) {
+		ActionPerformed ap(wo, *ae);
+		ap.type = ev->type;
+		ap.event.button = ev;
+		pekwm::actionHandler()->handleAction(&ap);
 	} else {
-		DockApp *da = pekwm::harbour()->findDockAppFromFrame(ev->window);
+		Harbour *harbour = pekwm::harbour();
+		DockApp *da = harbour->findDockAppFromFrame(ev->window);
 		if (da) {
-			pekwm::harbour()->handleButtonEvent(ev, da);
+			harbour->handleButtonEvent(ev, da);
 		}
 	}
 }
 
+PWinObj*
+WindowManager::updateWoForFrameClick(XButtonEvent *ev, PWinObj *wo)
+{
+	if (wo->getType() != PWinObj::WO_FRAME) {
+		return wo;
+	}
+
+	// update selected window object with the one clicked
+	// on the titlebar (if any), subwindow check is there
+	// to skip titlebar button press
+	Frame *frame = static_cast<Frame*>(wo);
+	if (ev->subwindow == None
+	    && ev->window == frame->getTitleWindow()) {
+		wo = frame->getChildFromPos(ev->x);
+	} else {
+		wo = frame->getActiveChild();
+	}
+
+	if (wo) {
+		wo->setLastActivity(ev->time);
+	}
+
+	return wo;
+}
+
 void
 WindowManager::handleConfigureRequestEvent(XConfigureRequestEvent *ev)
 {
 	Client *client = Client::findClientFromWindow(ev->window);
 
 	if (client) {
-		((Frame*) client->getParent())->handleConfigureRequest(ev, client);
-
+		Frame* frame = static_cast<Frame*>(client->getParent());
+		frame->handleConfigureRequest(ev, client);
 	} else {
 		DockApp *da = pekwm::harbour()->findDockApp(ev->window);
 		if (da) {
 			pekwm::harbour()->handleConfigureRequestEvent(ev, da);
 		} else {
 			// Since this window isn't yet a client lets delegate
-			// the configure request back to the window so it can use it.
+			// the configure request back to the window so it can
+			// use it.
 
 			XWindowChanges wc;
 
@@ -1047,7 +1088,8 @@ WindowManager::handleConfigureRequestEvent(XConfigureRequestEvent *ev)
 			wc.sibling = ev->above;
 			wc.stack_mode = ev->detail;
 
-			XConfigureWindow(X11::getDpy(), ev->window, ev->value_mask, &wc);
+			XConfigureWindow(X11::getDpy(), ev->window,
+					 ev->value_mask, &wc);
 		}
 	}
 }
@@ -1059,27 +1101,27 @@ WindowManager::handleConfigureRequestEvent(XConfigureRequestEvent *ev)
 void
 WindowManager::handleMotionEvent(XMotionEvent *ev)
 {
-	ActionEvent *ae = 0;
 	PWinObj *wo = PWinObj::findPWinObj(ev->window);
 	if (wo == pekwm::rootWo() && ev->subwindow != None) {
 		wo = PWinObj::findPWinObj(ev->subwindow);
 	}
 
 	if (wo) {
+		ActionEvent *ae = nullptr;
 		if (wo->getType() == PWinObj::WO_CLIENT) {
 			ae = wo->getParent()->handleMotionEvent(ev);
 
 		} else if (wo->getType() == PWinObj::WO_FRAME) {
 			ae = wo->handleMotionEvent(ev);
 
-			// this is done so that clicking the titlebar executes action on
-			// the client clicked on
+			// this is done so that clicking the titlebar executes
+			// action on the client clicked on
+			Frame* frame = static_cast<Frame*>(wo);
 			if (ev->subwindow != None) {
-				wo = static_cast<Frame*>(wo)->getActiveChild();
+				wo = frame->getActiveChild();
 			} else {
-				wo = static_cast<Frame*>(wo)->getChildFromPos(ev->x);
+				wo = frame->getChildFromPos(ev->x);
 			}
-
 		} else {
 			ae = wo->handleMotionEvent(ev);
 		}
@@ -1089,10 +1131,11 @@ WindowManager::handleMotionEvent(XMotionEvent *ev)
 			ap.type = ev->type;
 			ap.event.motion = ev;
 
-			pekwm::actionHandler()->handleAction(ap);
+			pekwm::actionHandler()->handleAction(&ap);
 		}
 	} else {
-		DockApp *da = pekwm::harbour()->findDockAppFromFrame(ev->window);
+		DockApp* da =
+			pekwm::harbour()->findDockAppFromFrame(ev->window);
 		if (da) {
 			pekwm::harbour()->handleMotionNotifyEvent(ev, da);
 		}
@@ -1185,7 +1228,8 @@ WindowManager::handleEnterNotify(XCrossingEvent *ev)
 	}
 
 	// Clear event queue
-	while (X11::checkTypedEvent(EnterNotify, (XEvent *) ev)) {
+	while (X11::checkTypedEvent(EnterNotify,
+				    reinterpret_cast<XEvent*>(ev))) {
 		if (! ev->send_event) {
 			X11::setLastEventTime(ev->time);
 		}
@@ -1211,7 +1255,7 @@ WindowManager::handleEnterNotify(XCrossingEvent *ev)
 			ap.type = ev->type;
 			ap.event.crossing = ev;
 
-			pekwm::actionHandler()->handleAction(ap);
+			pekwm::actionHandler()->handleAction(&ap);
 		}
 	}
 }
@@ -1220,7 +1264,8 @@ void
 WindowManager::handleLeaveNotify(XCrossingEvent *ev)
 {
 	// Clear event queue
-	while (X11::checkTypedEvent(LeaveNotify, (XEvent *) ev)) {
+	while (X11::checkTypedEvent(LeaveNotify,
+				    reinterpret_cast<XEvent*>(ev))) {
 		if (! ev->send_event) {
 			X11::setLastEventTime(ev->time);
 		}
@@ -1237,7 +1282,7 @@ WindowManager::handleLeaveNotify(XCrossingEvent *ev)
 			ActionPerformed ap(wo, *ae);
 			ap.type = ev->type;
 			ap.event.crossing = ev;
-			pekwm::actionHandler()->handleAction(ap);
+			pekwm::actionHandler()->handleAction(&ap);
 		}
 	}
 }
@@ -1253,59 +1298,65 @@ WindowManager::handleFocusInEvent(XFocusChangeEvent *ev)
 	}
 
 	PWinObj *wo = PWinObj::findPWinObj(ev->window);
-	if (wo) {
-		// To simplify logic, changing client in frame wouldn't update the
-		// focused window because wo != focused_wo woule be true.
-		if (wo->getType() == PWinObj::WO_FRAME) {
-			wo = static_cast<Frame*>(wo)->getActiveChild();
+	if (wo == nullptr) {
+		return;
+	}
+
+	// To simplify logic, changing client in frame wouldn't update the
+	// focused window because wo != focused_wo woule be true.
+	if (wo->getType() == PWinObj::WO_FRAME) {
+		wo = static_cast<Frame*>(wo)->getActiveChild();
+	}
+
+	if (! wo->isFocusable() || ! wo->isMapped()) {
+		Workspaces::findWOAndFocus(nullptr);
+
+	} else if (wo != PWinObj::getFocusedPWinObj()) {
+		// If it's a valid FocusIn event with accepatable target lets
+		// flush all EnterNotify and LeaveNotify as they can interfere
+		// with focusing if Sloppy or Follow like focus model is used.
+		XEvent e_flush;
+		while (X11::checkTypedEvent(EnterNotify, &e_flush)) {
+			if (! e_flush.xcrossing.send_event) {
+				X11::setLastEventTime(e_flush.xcrossing.time);
+			}
+		}
+		while (X11::checkTypedEvent(LeaveNotify, &e_flush)) {
+			if (! e_flush.xcrossing.send_event) {
+				X11::setLastEventTime(e_flush.xcrossing.time);
+			}
 		}
 
-		if (! wo->isFocusable() || ! wo->isMapped()) {
-			Workspaces::findWOAndFocus(nullptr);
+		PWinObj *focused_wo = PWinObj::getFocusedPWinObj();
 
-		} else if (wo != PWinObj::getFocusedPWinObj()) {
-			// If it's a valid FocusIn event with accepatable target lets flush
-			// all EnterNotify and LeaveNotify as they can interfere with
-			// focusing if Sloppy or Follow like focus model is used.
-			XEvent e_flush;
-			while (X11::checkTypedEvent(EnterNotify, &e_flush)) {
-				if (! e_flush.xcrossing.send_event) {
-					X11::setLastEventTime(e_flush.xcrossing.time);
-				}
-			}
-			while (X11::checkTypedEvent(LeaveNotify, &e_flush)) {
-				if (! e_flush.xcrossing.send_event) {
-					X11::setLastEventTime(e_flush.xcrossing.time);
-				}
-			}
-
-			PWinObj *focused_wo = PWinObj::getFocusedPWinObj(); // convenience
-
-			// unfocus last window
-			if (focused_wo) {
-				if (focused_wo->getType() == PWinObj::WO_CLIENT) {
-					focused_wo->getParent()->setFocused(false);
-				} else {
-					focused_wo->setFocused(false);
-				}
-				Workspaces::setLastFocused(Workspaces::getActive(), focused_wo);
-			}
-
-			PWinObj::setFocusedPWinObj(wo);
-
-			if (wo->getType() == PWinObj::WO_CLIENT) {
-				wo->getParent()->setFocused(true);
-				pekwm::rootWo()->setEwmhActiveWindow(wo->getWindow());
-
-				// update the MRU list (except for skip focus windows, see #297)
-				if (! static_cast<Client*>(wo)->isSkip(SKIP_FOCUS_TOGGLE)) {
-					Frame *frame = static_cast<Frame*>(wo->getParent());
-					Workspaces::addToMRUFront(frame);
-				}
+		// unfocus last window
+		if (focused_wo) {
+			if (focused_wo->getType() == PWinObj::WO_CLIENT) {
+				focused_wo->getParent()->setFocused(false);
 			} else {
-				wo->setFocused(true);
-				pekwm::rootWo()->setEwmhActiveWindow(None);
+				focused_wo->setFocused(false);
 			}
+			Workspaces::setLastFocused(Workspaces::getActive(),
+						   focused_wo);
+		}
+
+		PWinObj::setFocusedPWinObj(wo);
+
+		if (wo->getType() == PWinObj::WO_CLIENT) {
+			wo->getParent()->setFocused(true);
+			pekwm::rootWo()->setEwmhActiveWindow(wo->getWindow());
+
+			// update the MRU list (except for skip focus windows,
+			// see #297)
+			Client* client = static_cast<Client*>(wo);
+			if (! client->isSkip(SKIP_FOCUS_TOGGLE)) {
+				Frame *frame =
+					static_cast<Frame*>(wo->getParent());
+				Workspaces::addToMRUFront(frame);
+			}
+		} else {
+			wo->setFocused(true);
+			pekwm::rootWo()->setEwmhActiveWindow(None);
 		}
 	}
 }
@@ -1323,7 +1374,8 @@ WindowManager::handleClientMessageEvent(XClientMessageEvent *ev)
 		handlePekwmCmd(ev);
 	} if (ev->window == X11::getRoot()) {
 		if (ev->format == 32) {
-			if (ev->message_type == X11::getAtom(NET_CURRENT_DESKTOP)) {
+			if (ev->message_type
+			    == X11::getAtom(NET_CURRENT_DESKTOP)) {
 				Workspaces::setWorkspace(ev->data.l[0], true);
 			} else if (ev->message_type ==
 				   X11::getAtom(NET_NUMBER_OF_DESKTOPS)) {
@@ -1332,18 +1384,21 @@ WindowManager::handleClientMessageEvent(XClientMessageEvent *ev)
 				}
 			}
 		}
-	} else if (ev->message_type == X11::getAtom(NET_REQUEST_FRAME_EXTENTS)) {
+	} else if (ev->message_type
+		   == X11::getAtom(NET_REQUEST_FRAME_EXTENTS)) {
 		handleNetRequestFrameExtents(ev->window);
 	} else {
 		Client *client = Client::findClientFromWindow(ev->window);
 		if (client) {
-			Frame *frame = static_cast<Frame*>(client->getParent());
-			ActionEvent *ae = frame->handleClientMessage(ev, client);
+			Frame *frame =
+				static_cast<Frame*>(client->getParent());
+			ActionEvent *ae =
+				frame->handleClientMessage(ev, client);
 			if (ae) {
 				ActionPerformed ap(frame, *ae);
 				ap.type = ev->type;
 				ap.event.client = ev;
-				pekwm::actionHandler()->handleAction(ap);
+				pekwm::actionHandler()->handleAction(&ap);
 			}
 		}
 	}
@@ -1356,7 +1411,7 @@ WindowManager::handleNetRequestFrameExtents(Window win)
 	MwmThemeState mwm_state;
 	Client *client = Client::findClientFromWindow(win);
 	if (client == nullptr) {
-		MwmHints hints = {0};
+		MwmHints hints;
 		X11Util::readMwmHints(win, hints);
 		mwm_state.setHints(hints);
 		state = &mwm_state;
@@ -1373,14 +1428,15 @@ WindowManager::handleNetRequestFrameExtents(Window win)
 		Cardinal extents[4];
 		extents[0] = theme_gm.bdLeft(state);
 		extents[1] = theme_gm.bdRight(state);
-		extents[2] = theme_gm.bdTop(state) + theme_gm.titleHeight(state);
+		extents[2] = theme_gm.bdTop(state)
+			+ theme_gm.titleHeight(state);
 		extents[3] = theme_gm.bdBottom(state);
 
 		if (Debug::isLevel(Debug::LEVEL_TRACE)) {
 			std::ostringstream msg;
-			msg << "setting _NET_FRAME_EXTENTS (" << extents[0] << " ";
-			msg << extents[1] << " " << extents[2] << " " << extents[3];
-			msg << ") on " << win;
+			msg << "setting _NET_FRAME_EXTENTS (" << extents[0];
+			msg << " " << extents[1] << " " << extents[2] << " ";
+			msg << extents[3] << ") on " << win;
 			P_TRACE(msg.str());
 		}
 		X11::setCardinals(win, NET_FRAME_EXTENTS, extents, 4);
@@ -1392,8 +1448,8 @@ WindowManager::handleColormapEvent(XColormapEvent *ev)
 {
 	Client *client = Client::findClient(ev->window);
 	if (client) {
-		client =
-		  static_cast<Client*>(((Frame*) client->getParent())->getActiveChild());
+		Frame* frame = static_cast<Frame*>(client->getParent());
+		client = static_cast<Client*>(frame->getActiveChild());
 		client->handleColormapChange(ev);
 	}
 }
@@ -1403,12 +1459,17 @@ void
 WindowManager::handlePropertyEvent(XPropertyEvent *ev)
 {
 	if (ev->window == X11::getRoot()) {
-		return pekwm::rootWo()->handlePropertyChange(ev);
+		if (ev->atom == X11::getAtom(RESOURCE_MANAGER)) {
+			doReloadResources();
+		} else {
+			return pekwm::rootWo()->handlePropertyChange(ev);
+		}
 	}
 
 	Client *client = Client::findClientFromWindow(ev->window);
 	if (client) {
-		((Frame*) client->getParent())->handlePropertyChange(ev, client);
+		Frame* frame = static_cast<Frame*>(client->getParent());
+		frame->handlePropertyChange(ev, client);
 	}
 }
 
@@ -1439,7 +1500,7 @@ WindowManager::handleExposeEvent(XExposeEvent *ev)
 		ap.type = ev->type;
 		ap.event.expose = ev;
 
-		pekwm::actionHandler()->handleAction(ap);
+		pekwm::actionHandler()->handleAction(&ap);
 	}
 }
 
@@ -1448,65 +1509,60 @@ WindowManager::handleExposeEvent(XExposeEvent *ev)
 Client*
 WindowManager::createClient(Window window, bool is_new)
 {
-	Client *client = 0;
-	ClientInitConfig initConfig;
-
 	XWindowAttributes attr;
-	XGetWindowAttributes(X11::getDpy(), window, &attr);
-	if (! attr.override_redirect && (is_new || attr.map_state != IsUnmapped)) {
-		// We need to figure out whether or not this is a dockapp.
-		XWMHints *wm_hints = XGetWMHints(X11::getDpy(), window);
-		if (wm_hints) {
-			if ((wm_hints->flags&StateHint)
-			    && (wm_hints->initial_state == WithdrawnState)) {
-				pekwm::harbour()->addDockApp(new DockApp(window));
-			} else {
-				client = new Client(window, initConfig, is_new);
-			}
-			X11::free(wm_hints);
-		} else {
-			client = new Client(window, initConfig, is_new);
-		}
+	X11::getWindowAttributes(window, attr);
+	if (attr.override_redirect
+	    || (! is_new && attr.map_state == IsUnmapped)) {
+		return nullptr;
 	}
 
-	if (client) {
-		if (client->isAlive()) {
-			if (initConfig.parent_is_new) {
-				PWinObj *wo = client->getParent();
-				Workspaces::handleFullscreenBeforeRaise(wo);
-				Workspaces::insert(wo, true);
-			}
-			Workspaces::updateClientList();
+	XWMHints wm_hints;
+	if (X11::getWMHints(window, wm_hints)
+	    && (wm_hints.flags&StateHint)
+	    && (wm_hints.initial_state == WithdrawnState)) {
+		pekwm::harbour()->addDockApp(new DockApp(window));
+		return nullptr;
+	}
 
-			// Make sure the window is mapped, this is done after it has been
-			// added to the decor/frame as otherwise IsViewable state won't
-			// be correct and we don't know whether or not to place the window
-			if (initConfig.map) {
-				client->mapWindow();
-			}
+	ClientInitConfig initConfig;
+	Client *client = new Client(window, initConfig, is_new);
+	if (! client->isAlive()) {
+		delete client;
+		return nullptr;
+	}
 
-			// Focus was requested by the configuration, look out for
-			// focus stealing.
-			if (initConfig.focus) {
-				PWinObj *wo = PWinObj::getFocusedPWinObj();
-				Time time_protect =
-					static_cast<Time>(pekwm::config()->getFocusStealProtect());
+	if (initConfig.parent_is_new) {
+		PWinObj *wo = client->getParent();
+		Workspaces::handleFullscreenBeforeRaise(wo);
+		Workspaces::insert(wo, true);
+	}
+	Workspaces::updateClientList();
 
-				if (wo != nullptr
-				    && wo->isMapped()
-				    && wo->isKeyboardInput()
-				    && time_protect
-				    && time_protect >= (X11::getLastEventTime()
-							- wo->getLastActivity())) {
-					// WO exists, is mapped, and time protect is
-					// active and within it's limits.
-				} else {
-					client->getParent()->giveInputFocus();
-				}
-			}
-		} else{
-			delete client;
-			client = 0;
+	// Make sure the window is mapped, this is done after it has been
+	// added to the decor/frame as otherwise IsViewable state won't
+	// be correct and we don't know whether or not to place the window
+	if (initConfig.map) {
+		client->mapWindow();
+	}
+
+	// Focus was requested by the configuration, look out for
+	// focus stealing.
+	if (initConfig.focus) {
+		PWinObj *wo = PWinObj::getFocusedPWinObj();
+		Config *cfg = pekwm::config();
+		Time time_protect =
+			static_cast<Time>(cfg->getFocusStealProtect());
+
+		if (wo != nullptr
+		    && wo->isMapped()
+		    && wo->isKeyboardInput()
+		    && time_protect
+		    && time_protect >= (X11::getLastEventTime()
+					- wo->getLastActivity())) {
+			// WO exists, is mapped, and time protect is
+			// active and within it's limits.
+		} else {
+			client->getParent()->giveInputFocus();
 		}
 	}
 
@@ -1532,7 +1588,7 @@ WindowManager::handlePekwmCmd(XClientMessageEvent *ev)
 		}
 
 		ActionPerformed ap(wo, ae);
-		pekwm::actionHandler()->handleAction(ap);
+		pekwm::actionHandler()->handleAction(&ap);
 	}
 
 	_pekwm_cmd_buf = "";
@@ -1561,7 +1617,8 @@ WindowManager::recvPekwmCmd(XClientMessageEvent *ev)
 	case PEKWM_CMD_MULTI_CONT:
 	case PEKWM_CMD_MULTI_END:
 		if (_pekwm_cmd_buf.empty()) {
-			P_DBG("invalid _PEKWM_CMD, continuation on empty buffer");
+			P_DBG("invalid _PEKWM_CMD, continuation on empty "
+			      "buffer");
 			_pekwm_cmd_buf = "";
 			return false;
 		}
@@ -1577,7 +1634,8 @@ WindowManager::recvPekwmCmd(XClientMessageEvent *ev)
 		return op == PEKWM_CMD_MULTI_END;
 	default:
 		// invalid data
-		P_DBG("invalid _PEKMW_CMD, last byte " << op << " not in range 0-3");
+		P_DBG("invalid _PEKMW_CMD, last byte " << op
+		      << " not in range 0-3");
 		_pekwm_cmd_buf = "";
 		return false;
 	}
