@@ -1,5 +1,6 @@
 //
 // X11.cc for pekwm
+// Copyright (C) 2022-2023 Claes Nästén
 // Copyright (C) 2009-2021 the pekwm development team
 //
 // This program is licensed under the GNU GPL.
@@ -42,6 +43,8 @@ extern "C" {
 
 #include "X11.hh"
 #include "Debug.hh"
+#include "String.hh"
+#include "pekwm.hh"
 
 const uint X11::MODIFIER_TO_MASK[] = {
 	ShiftMask, LockMask, ControlMask,
@@ -79,7 +82,8 @@ extern "C" {
 		if (Debug::getLevel() >= Debug::LEVEL_TRACE) {
 			char error_buf[256];
 			XGetErrorText(dpy, ev->error_code, error_buf, 256);
-			P_TRACE("XError: " << error_buf << " id: " << ev->resourceid);
+			P_TRACE("XError: " << error_buf
+				<< " id: " << ev->resourceid);
 		}
 
 		return 0;
@@ -142,6 +146,7 @@ static const char *atomnames[] = {
 	"UTF8_STRING",
 	"STRING",
 	"MANAGER",
+	"RESOURCE_MANAGER",
 
 	// pekwm atoms
 	"_PEKWM_FRAME_ID",
@@ -173,6 +178,14 @@ static const char *atomnames[] = {
 
 	"_XROOTPMAP_ID",
 	"_XSETROOT_ID",
+
+	// xembed, systray
+	"_XEMBED",
+	"_XEMBED_INFO",
+	"_NET_SYSTEM_TRAY_OPCODE",
+	"_NET_SYSTEM_TRAY_MESSAGE_DATA",
+
+	nullptr
 };
 
 std::ostream&
@@ -212,6 +225,17 @@ Geometry::~Geometry(void)
 {
 }
 
+/**
+ * Update provided gm to be centered within this geometry.
+ */
+Geometry
+Geometry::center(Geometry gm) const
+{
+	gm.x = x + (width / 2) - (gm.width / 2);
+	gm.y = y + (height / 2) - (gm.height / 2);
+	return gm;
+}
+
 Geometry&
 Geometry::operator=(const Geometry& gm)
 {
@@ -255,6 +279,11 @@ Geometry::diffMask(const Geometry &old_gm)
 	return mask;
 }
 
+Strut::Strut(const long* s)
+{
+	*this = s;
+}
+
 Strut::Strut(long l, long r, long t, long b, int nhead)
 	: left(l),
 	  right(r),
@@ -268,12 +297,6 @@ Strut::~Strut(void)
 {
 }
 
-bool
-Strut::isSet(void) const
-{
-	return left != 0 || right != 0 || top != 0 || bottom != 0;
-}
-
 void
 Strut::clear(void)
 {
@@ -331,7 +354,7 @@ public:
 	inline void decRef(void) { if (_ref > 0) { _ref--; } }
 
 	inline bool operator==(const std::string &name) {
-		return (::strcasecmp(_name.c_str(), name.c_str()) == 0);
+		return pekwm::ascii_ncase_equal(_name, name);
 	}
 
 private:
@@ -378,7 +401,8 @@ X11::init(Display *dpy, bool synchronous, bool honour_randr)
 	_screen_gm.height = HeightOfScreen(ScreenOfDisplay(_dpy, _screen));
 
 	// create resize cursors
-	_cursor_map[CURSOR_TOP_LEFT] = XCreateFontCursor(_dpy, XC_top_left_corner);
+	_cursor_map[CURSOR_TOP_LEFT] =
+		XCreateFontCursor(_dpy, XC_top_left_corner);
 	_cursor_map[CURSOR_TOP] = XCreateFontCursor(_dpy, XC_top_side);
 	_cursor_map[CURSOR_TOP_RIGHT] =
 		XCreateFontCursor(_dpy, XC_top_right_corner);
@@ -426,7 +450,7 @@ X11::init(Display *dpy, bool synchronous, bool honour_randr)
 	initHeads();
 
 	// initialize array values
-	for (uint i = 0; i < (BUTTON_NO - 1); ++i) {
+	for (uint i = 0; i < BUTTON_NO; ++i) {
 		_last_click_time[i] = 0;
 	}
 
@@ -438,11 +462,14 @@ X11::init(Display *dpy, bool synchronous, bool honour_randr)
 	_xc_default.pixel = BlackPixel(_dpy, _screen);
 	_xc_default.red = _xc_default.green = _xc_default.blue = 0;
 
-	assert(sizeof(atomnames)/sizeof(char*) == MAX_NR_ATOMS);
+	assert(sizeof(atomnames)/sizeof(char*) == (MAX_NR_ATOMS + 1));
 	if (! XInternAtoms(_dpy, const_cast<char**>(atomnames), MAX_NR_ATOMS,
 			   0, _atoms)) {
 		P_ERR("XInternAtoms did not return all requested atoms");
 	}
+
+	XrmInitialize();
+	loadXrmResources();
 }
 
 //! @brief X11 destructor
@@ -467,6 +494,8 @@ X11::destruct(void) {
 		XFreeCursor(_dpy, _cursor_map[i]);
 	}
 
+	XrmDestroyDatabase(_xrm_db);
+
 	// Under certain circumstances trying to restart pekwm can cause it to
 	// use 100% of the CPU without making any progress with the restart.
 	// This X11:sync() seems to be work around the issue (c.f. #300).
@@ -476,10 +505,39 @@ X11::destruct(void) {
 	_dpy = 0;
 }
 
+/**
+ * Wrapper for XGetSelectionOwner.
+ */
+Window
+X11::getSelectionOwner(Atom atom)
+{
+	if (_dpy) {
+		return XGetSelectionOwner(_dpy, atom);
+	}
+	return None;
+}
+
+/**
+ * Wrapper for XSetSelectionOwner with default display and default timestamp
+ * from last registered timestamp.
+ *
+ * @param timestamp Use -1 for last event time (default)
+ */
+void
+X11::setSelectionOwner(Atom atom, Window owner, Time timestamp)
+{
+	if (_dpy) {
+		if (timestamp == static_cast<Time>(-1)) {
+			timestamp = _last_event_time;
+		}
+		XSetSelectionOwner(_dpy, atom, owner, timestamp);
+	}
+}
+
 XColor *
 X11::getColor(const std::string &color)
 {
-	if (strcasecmp(color.c_str(), "EMPTY") == 0) {
+	if (pekwm::ascii_ncase_equal(color.c_str(), "EMPTY")) {
 		return &_xc_default;
 	}
 
@@ -522,7 +580,8 @@ X11::returnColor(XColor*& xc)
 			(*it)->decRef();
 			if (((*it)->getRef() == 0)) {
 				ulong pixels[1] = { (*it)->getColor()->pixel };
-				XFreeColors(X11::getDpy(), X11::getColormap(), pixels, 1, 0);
+				XFreeColors(X11::getDpy(), X11::getColormap(),
+					    pixels, 1, 0);
 
 				delete *it;
 				_colors.erase(it);
@@ -569,7 +628,7 @@ X11::moveWindow(Window win, int x, int y)
 }
 void
 X11::resizeWindow(Window win,
-                  unsigned int width, unsigned int height)
+		  unsigned int width, unsigned int height)
 {
 	if (_dpy) {
 		XResizeWindow(_dpy, win, width, height);
@@ -578,7 +637,7 @@ X11::resizeWindow(Window win,
 
 void
 X11::moveResizeWindow(Window win, int x, int y,
-                      unsigned int width, unsigned int height)
+		      unsigned int width, unsigned int height)
 {
 	if (_dpy) {
 		XMoveResizeWindow(_dpy, win, x, y, width, height);
@@ -611,8 +670,10 @@ X11::stripButtonModifiers(unsigned int *state)
 void
 X11::setLockKeys(void)
 {
-	_num_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Num_Lock));
-	_scroll_lock = getMaskFromKeycode(XKeysymToKeycode(_dpy, XK_Scroll_Lock));
+	KeyCode kc_num = XKeysymToKeycode(_dpy, XK_Num_Lock);
+	KeyCode kc_scroll = XKeysymToKeycode(_dpy, XK_Scroll_Lock);
+	_num_lock = getMaskFromKeycode(kc_num);
+	_scroll_lock = getMaskFromKeycode(kc_scroll);
 }
 
 void
@@ -641,6 +702,12 @@ X11::pending(void)
 bool
 X11::getNextEvent(XEvent &ev, struct timeval *timeout)
 {
+	// A call to flush was previously used when no pending events was
+	// found, however accoarding to the XFlush man page XPending does
+	// flush by itself.
+	//
+	// This reportedly fixes lockups and the change was suggested
+	// by Christian Zander
 	if (pending()) {
 		XNextEvent(_dpy, &ev);
 		return true;
@@ -649,8 +716,6 @@ X11::getNextEvent(XEvent &ev, struct timeval *timeout)
 	int ret;
 	fd_set rfds;
 
-	flush();
-
 	FD_ZERO(&rfds);
 	FD_SET(_fd, &rfds);
 
@@ -688,16 +753,18 @@ X11::grabServer(void)
 bool
 X11::ungrabServer(bool sync)
 {
-	if (_server_grabs > 0) {
-		if (--_server_grabs == 0) {
-			if (sync) {
-				P_TRACE("0 server grabs left, syncing and ungrab.");
-				X11::sync(False);
-			} else {
-				P_TRACE("0 server grabs left, ungrabbing server.");
-			}
-			XUngrabServer(_dpy);
+	if (_server_grabs == 0) {
+		return true;
+	}
+
+	if (--_server_grabs == 0) {
+		if (sync) {
+			P_TRACE("0 server grabs left, syncing and ungrab.");
+			X11::sync(False);
+		} else {
+			P_TRACE("0 server grabs left, ungrabbing server.");
 		}
+		XUngrabServer(_dpy);
 	}
 	return _server_grabs == 0;
 }
@@ -730,7 +797,8 @@ X11::grabPointer(Window win, uint event_mask, CursorType type)
 {
 	P_TRACE("grabbing pointer");
 	Cursor cursor = type < CURSOR_NONE ? _cursor_map[type] : None;
-	if (XGrabPointer(_dpy, win, false, event_mask, GrabModeAsync, GrabModeAsync,
+	if (XGrabPointer(_dpy, win, false, event_mask,
+			 GrabModeAsync, GrabModeAsync,
 			 None, cursor, CurrentTime) == GrabSuccess) {
 		return true;
 	}
@@ -749,6 +817,19 @@ X11::ungrabPointer(void)
 	return false;
 }
 
+/**
+ * Translate coordinates relative to the root window getting the relative
+ * coordinates and matching window.
+ */
+Window
+X11::translateRootCoordinates(int x, int y, int *ret_x, int *ret_y)
+{
+	Window win = None;
+	XTranslateCoordinates(_dpy, _root, _root, x, y, ret_x, ret_y,
+			      &win);
+	return win;
+}
+
 /**
  * Refetches the root-window size.
  */
@@ -808,69 +889,118 @@ X11::getScreenChangeNotification(XEvent *ev, ScreenChangeNotification &scn)
 	return false;
 }
 
+void
+X11::setHonourRandr(bool honour_randr)
+{
+	if (_honour_randr != honour_randr) {
+		initHeads();
+	}
+	_honour_randr = honour_randr;
+
+}
+
 //! @brief Searches for the head closest to the coordinates x,y.
 //! @return The nearest head.  Head numbers are indexed from 0.
 uint
 X11::getNearestHead(int x, int y)
 {
-	if(_heads.size() > 1) {
-		// set distance to the highest uint value
-		uint min_distance = std::numeric_limits<uint>::max();
-		uint nearest_head = 0;
+	int nearest_head =
+		getNearestHead(x, y, DIRECTION_IGNORED, DIRECTION_IGNORED);
+	return nearest_head != -1 ? nearest_head : 0;
+}
+
+//! @brief Searches for the head closest to the coordinates x,y.
+//!
+//! If a direction is not DIRECTION_IGNORED, only nearest window in
+//! that direction is selected.
+//!
+//! @return The nearest head or -1 if not found
+int
+X11::getNearestHead(int x, int y, DirectionType dx, DirectionType dy)
+{
+	if (_heads.size() < 2) {
+		return -1;
+	}
+
+	// set distance to the highest uint value
+	uint min_distance = std::numeric_limits<uint>::max();
+	int nearest_head = -1;
+
+	for(uint head = 0; head < _heads.size(); ++head) {
+		int head_t = _heads[head].y;
+		int head_b = _heads[head].y + _heads[head].height;
+		int head_l = _heads[head].x;
+		int head_r = _heads[head].x + _heads[head].width;
+
+		int head_dx;
+		int head_dy;
+		if(y < head_t) {
+			head_dy = DIRECTION_DOWN;
+		} else if(y > head_b) {
+			head_dy = DIRECTION_UP;
+		} else {
+			head_dy = DIRECTION_NO;
+		}
 
 		uint distance;
-		int head_t, head_b, head_l, head_r;
-		for(uint head = 0; head < _heads.size(); ++head) {
-			head_t = _heads[head].y;
-			head_b = _heads[head].y + _heads[head].height;
-			head_l = _heads[head].x;
-			head_r = _heads[head].x + _heads[head].width;
-
-			if(x > head_r) {
-				if(y < head_t) {
-					// above and right of the head
-					distance = calcDistance(x, y, head_r, head_t);
-				} else if(y > head_b) {
-					// below and right of the head
-					distance = calcDistance(x, y, head_r, head_b);
-				} else {
-					// right of the head
-					distance = calcDistance(x, head_r);
-				}
-			} else if(x < head_l) {
-				if(y < head_t) {
-					// above and left of the head
-					distance = calcDistance(x, y, head_l, head_t);
-				} else if(y > head_b) {
-					// below and left of the head
-					distance = calcDistance(x, y, head_l, head_b);
-				} else {
-					// left of the head
-					distance = calcDistance(x, head_l);
-				}
+		if(x > head_r) {
+			head_dx = DIRECTION_LEFT;
+			if(y < head_t) {
+				// above and right of the head
+				distance = calcDistance(x, y, head_r, head_t);
+			} else if(y > head_b) {
+				// below and right of the head
+				distance = calcDistance(x, y, head_r, head_b);
 			} else {
-				if(y < head_t) {
-					// above the head
-					distance = calcDistance(y, head_t);
-				} else if(y > head_b) {
-					// below the head
-					distance = calcDistance(y, head_b);
-				} else {
-					// on the head
+				// right of the head
+				distance = calcDistance(x, head_r);
+			}
+		} else if(x < head_l) {
+			head_dx = DIRECTION_RIGHT;
+			if(y < head_t) {
+				// above and left of the head
+				distance =
+					calcDistance(x, y,
+						     head_l, head_t);
+			} else if(y > head_b) {
+				// below and left of the head
+				distance =
+					calcDistance(x, y,
+						     head_l, head_b);
+			} else {
+				// left of the head
+				distance = calcDistance(x, head_l);
+			}
+		} else {
+			head_dx = DIRECTION_NO;
+			if(y < head_t) {
+				// above the head
+				distance = calcDistance(y, head_t);
+			} else if(y > head_b) {
+				// below the head
+				distance = calcDistance(y, head_b);
+			} else {
+				// on the head
+				if(dx == DIRECTION_IGNORED
+				   && dy == DIRECTION_IGNORED) {
 					return head;
 				}
-			}
-
-
-			if(distance < min_distance) {
-				min_distance = distance;
-				nearest_head = head;
+				distance = 0;
 			}
 		}
-		return nearest_head;
-	} else {
-		return 0;
+
+		if((dx != DIRECTION_IGNORED && dx != head_dx) ||
+		   (dy != DIRECTION_IGNORED && dy != head_dy)) {
+			continue;
+		}
+
+		if(distance < min_distance) {
+			min_distance = distance;
+			nearest_head = head;
+		}
 	}
+
+	return nearest_head;
 }
 
 /**
@@ -952,18 +1082,59 @@ X11::getHeadGeometry(uint head)
 	return gm;
 }
 
+/**
+ * Find a head by name case-insensitively (xrandr only). If "name" is
+ * "primary" then return the primary head. Return -1 if not matching.
+ */
+int
+X11::findHeadByName(const std::string& name)
+{
+	bool find_primary = pekwm::ascii_ncase_equal(name, "PRIMARY");
+	for (uint i = 0; i < _heads.size(); i++) {
+		if (find_primary) {
+			if (_heads[i].primary) {
+				return i;
+			}
+		} else if (pekwm::ascii_ncase_equal(_heads[i].name,
+							    name)) {
+			return i;
+		}
+	}
+	return -1;
+}
+
 int
 X11::getNumHeads(void)
 {
 	return _heads.size();
 }
 
-const char*
-X11::getAtomString(AtomName name)
+AtomName
+X11::getAtomName(const std::string& name)
 {
-	return atomnames[name];
+	for (int i = 0; atomnames[i] != nullptr; ++i) {
+		if (name == atomnames[i]) {
+			return AtomName(i);
+		}
+	}
+	return MAX_NR_ATOMS;
 }
 
+/**
+ * Return Atom from provided string.
+ */
+Atom
+X11::getAtomId(const std::string& str)
+{
+	if (_dpy) {
+		return XInternAtom(_dpy, str.c_str(), False);
+	}
+	return 0;
+}
+
+/**
+ * Lookup string representation of AtomId.
+ */
 std::string
 X11::getAtomIdString(Atom id)
 {
@@ -997,15 +1168,16 @@ X11::setAtom(Window win, AtomName aname, AtomName value)
 void
 X11::setAtoms(Window win, AtomName aname, Atom *values, int size)
 {
-	changeProperty(win, _atoms[aname], XA_ATOM, 32,
-		       PropModeReplace, (uchar *) values, size);
+	changeProperty(win, _atoms[aname], XA_ATOM, 32, PropModeReplace,
+		       reinterpret_cast<uchar*>(values), size);
 }
 
 void
 X11::setEwmhAtomsSupport(Window win)
 {
 	changeProperty(win, _atoms[NET_SUPPORTED], XA_ATOM, 32,
-		       PropModeReplace, (uchar *) _atoms, UTF8_STRING+1);
+		       PropModeReplace,
+		       reinterpret_cast<uchar*>(_atoms), UTF8_STRING+1);
 }
 
 bool
@@ -1023,16 +1195,15 @@ X11::getWindow(Window win, AtomName aname, Window& value)
 void
 X11::setWindow(Window win, AtomName aname, Window value)
 {
-	changeProperty(win, _atoms[aname], XA_WINDOW, 32,
-		       PropModeReplace, (uchar *) &value, 1);
+	changeProperty(win, _atoms[aname], XA_WINDOW, 32, PropModeReplace,
+		       reinterpret_cast<uchar*>(&value), 1);
 }
 
 void
-X11::setWindows(Window win, AtomName aname, Window *values,
-                int size)
+X11::setWindows(Window win, AtomName aname, Window *values, int size)
 {
-	changeProperty(win, _atoms[aname], XA_WINDOW, 32,
-		       PropModeReplace, (uchar *) values, size);
+	changeProperty(win, _atoms[aname], XA_WINDOW, 32, PropModeReplace,
+		       reinterpret_cast<uchar*>(values), size);
 }
 
 bool
@@ -1049,7 +1220,7 @@ X11::getCardinal(Window win, AtomName aname, Cardinal &value, long format)
 
 void
 X11::setCardinals(Window win, AtomName aname,
-                  Cardinal *values, int num)
+		  Cardinal *values, int num)
 {
 	changeProperty(win, _atoms[aname], XA_CARDINAL, 32, PropModeReplace,
 		       reinterpret_cast<uchar*>(values), num);
@@ -1075,7 +1246,7 @@ X11::getUtf8StringId(Window win, Atom id, std::string &value)
 
 void
 X11::setUtf8String(Window win, AtomName aname,
-                   const std::string &value)
+		   const std::string &value)
 {
 	changeProperty(win, _atoms[aname], _atoms[UTF8_STRING], 8,
 		       PropModeReplace,
@@ -1085,7 +1256,7 @@ X11::setUtf8String(Window win, AtomName aname,
 
 void
 X11::setUtf8StringArray(Window win, AtomName aname,
-                        unsigned char *values, uint length)
+			unsigned char *values, uint length)
 {
 	changeProperty(win, _atoms[aname], _atoms[UTF8_STRING], 8,
 		       PropModeReplace, values, length);
@@ -1103,10 +1274,11 @@ X11::getStringId(Window win, Atom id, std::string &value)
 	uchar *data = 0;
 	long uint actual;
 	if (getProperty(win, id, XA_STRING, 0, &data, &actual)) {
-		value = std::string((const char*) data);
+		value = std::string(reinterpret_cast<const char*>(data));
 		while (value.size() < actual) {
 			value += ',';
-			value += ((const char*) data) + value.size();
+			value += reinterpret_cast<const char*>(data)
+				+ value.size();
 		}
 		X11::free(data);
 		return true;
@@ -1118,7 +1290,8 @@ void
 X11::setString(Window win, AtomName aname, const std::string &value)
 {
 	changeProperty(win, _atoms[aname], XA_STRING, 8, PropModeReplace,
-		       (uchar*)value.c_str(), value.size());
+		       reinterpret_cast<const uchar*>(value.c_str()),
+		       value.size());
 }
 
 bool
@@ -1141,8 +1314,12 @@ X11::listProperties(Window win, std::vector<Atom>& atoms)
 
 bool
 X11::getProperty(Window win, Atom atom, Atom type,
-                 ulong expected, uchar **data_ret, ulong *actual)
+		 ulong expected, uchar **data_ret, ulong *actual)
 {
+	if (! _dpy) {
+		return false;
+	}
+
 	if (expected == 0) {
 		expected = 1024;
 	}
@@ -1161,7 +1338,8 @@ X11::getProperty(Window win, Atom atom, Atom type,
 		status =
 			XGetWindowProperty(_dpy, win, atom,
 					   0L, expected, False, type,
-					   &r_type, &r_format, &read, &left, &data);
+					   &r_type, &r_format, &read, &left,
+					   &data);
 		if (status != Success || type != r_type || read == 0) {
 			if (data != nullptr) {
 				X11::free(data);
@@ -1237,7 +1415,8 @@ X11::getMousePosition(int &x, int &y)
 	int win_x, win_y;
 	uint mask;
 
-	XQueryPointer(_dpy, _root, &d_root, &d_win, &x, &y, &win_x, &win_y, &mask);
+	XQueryPointer(_dpy, _root, &d_root, &d_win, &x, &y,
+		      &win_x, &win_y, &mask);
 }
 
 uint
@@ -1261,7 +1440,7 @@ X11::getButtonFromState(uint state)
 
 int
 X11::sendEvent(Window dest, Window win, Atom atom, long mask,
-               long v1, long v2, long v3, long v4, long v5)
+	       long v1, long v2, long v3, long v4, long v5)
 {
 	XEvent e;
 	e.type = e.xclient.type = ClientMessage;
@@ -1298,7 +1477,7 @@ X11::setCardinal(Window win, AtomName aname, Cardinal value, long format)
 
 int
 X11::changeProperty(Window win, Atom prop, Atom type, int format,
-                    int mode, const unsigned char *data, int num_e)
+		    int mode, const unsigned char *data, int num_e)
 {
 	if (_dpy) {
 		return XChangeProperty(_dpy, win, prop, type, format, mode,
@@ -1320,15 +1499,29 @@ X11::getGeometry(Window win, unsigned *w, unsigned *h, unsigned *bw)
 	return BadImplementation;
 }
 
-bool
-X11::getWindowAttributes(Window win, XWindowAttributes *wa)
+Status
+X11::getWindowAttributes(Window win, XWindowAttributes &wa)
 {
 	if (_dpy) {
-		return XGetWindowAttributes(_dpy, win, wa);
+		return XGetWindowAttributes(_dpy, win, &wa);
 	}
 	return BadImplementation;
 }
 
+bool
+X11::getWMHints(Window win, XWMHints &hints)
+{
+	if (_dpy) {
+		XWMHints *hints_ptr = XGetWMHints(_dpy, win);
+		if (hints_ptr) {
+			hints = *hints_ptr;
+			X11::free(hints_ptr);
+			return true;
+		}
+	}
+	return false;
+}
+
 GC
 X11::createGC(Drawable d, ulong mask, XGCValues *values)
 {
@@ -1385,7 +1578,7 @@ X11::createImage(char *data, uint width, uint height)
 
 XImage*
 X11::getImage(Drawable src, int x, int y, uint width, uint height,
-              unsigned long plane_mask, int format)
+	      unsigned long plane_mask, int format)
 {
 	if (_dpy) {
 		return XGetImage(_dpy, src, x, y, width, height,
@@ -1396,8 +1589,8 @@ X11::getImage(Drawable src, int x, int y, uint width, uint height,
 
 void
 X11::putImage(Drawable dest, GC gc, XImage *ximage,
-              int src_x, int src_y, int dest_x, int dest_y,
-              uint width, uint height)
+	      int src_x, int src_y, int dest_x, int dest_y,
+	      uint width, uint height)
 {
 	if (_dpy) {
 		XPutImage(_dpy, dest, gc, ximage,
@@ -1413,6 +1606,17 @@ X11::destroyImage(XImage *ximage)
 	}
 }
 
+void
+X11::copyArea(Drawable src, Drawable dst, int src_x, int src_y,
+	      unsigned int width, unsigned int height,
+	      int dest_x, int dest_y)
+{
+	if (_dpy && width > 0 && height > 0) {
+		XCopyArea(_dpy, src, dst, getGC(),
+			  src_x, src_y, width, height, dest_x, dest_y);
+	}
+}
+
 void
 X11::setWindowBackground(Window window, ulong pixel)
 {
@@ -1437,14 +1641,6 @@ X11::clearWindow(Window window)
 	}
 }
 
-void
-X11::clearArea(Window window, int x, int y, uint width, uint height)
-{
-	if (_dpy) {
-		XClearArea(_dpy, window, x, y, width, height, False);
-	}
-}
-
 #ifdef PEKWM_HAVE_SHAPE
 void
 X11::shapeSelectInput(Window window, ulong mask)
@@ -1464,7 +1660,7 @@ X11::shapeQuery(Window dst, int *bshaped)
 
 void
 X11::shapeCombine(Window dst, int kind, int x, int y,
-                  Window src, int op)
+		  Window src, int op)
 {
 	XShapeCombineShape(_dpy, dst, kind, x, y, src, kind, op);
 }
@@ -1488,13 +1684,6 @@ X11::shapeSetMask(Window dst, int kind, Pixmap pix)
 {
 	XShapeCombineMask(_dpy, dst, kind, 0, 0, pix, ShapeSet);
 }
-
-XRectangle*
-X11::shapeGetRects(Window win, int kind, int *num)
-{
-	int ordering;
-	return XShapeGetRectangles(_dpy, win, kind, num, &ordering);
-}
 #else // ! PEKWM_HAVE_SHAPE
 void
 X11::shapeSelectInput(Window window, ulong mask)
@@ -1509,7 +1698,7 @@ X11::shapeQuery(Window dst, int *bshaped)
 
 void
 X11::shapeCombine(Window dst, int kind, int x, int y,
-                  Window src, int op)
+		  Window src, int op)
 {
 }
 
@@ -1527,13 +1716,6 @@ void
 X11::shapeSetMask(Window dst, int kind, Pixmap pix)
 {
 }
-
-XRectangle*
-X11::shapeGetRects(Window win, int kind, int *num)
-{
-	num = 0;
-	return nullptr;
-}
 #endif // PEKWM_HAVE_SHAPE
 
 //! @brief Initialize head information
@@ -1550,9 +1732,22 @@ X11::initHeads(void)
 		initHeadsXinerama();
 
 		if (! _heads.size()) {
-			addHead(Head(0, 0, _screen_gm.width, _screen_gm.height));
+			addHead(Head(0, 0,
+				     _screen_gm.width, _screen_gm.height));
 		}
 	}
+
+	// Fallback to head 0 as primary
+	bool has_primary = false;
+	for (uint i = 0; i < _heads.size(); i++) {
+		if (_heads[i].primary) {
+			has_primary = true;
+			break;
+		}
+	}
+	if (!has_primary) {
+		_heads[0].primary = true;
+	}
 }
 
 //! @brief Initialize head information from Xinerama
@@ -1591,12 +1786,19 @@ X11::initHeadsRandr(void)
 		return;
 	}
 
+	RROutput primary_output = XRRGetOutputPrimary(_dpy, _root);
+
 	for (int i = 0; i < resources->noutput; ++i) {
 		XRROutputInfo* output =
-			XRRGetOutputInfo(_dpy, resources, resources->outputs[i]);
+			XRRGetOutputInfo(_dpy, resources,
+					 resources->outputs[i]);
 		if (output->crtc) {
-			XRRCrtcInfo* crtc = XRRGetCrtcInfo(_dpy, resources, output->crtc);
-			addHead(Head(crtc->x, crtc->y, crtc->width, crtc->height));
+			XRRCrtcInfo* crtc =
+				XRRGetCrtcInfo(_dpy, resources, output->crtc);
+			addHead(Head(crtc->x, crtc->y,
+				     crtc->width, crtc->height,
+				     output->name,
+				     resources->outputs[i] == primary_output));
 			XRRFreeCrtcInfo (crtc);
 		}
 		XRRFreeOutputInfo (output);
@@ -1636,9 +1838,10 @@ X11::getMaskFromKeycode(KeyCode keycode)
 
 	// .h files states that modifiermap is an 8 * max_keypermod array.
 	int max_info = _modifier_map->max_keypermod * 8;
+	int mkpm = _modifier_map->max_keypermod;
 	for (int i = 0; i < max_info; ++i) {
 		if (_modifier_map->modifiermap[i] == keycode) {
-			return MODIFIER_TO_MASK[i / _modifier_map->max_keypermod];
+			return MODIFIER_TO_MASK[i / mkpm];
 		}
 	}
 
@@ -1659,10 +1862,11 @@ X11::getKeycodeFromMask(uint mask)
 		return 0;
 	}
 
+	const int mkpm = _modifier_map->max_keypermod;
 	for (int i = 0; i < 8; ++i) {
 		if (MODIFIER_TO_MASK[i] == mask) {
 			// FIXME: Is iteration over the range required?
-			return _modifier_map->modifiermap[i * _modifier_map->max_keypermod];
+			return _modifier_map->modifiermap[i * mkpm];
 		}
 	}
 
@@ -1732,7 +1936,8 @@ X11::parseGeometry(const std::string& str, Geometry& gm)
 		s_end = str.size();
 	} else {
 		// position
-		std::string::size_type y_start = str.find_first_of("+-", s_end + 1);
+		std::string::size_type y_start =
+			str.find_first_of("+-", s_end + 1);
 		if ((ret = parseGeometryVal(cstr + s_end + 1,
 					    cstr + y_start, gm.x)) > 0) {
 			mask |= X_VALUE;
@@ -1758,8 +1963,10 @@ X11::parseGeometry(const std::string& str, Geometry& gm)
 	if (s_end > s_start) {
 		// size
 		int width, height;
-		std::string::size_type h_start = str.find_first_of("xX", s_start);
-		if ((ret = parseGeometryVal(cstr + s_start, cstr + h_start, width)) > 0
+		std::string::size_type h_start =
+			str.find_first_of("xX", s_start);
+		if ((ret = parseGeometryVal(cstr + s_start,
+					    cstr + h_start, width)) > 0
 		    && width > 0) {
 			gm.width = width;
 			mask |= WIDTH_VALUE;
@@ -1814,10 +2021,10 @@ X11::keepVisible(Geometry &gm)
 
 Window
 X11::createWindow(Window parent,
-                  int x, int y, uint width, uint height,
-                  uint border_width, uint depth, uint _class,
-                  Visual* visual, ulong valuemask,
-                  XSetWindowAttributes* attrs)
+		  int x, int y, uint width, uint height,
+		  uint border_width, uint depth, uint _class,
+		  Visual* visual, ulong valuemask,
+		  XSetWindowAttributes* attrs)
 {
 	if (_dpy) {
 		return XCreateWindow(_dpy, parent,
@@ -1827,17 +2034,20 @@ X11::createWindow(Window parent,
 	return None;
 }
 
+/**
+ * Create window to be used by the window manager, having override redirect
+ * set to true.
+ */
 Window
-X11::createSimpleWindow(Window parent,
-                        int x, int y, uint width, uint height,
-                        uint border_width,
-                        ulong border, ulong background)
+X11::createWmWindow(Window parent, int x, int y, uint width, uint height,
+		    uint _class, ulong event_mask)
 {
-	if (_dpy) {
-		return XCreateSimpleWindow(_dpy, parent, x, y, width, height,
-					   border_width, border, background);
-	}
-	return None;
+	XSetWindowAttributes attr;
+	attr.event_mask = event_mask;
+	attr.override_redirect = True;
+	return createWindow(parent, x, y, width, height, 0,
+			    CopyFromParent, _class, CopyFromParent,
+			    CWEventMask|CWOverrideRedirect, &attr);
 }
 
 void
@@ -1850,7 +2060,7 @@ X11::destroyWindow(Window win)
 
 void
 X11::changeWindowAttributes(Window win, ulong mask,
-                            XSetWindowAttributes &attrs)
+			    XSetWindowAttributes &attrs)
 {
 	if (_dpy) {
 		XChangeWindowAttributes(_dpy, win, mask, &attrs);
@@ -1859,9 +2069,9 @@ X11::changeWindowAttributes(Window win, ulong mask,
 
 void
 X11::grabButton(unsigned b, unsigned int mod, Window win,
-                unsigned mask, int mode)
+		unsigned mask, int mode)
 {
-	XGrabButton(_dpy, b, mod, win, true, mask, mode,
+	XGrabButton(_dpy, b, mod, win, False, mask, mode,
 		    GrabModeAsync, None, None);
 }
 
@@ -1930,12 +2140,24 @@ X11::stackWindows(Window *wins, unsigned len)
 	}
 }
 
+bool
+X11::maskEvent(long event_mask, XEvent *ev)
+{
+	return XMaskEvent(_dpy, event_mask, ev);
+}
+
 bool
 X11::checkTypedEvent(int type, XEvent *ev)
 {
 	return XCheckTypedEvent(_dpy, type, ev);
 }
 
+bool
+X11::checkTypedWindowEvent(Window w, int type, XEvent *ev)
+{
+	return XCheckTypedWindowEvent(_dpy, w, type, ev);
+}
+
 void
 X11::sync(Bool discard)
 {
@@ -1959,6 +2181,69 @@ X11::setInputFocus(Window w)
 	XSetInputFocus(_dpy, w, RevertToPointerRoot, CurrentTime);
 }
 
+void
+X11::loadXrmResources()
+{
+	XrmDestroyDatabase(_xrm_db);
+	_xrm_db = nullptr;
+
+	// not using XResourceManagerString as the result is cached
+	// even if the resources on the display gets updated
+	std::string xrm;
+	if (getString(_root, RESOURCE_MANAGER, xrm)) {
+		_xrm_db = XrmGetStringDatabase(xrm.c_str());
+	}
+}
+
+/**
+ * Lookup String resource from REOURCE_MANAGER.
+ */
+bool
+X11::getXrmString(const std::string& name, std::string& val)
+{
+	if (_xrm_db) {
+		char *type;
+		XrmValue value;
+		if (XrmGetResource(_xrm_db, name.c_str(), "String",
+				   &type, &value)
+		    && value.size > 0
+		    && strcmp(type, "String") == 0) {
+			val = std::string(value.addr, value.size - 1);
+			return true;
+		}
+	}
+	return false;
+}
+
+/**
+ * Clear referenced resources from previous lookups.
+ */
+void
+X11::clearRefResources(void)
+{
+	_ref_resources.clear();
+}
+
+/**
+ * Get map of referenced resources and their values.
+ */
+const std::map<std::string, std::string>&
+X11::getRefResources(void)
+{
+	return _ref_resources;
+}
+
+/**
+ * Add resource -> value to registered color resources.
+ */
+void
+X11::registerRefResource(const std::string& res, const std::string& value)
+{
+	_ref_resources[res] = value;
+}
+
+const long XEMBED_VERSION = 0;
+
 Display *X11::_dpy;
 bool X11::_honour_randr = false;
 int X11::_fd = -1;
@@ -1982,7 +2267,10 @@ std::vector<Head> X11::_heads;
 uint X11::_server_grabs;
 Time X11::_last_event_time;
 Window X11::_last_click_id = None;
-Time X11::_last_click_time[BUTTON_NO - 1];
+Time X11::_last_click_time[BUTTON_NO];
 std::vector<X11::ColorEntry*> X11::_colors;
 XColor X11::_xc_default;
 Cursor X11::_cursor_map[CURSOR_NONE];
+XrmDatabase X11::_xrm_db = 0;
+std::map<std::string, std::string> X11::_ref_resources =
+	std::map<std::string, std::string>();
