@@ -1,25 +1,27 @@
 //
 // pekwm_dialog.cc for pekwm
-// Copyright (C) 2021 Claes Nästén <pekdon@gmail.com>
+// Copyright (C) 2021-2023 Claes Nästén <pekdon@gmail.com>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "pekwm.hh"
-
 #include "CfgParser.hh"
+#include "CfgUtil.hh"
 #include "Charset.hh"
 #include "Debug.hh"
 #include "FontHandler.hh"
 #include "ImageHandler.hh"
 #include "TextureHandler.hh"
 #include "Theme.hh"
+#include "PPixmapSurface.hh"
 #include "PWinObj.hh"
 #include "Render.hh"
 #include "Util.hh"
 #include "X11App.hh"
 #include "X11.hh"
+#include "pekwm.hh"
+#include "pekwm_env.hh"
 
 #include <algorithm>
 #include <iostream>
@@ -40,6 +42,7 @@ typedef void(*stop_fun)(int);
 static const uint WIDTH_DEFAULT = 250;
 static const uint HEIGHT_DEFAULT = 50;
 
+static std::string _config_script_path;
 static ObserverMapping *_observer_mapping = nullptr;
 static FontHandler* _font_handler = nullptr;
 static ImageHandler* _image_handler = nullptr;
@@ -47,7 +50,12 @@ static TextureHandler* _texture_handler = nullptr;
 
 namespace pekwm
 {
-	ObserverMapping* observerMapping(void)
+	const std::string& configScriptPath()
+	{
+		return _config_script_path;
+	}
+
+	ObserverMapping* observerMapping()
 	{
 		return _observer_mapping;
 	}
@@ -79,12 +87,14 @@ public:
 
 	int getX(void) const { return _gm.x; }
 	int getY(void) const { return _gm.y; }
+	uint getHeight(void) const { return _gm.height; }
+	void setHeight(uint height) { _gm.height = height; }
 
 	virtual bool setState(Window, ButtonState) {
 		return false;
 	}
 	virtual bool click(Window) { return false; }
-	virtual void render(Render &rend) = 0;
+	virtual void render(Render &rend, PSurface &surface) = 0;
 
 	virtual void place(int x, int y, uint width, uint)
 	{
@@ -117,6 +127,7 @@ protected:
 	Theme::DialogData *_data;
 	Window _window;
 	PWinObj &_parent;
+	/** Widget geometry relative to dialog window */
 	Geometry _gm;
 };
 
@@ -131,8 +142,7 @@ public:
 			return false;
 		}
 		_state = state;
-		X11Render rend(None);
-		render(rend);
+		render();
 		return true;
 	}
 
@@ -160,11 +170,16 @@ public:
 		return _font->getHeight() + _data->padHorz();
 	}
 
-	virtual void render(Render&) {
-		_data->getButton(_state)->render(_background, 0, 0,
+	virtual void render(Render&, PSurface&) {
+		render();
+	}
+
+private:
+	void render(void) {
+		_data->getButton(_state)->render(&_background, 0, 0,
 						 _gm.width, _gm.height);
 		_font->setColor(_data->getButtonColor());
-		_font->draw(_background,
+		_font->draw(&_background,
 			    _data->getPad(PAD_LEFT), _data->getPad(PAD_UP),
 			    _text);
 
@@ -177,26 +192,25 @@ private:
 	std::string _text;
 	PFont *_font;
 
-	Pixmap _background;
+	PPixmapSurface _background;
 	ButtonState _state;
 };
 
 Button::Button(Theme::DialogData* data, PWinObj& parent,
-               stop_fun stop, int retcode, const std::string& text)
+	       stop_fun stop, int retcode, const std::string& text)
 	: DialogWidget(data, parent),
 	  _stop(stop),
 	  _retcode(retcode),
 	  _text(text),
 	  _font(data->getButtonFont()),
-	  _background(None),
 	  _state(BUTTON_STATE_FOCUSED)
 {
 	_gm.width = widthReq();
 	_gm.height = heightReq(_gm.width);
-	_background = X11::createPixmap(_gm.width, _gm.height);
+	_background.resize(_gm.width, _gm.height);
 
 	XSetWindowAttributes attr;
-	attr.background_pixmap = _background;
+	attr.background_pixmap = _background.getDrawable();
 	attr.override_redirect = True;
 	attr.event_mask =
 		ButtonPressMask|ButtonReleaseMask|
@@ -213,7 +227,6 @@ Button::Button(Theme::DialogData* data, PWinObj& parent,
 
 Button::~Button(void)
 {
-	X11::freePixmap(_background);
 }
 
 class ButtonsRow : public DialogWidget
@@ -282,10 +295,10 @@ public:
 		return height + _data->padVert();
 	}
 
-	virtual void render(Render &rend) {
+	virtual void render(Render &rend, PSurface &surface) {
 		std::vector<Button*>::iterator it = _buttons.begin();
 		for (; it != _buttons.end(); ++it) {
-			(*it)->render(rend);
+			(*it)->render(rend, surface);
 		}
 	}
 
@@ -294,7 +307,7 @@ private:
 };
 
 ButtonsRow::ButtonsRow(Theme::DialogData* data, PWinObj& parent,
-                       stop_fun stop, std::vector<std::string> options)
+		       stop_fun stop, std::vector<std::string> options)
 	: DialogWidget(data, parent)
 {
 	int i = 0;
@@ -329,18 +342,20 @@ public:
 	virtual uint heightReq(uint width) const
 	{
 		if (_image->getWidth() > width) {
-			float aspect = float(_image->getWidth()) / _image->getHeight();
+			float aspect = float(_image->getWidth())
+					     / _image->getHeight();
 			return static_cast<uint>(width / aspect);
 		}
 		return _image->getHeight();
 	}
 
-	virtual void render(Render &rend)
+	virtual void render(Render &rend, PSurface&)
 	{
 		if (_image->getWidth() > _gm.width) {
-			float aspect = float(_image->getWidth()) / _image->getHeight();
-			_image->draw(rend,
-				     _gm.x, _gm.y, _gm.width, static_cast<uint>(_gm.width / aspect));
+			float aspect = float(_image->getWidth())
+					     / _image->getHeight();
+			_image->draw(rend, _gm.x, _gm.y, _gm.width,
+				     static_cast<uint>(_gm.width / aspect));
 		} else {
 			// render image centered on available width
 			uint x = (_gm.width - _image->getWidth()) / 2;
@@ -375,19 +390,19 @@ public:
 		return _font->getHeight() * num_lines + _data->padVert();
 	}
 
-	virtual void render(Render &rend)
+	virtual void render(Render &rend, PSurface &surface)
 	{
 		if (_lines.empty()) {
 			getLines(_gm.width, _lines);
 		}
 
-		_font->setColor(_is_title
-				? _data->getTitleColor() : _data->getTextColor());
+		_font->setColor(_is_title ? _data->getTitleColor()
+					  : _data->getTextColor());
 
 		uint y = _gm.y + _data->getPad(PAD_UP);
 		std::vector<std::string>::iterator line = _lines.begin();
 		for (; line != _lines.end(); ++line) {
-			_font->draw(rend.getDrawable(),
+			_font->draw(&surface,
 				    _gm.x + _data->getPad(PAD_LEFT), y, *line);
 			y += _font->getHeight();
 		}
@@ -411,7 +426,9 @@ private:
 				if (line == *word) {
 					lines.push_back(line);
 				} else {
-					line = line.substr(0, line.size() - word->size() - 1);
+					size_t len =
+						line.size() - word->size() - 1;
+					line.resize(len);
 					lines.push_back(line);
 					line = *word;
 				}
@@ -434,7 +451,7 @@ private:
 };
 
 Text::Text(Theme::DialogData* data, PWinObj& parent,
-           const std::string& text, bool is_title)
+	   const std::string& text, bool is_title)
 	: DialogWidget(data, parent),
 	  _font(is_title ? data->getTitleFont() : data->getTextFont()),
 	  _text(text),
@@ -471,7 +488,7 @@ public:
 		    const Geometry &gm,
 		    bool raise, const std::string& title, PImage* image,
 		    const std::string& message,
-		    std::vector<std::string> options);
+		    const std::vector<std::string>& options);
 	virtual ~PekwmDialog(void);
 
 	static PekwmDialog *instance(void) { return _instance; }
@@ -522,10 +539,10 @@ public:
 		std::vector<DialogWidget*>::iterator it = _widgets.begin();
 		for (; it != _widgets.end(); ++it) {
 			(*it)->place((*it)->getX(), y, width, height);
-			y += (*it)->heightReq(width);
+			(*it)->setHeight((*it)->heightReq(width));
+			y += (*it)->getHeight();
 		}
 
-		X11::freePixmap(_background);
 		render();
 	}
 
@@ -541,16 +558,17 @@ public:
 
 	void render(void)
 	{
-		if (_background == None) {
-			_background = X11::createPixmap(_gm.width, _gm.height);
-			X11::setWindowBackgroundPixmap(_window, _background);
+		if (_background.resize(_gm.width, _gm.height)) {
+			X11::setWindowBackgroundPixmap(
+					_window, _background.getDrawable());
 		}
-		X11Render rend(_background);
+		X11Render rend(_background.getDrawable());
+		RenderSurface surface(rend, _gm);
 		_data->getBackground()->render(rend,
 					       0, 0, _gm.width, _gm.height);
 		std::vector<DialogWidget*>::iterator it = _widgets.begin();
 		for (; it != _widgets.end(); ++it) {
-			(*it)->render(rend);
+			(*it)->render(rend, surface);
 		}
 		X11::clearWindow(_window);
 	}
@@ -578,7 +596,7 @@ public:
 protected:
 	void initWidgets(const std::string& title, PImage* image,
 			 const std::string& message,
-			 std::vector<std::string> options);
+			 const std::vector<std::string>& options);
 
 	void placeWidgets(void)
 	{
@@ -600,10 +618,12 @@ protected:
 		it = _widgets.begin();
 		for (; it != _widgets.end(); ++it) {
 			(*it)->place(0, y, width, 0);
-			y += (*it)->heightReq(width);
+			(*it)->setHeight((*it)->heightReq(width));
+			y += (*it)->getHeight();
 		}
 
-		PWinObj::resize(std::max(width, _gm.width), std::max(y, _gm.height));
+		PWinObj::resize(std::max(width, _gm.width),
+				std::max(y, _gm.height));
 	}
 
 private:
@@ -613,22 +633,21 @@ private:
 	Theme::DialogData* _data;
 	bool _raise;
 
-	Pixmap _background;
+	PPixmapSurface _background;
 	std::vector<DialogWidget*> _widgets;
 
 	static PekwmDialog *_instance;
 };
 
 PekwmDialog::PekwmDialog(Theme::DialogData* data,
-                         const Geometry &gm,
-                         bool raise, const std::string& title, PImage* image,
-                         const std::string& message,
-                         std::vector<std::string> options)
+			 const Geometry &gm,
+			 bool raise, const std::string& title, PImage* image,
+			 const std::string& message,
+			 const std::vector<std::string>& options)
 	: X11App(gm, title.empty() ? "pekwm_dialog" : title,
 		 "dialog", "pekwm_dialog", WINDOW_TYPE_NORMAL),
 	  _data(data),
-	  _raise(raise),
-	  _background(None)
+	  _raise(raise)
 {
 	assert(_instance == nullptr);
 	_instance = this;
@@ -643,14 +662,13 @@ PekwmDialog::~PekwmDialog(void)
 	for (; it != _widgets.end(); ++it) {
 		delete *it;
 	}
-	X11::freePixmap(_background);
 	_instance = nullptr;
 }
 
 void
 PekwmDialog::initWidgets(const std::string& title, PImage* image,
-                         const std::string& message,
-                         std::vector<std::string> options)
+			 const std::string& message,
+			 const std::vector<std::string>& options)
 {
 	if (title.size()) {
 		_widgets.push_back(new Text(_data, *this, title, true));
@@ -673,10 +691,13 @@ PekwmDialog::stopDialog(int retcode)
 
 PekwmDialog* PekwmDialog::_instance = nullptr;
 
-static void init(Display* dpy)
+static void init(Display* dpy,
+		 bool font_default_x11,
+		 const std::string &font_charset_override)
 {
 	_observer_mapping = new ObserverMapping();
-	_font_handler = new FontHandler();
+	_font_handler =
+		new FontHandler(font_default_x11, font_charset_override);
 	_image_handler = new ImageHandler();
 	_texture_handler = new TextureHandler();
 }
@@ -692,11 +713,14 @@ static void cleanup()
 static void usage(const char* name, int ret)
 {
 	std::cout << "usage: " << name;
-	std::cout << " [-dhitfl] [-o option|-o option...] message" << std::endl;
+	std::cout << " [-dhitfl] [-o option|-o option...] message"
+		  << std::endl;
 	std::cout << "  -d --display dpy    Display" << std::endl;
-	std::cout << "  -h --help           Display this information" << std::endl;
+	std::cout << "  -h --help           Display this information"
+		  << std::endl;
 	std::cout << "  -i --image          Image under title" << std::endl;
-	std::cout << "  -o --option         Option (many allowed)" << std::endl;
+	std::cout << "  -o --option         Option (many allowed)"
+		  << std::endl;
 	std::cout << "  -t --title          Dialog title" << std::endl;
 	std::cout << "  -f --log-file        Set log file." << std::endl;
 	std::cout << "  -l --log-level       Set log level." << std::endl;
@@ -716,19 +740,24 @@ int main(int argc, char* argv[])
 
 	static struct option opts[] = {
 		{const_cast<char*>("config"), required_argument, nullptr, 'c'},
-		{const_cast<char*>("display"), required_argument, nullptr, 'd'},
-		{const_cast<char*>("geometry"), required_argument, nullptr, 'g'},
+		{const_cast<char*>("display"), required_argument, nullptr,
+		 'd'},
+		{const_cast<char*>("geometry"), required_argument, nullptr,
+		 'g'},
 		{const_cast<char*>("help"), no_argument, nullptr, 'h'},
 		{const_cast<char*>("image"), required_argument, nullptr, 'i'},
 		{const_cast<char*>("option"), required_argument, nullptr, 'o'},
 		{const_cast<char*>("raise"), no_argument, nullptr, 'r'},
 		{const_cast<char*>("title"), required_argument, nullptr, 't'},
-		{const_cast<char*>("log-level"), required_argument, nullptr, 'l'},
-		{const_cast<char*>("log-file"), required_argument, nullptr, 'f'},
+		{const_cast<char*>("log-level"), required_argument, nullptr,
+		 'l'},
+		{const_cast<char*>("log-file"), required_argument, nullptr,
+		 'f'},
 		{nullptr, 0, nullptr, 0}
 	};
 
 	Charset::init();
+	initEnv(false);
 
 	int ch;
 	while ((ch = getopt_long(argc, argv, "c:d:g:hi:o:rt:",
@@ -761,7 +790,8 @@ int main(int argc, char* argv[])
 			break;
 		case 'f':
 			if (! Debug::setLogFile(optarg)) {
-				std::cerr << "Failed to open log file " << optarg << std::endl;
+				std::cerr << "Failed to open log file "
+					  << optarg << std::endl;
 			}
 			break;
 		case 'l':
@@ -774,7 +804,7 @@ int main(int argc, char* argv[])
 	}
 
 	if (config_file.empty()) {
-		config_file = "~/.pekwm/config";
+		config_file = Util::getConfigDir() + "/config";
 	}
 	Util::expandFileName(config_file);
 
@@ -800,8 +830,22 @@ int main(int argc, char* argv[])
 		return 1;
 	}
 
+	std::string theme_dir, theme_variant;
+	bool font_default_x11;
+	std::string font_charset_override;
+	{
+		CfgParser cfg(CfgParserOpt(""));
+		cfg.parse(config_file, CfgParserSource::SOURCE_FILE, true);
+		std::string theme_path;
+		CfgUtil::getThemeDir(cfg.getEntryRoot(),
+				     theme_dir, theme_variant, theme_path);
+		CfgUtil::getFontSettings(cfg.getEntryRoot(),
+					 font_default_x11,
+					 font_charset_override);
+	}
+
 	X11::init(dpy, true);
-	init(dpy);
+	init(dpy, font_default_x11, font_charset_override);
 
 	_image_handler->path_push_back("./");
 	PImage *image = nullptr;
@@ -812,14 +856,6 @@ int main(int argc, char* argv[])
 		}
 	}
 
-	std::string theme_dir, theme_variant, theme_path;
-	{
-		CfgParser cfg;
-		cfg.parse(config_file, CfgParserSource::SOURCE_FILE, true);
-		Util::getThemeDir(cfg.getEntryRoot(),
-				  theme_dir, theme_variant, theme_path);
-	}
-
 	int ret;
 	{
 		// run in separate scope to get Theme destructed before cleanup
