@@ -1,22 +1,29 @@
 //
 // test_pekwm.cc for pekwm
-// Copyright (C) 2021 Claes Nästén <pekdon@gmail.com>
+// Copyright (C) 2021-2023 Claes Nästén <pekdon@gmail.com>
 //
 // This program is licensed under the GNU GPL.
 // See the LICENSE file for more information.
 //
 
-#include "Compat.hh"
+#include "config.h"
 #include "test.hh"
+
+#include "Compat.hh"
 #include "Debug.hh"
 
 #include "test_Action.hh"
 #include "test_Config.hh"
+#include "test_FontHandler.hh"
 #include "test_Frame.hh"
 #include "test_InputDialog.hh"
 #include "test_ManagerWindows.hh"
 #include "test_Observable.hh"
 #include "test_PFont.hh"
+#ifdef PEKWM_HAVE_PANGO
+#include "test_PFontPango.hh"
+#endif // PEKWM_HAVE_PANGO
+#include "test_PFontXmb.hh"
 #include "test_Theme.hh"
 #include "test_WindowManager.hh"
 #include "test_X11.hh"
@@ -49,8 +56,15 @@ main_tests(int argc, char *argv[])
 	// Observable
 	TestObserverMapping testObserverMapping;
 
+	// FontHandler
+	TestFontHandler testFontHandler;
+
 	// PFont
 	TestPFont testPFont;
+#ifdef PEKWM_HAVE_PANGO
+	TestPFontPango testPFontPango;
+#endif // PEKWM_HAVE_PANGO
+	TestPFontXmb testPFontXmb;
 
 	// Theme
 	TestTheme testTheme;
@@ -59,6 +73,7 @@ main_tests(int argc, char *argv[])
 	TestWindowManager testWindowManager;
 
 	// x11
+	TestGeometry testGeometry;
 	TestX11 testX11;
 
 	return TestSuite::main(argc, argv);
